package com.hling.demo.AdsView

import android.Manifest
import android.annotation.SuppressLint
import android.content.Intent
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.provider.Settings
import android.util.Log
import android.view.KeyEvent
import android.view.ViewTreeObserver.OnGlobalLayoutListener
import android.view.Window
import android.view.WindowManager
import android.widget.FrameLayout
import android.widget.Toast
import androidx.annotation.RequiresApi
import com.halomobi.ssp.base.utils.LogUtils
import com.halomobi.ssp.bussiness.module_sdk.base.BaseAdActivity
import com.halomobi.ssp.bussiness.module_sdk.manager.HlSplashAd
import com.halomobi.ssp.bussiness.module_sdk.managerlistener.HlSplashListener
import com.hling.demo.Config.AppConfig
import com.hling.demo.MainActivity
import com.zjkj.gtm.R

class SplashActivity : BaseAdActivity() {
    private var hlSplashAd: HlSplashAd? = null
    private var splashView: FrameLayout? = null

    private var canJump = false


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_splash)
        splashView = findViewById(R.id.splashview)
        loadAD()
    }


    private fun loadAD() {
        hlSplashAd = HlSplashAd(this, AppConfig.SPLASH_ID, splashView, hlSplashListener)
        hlSplashAd?.loadAd()
    }

    private val hlSplashListener: HlSplashListener = object : HlSplashListener {
        override fun onDisplayAd() {
            LogUtils.d("HaloSDKDemo:Splash::onDisplay");
        }

        override fun onClickAd() {
            LogUtils.d("HaloSDKDemo:Splash::onClick");
        }

        override fun onCloseAd() {
            LogUtils.d("HaloSDKDemo:Splash::onClose");
            jumpDismiss()
        }

        override fun onAdLoadSuccess() {
            if (hlSplashAd != null) {
                LogUtils.d("HaloSDKDemo:Splash::onAdLoadSuccess：" + "ecpm:" + hlSplashAd?.ecpm);
                hlSplashAd?.showAd()
            } else {
                LogUtils.d("HaloSDKDemo:Splash::onAdError");
                jump() //不可展示广告
            }
        }

        override fun onAdFailed(msg: String?, code: Int) {
            LogUtils.d("HaloSDKDemo:Splash::onAdFailedMsg:" + msg + "onAdFailedCode:" + code);
            jump()
        }
    }

    @SuppressLint("UseKtx")
    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode != 1024 || !hasAllPermissionsGranted(grantResults)) {
            Toast.makeText(
                this,
                "应用缺少必要的权限！请点击\"权限\"，打开所需要的权限。",
                Toast.LENGTH_LONG
            ).show()
            val intent = Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
            intent.setData(Uri.parse("package:$packageName"))
            startActivity(intent)
        }
    }


    private fun hasAllPermissionsGranted(grantResults: IntArray): Boolean {
        for (grantResult in grantResults) {
            if (grantResult == PackageManager.PERMISSION_DENIED) {
                return false
            }
        }
        return true
    }

    @RequiresApi(Build.VERSION_CODES.M)
    private fun checkAndRequestPermission() {
        val lackedPermission: MutableList<String> = ArrayList()
        if (checkSelfPermission(Manifest.permission.READ_PHONE_STATE) != PackageManager.PERMISSION_GRANTED) {
            lackedPermission.add(Manifest.permission.READ_PHONE_STATE)
        }
        if (checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            lackedPermission.add(Manifest.permission.ACCESS_FINE_LOCATION)
        }

        // 如果需要的权限都已经有了，那么直接调用SDK
        if (lackedPermission.isEmpty()) {
            loadAD()
        } else {
            // 否则，建议请求所缺少的权限，在onRequestPermissionsResult中再看是否获得权限
            val requestPermissions: Array<String> = lackedPermission.toTypedArray()
            requestPermissions(requestPermissions, 1024)
        }
    }


    override fun onPause() {
        LogUtils.d("HaloSDKDemo:onPause:$canJump");
        super.onPause()
        canJump = false

    }

    //点击开屏还需要在onResume中调用jumpDismiss接口。
    override fun onResume() {
        super.onResume()
        LogUtils.d("HaloSDKDemo:onResume:$canJump");
        if (canJump) {
            jumpDismiss()
        }
        canJump = true
    }


    //当设置开屏可点击时，需要等待跳转页面关闭后，再切换至您的主窗口 故此时需要增加canJump判断
    private fun jumpDismiss() {
        LogUtils.d("HaloSDKDemo:jumpDismiss:$canJump");
        if (canJump) {
            val intent = Intent(this, MainActivity::class.java)
            this.startActivity(intent)
            this.finish()
        } else {
            canJump = true
        }
    }


    /**
     * 不可点击的开屏，使用该jump方法，而不是用jumpDismiss
     */
    private fun jump() {
        this.startActivity(Intent(this@SplashActivity, MainActivity::class.java))
        this.finish()
    }


    /**
     * 开屏页一定要禁止用户对返回按钮的控制，否则将可能导致用户手动退出了App而广告无法正常曝光和计费
     */
    override fun onKeyDown(keyCode: Int, event: KeyEvent): Boolean {
        if (keyCode == KeyEvent.KEYCODE_BACK || keyCode == KeyEvent.KEYCODE_HOME) {
            return true
        }
        return super.onKeyDown(keyCode, event)
    }


    override fun onDestroy() {
        super.onDestroy()
        if (hlSplashAd != null) {
            hlSplashAd?.release()
        }
        if (splashView != null) {
            splashView?.removeAllViews()
            splashView = null
        }
    }


}
