package com.halomobi.ssp.base.common

import android.content.Context
import android.content.pm.PackageManager
import android.content.res.Configuration
import android.text.TextUtils
import com.halomobi.ssp.base.utils.CuidUtils
import com.halomobi.ssp.base.utils.SystemUtils
import com.halomobi.ssp.base.utils.Utils.context
import com.halomobi.ssp.model.request.config.SDKInfo
import java.util.UUID

object ConfigPool {
    /**
     * 广告请求超时时间
     */
    const val AD_REQUEST_TIME_OUT: Long = (3 * 1000).toLong()

    /**
     * 广告渲染超时时间
     */
    const val AD_RENDER_TIME_OUT: Long = (2 * 1000).toLong()

    /**
     * 检测View遮挡时间
     */
    const val CHECK_VIEW_TIME: Long = (3 * 1000).toLong()

    /**
     * Banner 轮播间隔
     */
    var BANNER_REQUEST_INTERVAL: Long = (30 * 1000).toLong()

    /**
     * 悬浮按钮 轮播间隔
     */
    var FLOAT_REQUEST_INTERVAL: Long = (30 * 1000).toLong()


    /**
     * 操作系统
     */
    const val OS_TYPE: Int = 2

    /**
     * SDK宿主APP版本
     */
    private var mAppVersion: String? = null

    /**
     * SDK宿主APP包名
     */
    private var mAppPackageName = ""

    /**
     * SDK宿主APP名字
     */
    private var mAppName: String? = null

    /**
     * 设备类型
     */
    private var mDeviceType = 0

    /**
     * 操作系统 版本
     */
    private var mOsVersion: String? = null

    /**
     * 手机品牌
     */
    private var mVendor: String? = null

    /**
     * 手机型号
     */
    private var mDeviceModel: String? = null

    /**
     * 安卓ID
     */
    private val mAndroidID: String? = null

    /**
     * 手机IMEI串号
     */
    private val mImei: String? = null

    /**
     * 手机IMEI串号的MD5
     */
    private val mImeiMD5: String? = null

    /**
     * 手机IMSI
     */
    private val mImsi: String? = null

    /**
     * mac地址
     */
    private val mMacAddress: String? = null

    /**
     * 系统语言
     */
    private var mLanguage: String? = null

    /**
     * 手机分辨率
     */
    private var mResolution: String? = null

    // Device
    //以下为新加参数
    /**
     * 是否root
     */
    private var mIsRoot = -1

    /**
     * 蓝牙mac
     */
    private var mBluetoothMac: String? = null

    /**
     * pseudo-unique ID
     */
    private var mSerial: String? = null

    /**
     * cpu类型
     */
    private var mCpuModel: String? = null

    /**
     * CPU序列号
     */
    private var mCpuSerial: String? = null

    /**
     * SDK 名字
     */
    var SDK_NAME: String = SDKInfo.SDK

    /**
     * SDK渠道信息
     */
    var SDK_CHANNEL: String = SDKInfo.CHANNEL_ID

    //公参
    /**
     * 设备标识
     */
    private var mDevice: String? = null

    /**
     * 用户标识
     */
    private var mCuid: String? = null

    /**
     * 终端ID
     */
    private var mClientID: String? = null

    /**
     * 设备ID
     */
    private var mDeviceID: String? = null

    /**
     * Android系统版本对应码
     */
    private var mOsVersionInt: String? = null
    private var mDpi = -1f
    private var mDeviceDpi = -1.0

    /**
     * 广告类型name
     */
    @JvmField
    var AdShowTypeName: String = ""


    //公参
    /**
     * SDK宿主APPID
     */
    @JvmField
    var mAppId: String = ""

    /**
     * SDK版本
     */
    var SDK_VERSION: String = SDKInfo.SDK_VERSION

    /**
     * 获取网络当前网络环境（测试、线上等）
     *
     * @return 当前网络环境
     */
    //当前网络环境（测试、线上等）
    var netEnvironment: NetEnvironment = NetEnvironment.ONLINE

    @JvmStatic
    val appVersion: String?
        /**
         * SDK宿主APP版本
         */
        get() {
            if (TextUtils.isEmpty(mAppVersion)) {
                mAppVersion = SystemUtils.appVer
            }
            return mAppVersion
        }

    @JvmStatic
    val appPackageName: String
        /**
         * SDK宿主APP包名
         */
        get() {
            if (TextUtils.isEmpty(mAppPackageName) && context != null) {
                mAppPackageName =
                    context?.packageName.toString()
            }
            return mAppPackageName
        }

    @JvmStatic
    fun getAppName(packageName: String?): String? {
        if (TextUtils.isEmpty(mAppName)) {
            val pm = context?.packageManager
            try {
                val info = pm?.getPackageInfo(packageName.toString(), PackageManager.GET_ACTIVITIES)
                mAppName = info?.applicationInfo?.loadLabel(pm).toString()
            } catch (e: PackageManager.NameNotFoundException) {
                e.printStackTrace()
            }
        }
        return mAppName
    }

    @JvmStatic
    val deviceType: Int
        /**
         * 设备类型
         */
        get() {
            if (mDeviceType == 0) {
                mDeviceType = SystemUtils.deviceType
            }
            return mDeviceType
        }

    @JvmStatic
    val oSVersion: String?
        /**
         * 操作系统 版本
         */
        get() {
            if (TextUtils.isEmpty(mOsVersion)) {
                mOsVersion = SystemUtils.oSVersion
            }
            return mOsVersion
        }

    @JvmStatic
    val vendor: String?
        /**
         * 手机品牌
         */
        get() {
            if (TextUtils.isEmpty(mVendor)) {
                mVendor = SystemUtils.vendor
            }
            return mVendor
        }

    @JvmStatic
    val deviceModel: String?
        /**
         * 手机型号
         */
        get() {
            if (TextUtils.isEmpty(mDeviceModel)) {
                mDeviceModel = SystemUtils.deviceModel
            }
            return mDeviceModel
        }

    val androidID: String
        /**
         * 安卓ID
         */
        get() = SystemUtils.getInfoByJson("dpid")

    val iMEI: String
        /**
         * 手机imei串号
         */
        get() = SystemUtils.getInfoByJson("did")

    val macAddress: String
        /**
         * mac地址
         */
        get() = SystemUtils.getInfoByJson("mac")

    val device: String?
        /**
         * 设备标识
         */
        get() {
            if (TextUtils.isEmpty(mDevice)) {
                mDevice = SystemUtils.device
            }
            return mDevice
        }

    val cuid: String?
        /**
         * 用户标识
         */
        get() {
            if (TextUtils.isEmpty(mCuid)) {
                mCuid = CuidUtils.cuid
            }
            return mCuid
        }

    val clientID: String?
        /**
         * 终端ID
         */
        get() {
            if (TextUtils.isEmpty(mClientID)) {
                mClientID = SystemUtils.clientID
            }
            return mClientID
        }

    val deviceID: String?
        /**
         * 设备ID
         */
        get() {
            if (TextUtils.isEmpty(mDeviceID)) {
                mDeviceID = SystemUtils.deviceID
            }
            return mDeviceID
        }

    val osVersionInt: String?
        /**
         * Android系统版本对应码
         */
        get() {
            if (TextUtils.isEmpty(mOsVersionInt)) {
                mOsVersionInt = SystemUtils.sDK_INT.toString() + ""
            }
            return mOsVersionInt
        }

    val dpi: Float
        get() {
            if (mDpi == -1f) {
                mDpi = SystemUtils.dPI
            }
            return mDpi
        }

    val deviceDpi: Double
        get() {
            if (mDeviceDpi == -1.0) {
                mDeviceDpi = SystemUtils.deviceDPI
            }
            return mDeviceDpi
        }

    val language: String?
        /**
         * 系统语言
         */
        get() {
            if (TextUtils.isEmpty(mLanguage)) {
                mLanguage = SystemUtils.language
            }
            return mLanguage
        }

    val resolution: String?
        /**
         * 手机分辨率
         */
        get() {
            if (TextUtils.isEmpty(mResolution)) {
                mResolution = SystemUtils.resolution
            }
            return mResolution
        }

    val isRoot: Int
        /**
         * 是否root
         */
        get() {
            if (mIsRoot == -1) {
                mIsRoot = if (SystemUtils.isRoot) 1 else 0
            }
            return mIsRoot
        }

    val bluetoothMac: String?
        /**
         * 蓝牙mac
         */
        get() {
            if (TextUtils.isEmpty(mBluetoothMac)) {
                mBluetoothMac = SystemUtils.bluetoothMac
            }
            return mBluetoothMac
        }

    val serial: String?
        /**
         * pseudo-unique ID
         */
        get() {
            if (TextUtils.isEmpty(mSerial)) {
                mSerial = SystemUtils.serial
            }
            return mSerial
        }

    val cpuModel: String?
        /**
         * 获取CPU类型
         */
        get() {
            if (TextUtils.isEmpty(mCpuModel)) {
                mCpuModel = SystemUtils.cpuModel
            }
            return mCpuModel
        }

    val cPUSerial: String?
        /**
         * CPU序列号
         */
        get() {
            if (TextUtils.isEmpty(mCpuSerial)) {
                mCpuSerial = SystemUtils.cPUSerial
            }
            return mCpuSerial
        }

    @JvmStatic
    val uUID: String
        /**
         * 随机唯一id
         */
        get() = UUID.randomUUID().toString().replace("-", "")

    /**
     * 初始化
     * .
     */
    fun init(appID: String) {
        mAppId = appID
        SystemUtils.initUserAgent() //获取系统浏览器UA
    }

    /**
     * 设置SDK版本号
     *
     * @param version 最新版本号
     */
    fun setSdkVersion(version: String) {
        SDK_VERSION = version
    }

    fun isTabletDevice(context: Context): Boolean {
        return (context.resources.configuration.screenLayout and Configuration.SCREENLAYOUT_SIZE_MASK) >=
                Configuration.SCREENLAYOUT_SIZE_LARGE
    }
}
