package com.halomobi.ssp.base.utils

import java.io.UnsupportedEncodingException

/**
 * This class implements Base64 encoding/decoding functionality
 * as specified in RFC 2045 (http://www.ietf.org/rfc/rfc2045.txt).
 */
object Base64Utils {
    /**
     * BASE 64 MAP.
     */
    private val MAP = byteArrayOf(
        'A'.toByte(),
        'B'.toByte(),
        'C'.toByte(),
        'D'.toByte(),
        'E'.toByte(),
        'F'.toByte(),
        'G'.toByte(),
        'H'.toByte(),
        'I'.toByte(),
        'J'.toByte(),
        'K'.toByte(),
        'L'.toByte(),
        'M'.toByte(),
        'N'.toByte(),
        'O'.toByte(),
        'P'.toByte(),
        'Q'.toByte(),
        'R'.toByte(),
        'S'.toByte(),
        'T'.toByte(),
        'U'.toByte(),
        'V'.toByte(),
        'W'.toByte(),
        'X'.toByte(),
        'Y'.toByte(),
        'Z'.toByte(),
        'a'.toByte(),
        'b'.toByte(),
        'c'.toByte(),
        'd'.toByte(),
        'e'.toByte(),
        'f'.toByte(),
        'g'.toByte(),
        'h'.toByte(),
        'i'.toByte(),
        'j'.toByte(),
        'k'.toByte(),
        'l'.toByte(),
        'm'.toByte(),
        'n'.toByte(),
        'o'.toByte(),
        'p'.toByte(),
        'q'.toByte(),
        'r'.toByte(),
        's'.toByte(),
        't'.toByte(),
        'u'.toByte(),
        'v'.toByte(),
        'w'.toByte(),
        'x'.toByte(),
        'y'.toByte(),
        'z'.toByte(),
        '0'.toByte(),
        '1'.toByte(),
        '2'.toByte(),
        '3'.toByte(),
        '4'.toByte(),
        '5'.toByte(),
        '6'.toByte(),
        '7'.toByte(),
        '8'.toByte(),
        '9'.toByte(),
        '+'.toByte(),
        '/'.toByte()
    )

    /**
     * 对base64编码数据进行解码。
     *
     * @param in  base64编码输入数据
     * @param len 长度
     * @return 解码后的数据
     */
    /**
     * 对输入进行base64编码。
     *
     * @param in 输入
     * @return base64编码字符串。
     */
    @JvmOverloads
    fun decode(`in`: ByteArray, len: Int = `in`.size): ByteArray? {
        // approximate output length
        var len = len
        val length = len / 4 * 3 // SUPPRESS CHECKSTYLE
        // return an empty array on emtpy or short input without padding
        if (length == 0) {
            return ByteArray(0)
        }
        // temporary array
        val out = ByteArray(length)
        // number of padding characters ('=')
        var pad = 0
        var chr: Byte
        // compute the number of the padding characters
        // and adjust the length of the input
        while (true) {
            chr = `in`[len - 1]
            // skip the neutral characters
            if ((chr == '\n'.toByte()) || (chr == '\r'.toByte()) || (chr == ' '.toByte()) || (chr == '\t'.toByte())) {
                len--
                continue
            }
            if (chr == '='.toByte()) {
                pad++
            } else {
                break
            }
            len--
        }
        // index in the output array
        var outIndex = 0
        // index in the input array
        var inIndex = 0
        // holds the value of the input character
        var bits = 0
        // holds the value of the input quantum
        var quantum = 0
        for (i in 0 until len) {
            chr = `in`[i]
            // skip the neutral characters
            if ((chr == '\n'.toByte()) || (chr == '\r'.toByte()) || (chr == ' '.toByte()) || (chr == '\t'.toByte())) {
                continue
            }
            bits = if ((chr >= 'A'.toByte()) && (chr <= 'Z'.toByte())) {
                // char ASCII value
                //  A    65    0
                //  Z    90    25 (ASCII - 65)
                chr - 65 // SUPPRESS CHECKSTYLE
            } else if ((chr >= 'a'.toByte()) && (chr <= 'z'.toByte())) {
                // char ASCII value
                //  a    97    26
                //  z    122   51 (ASCII - 71)
                chr - 71 // SUPPRESS CHECKSTYLE
            } else if ((chr >= '0'.toByte()) && (chr <= '9'.toByte())) {
                // char ASCII value
                //  0    48    52
                //  9    57    61 (ASCII + 4)
                chr + 4 // SUPPRESS CHECKSTYLE
            } else if (chr == '+'.toByte()) {
                62 // SUPPRESS CHECKSTYLE
            } else if (chr == '/'.toByte()) {
                63 // SUPPRESS CHECKSTYLE
            } else {
                return null
            }
            // append the value to the quantum
            quantum = (quantum shl 6) or bits.toByte().toInt() // SUPPRESS CHECKSTYLE
            if (inIndex % 4 == 3) { // SUPPRESS CHECKSTYLE
                // 4 characters were read, so make the output:
                out[outIndex++] = ((quantum and 0x00FF0000) shr 16).toByte() // SUPPRESS CHECKSTYLE
                out[outIndex++] = ((quantum and 0x0000FF00) shr 8).toByte() // SUPPRESS CHECKSTYLE
                out[outIndex++] = (quantum and 0x000000FF).toByte() // SUPPRESS CHECKSTYLE
            }
            inIndex++
        }
        if (pad > 0) {
            // adjust the quantum value according to the padding
            quantum = quantum shl (6 * pad) // SUPPRESS CHECKSTYLE
            // make output
            out[outIndex++] = ((quantum and 0x00FF0000) shr 16).toByte() // SUPPRESS CHECKSTYLE
            if (pad == 1) {
                out[outIndex++] = ((quantum and 0x0000FF00) shr 8).toByte() // SUPPRESS CHECKSTYLE
            }
        }
        // create the resulting array
        val result = ByteArray(outIndex)
        System.arraycopy(out, 0, result, 0, outIndex)
        return result
    }

    /**
     * base64编码。
     *
     * @param in          原始输入
     * @param charsetName 字符串集
     * @return base64编码后的字符串。
     */
    fun encode(`in`: ByteArray, charsetName: String? = null): String {
        var length = `in`.size * 4 / 3 // SUPPRESS CHECKSTYLE
        length += length / 76 + 3 // SUPPRESS CHECKSTYLE // for crlr 
        val out = ByteArray(length)
        var index = 0
        var crlr = 0
        val end = `in`.size - `in`.size % 3 // SUPPRESS CHECKSTYLE
        var i = 0
        while (i < end) {
            // SUPPRESS CHECKSTYLE
            out[index++] = MAP[(`in`[i].toInt() and 0xff) shr 2] // SUPPRESS CHECKSTYLE
            out[index++] = MAP[((`in`[i].toInt() and 0x03) shl 4) // SUPPRESS CHECKSTYLE
                    or ((`in`[i + 1].toInt() and 0xff) shr 4)] // SUPPRESS CHECKSTYLE
            out[index++] = MAP[((`in`[i + 1].toInt() and 0x0f) shl 2) // SUPPRESS CHECKSTYLE
                    or ((`in`[i + 2].toInt() and 0xff) shr 6)] // SUPPRESS CHECKSTYLE
            out[index++] = MAP[(`in`[i + 2].toInt() and 0x3f)] // SUPPRESS CHECKSTYLE
            if (((index - crlr) % 76 == 0) && (index != 0)) { // SUPPRESS CHECKSTYLE
                out[index++] = '\n'.toByte()
                crlr++
                //out[index++] = '\r';
                //crlr++;
            }
            i += 3
        }

        when (`in`.size % 3) {
            1 -> {
                out[index++] =
                    MAP[(`in`[end].toInt() and 0xff) shr 2] // SUPPRESS CHECKSTYLE
                out[index++] =
                    MAP[(`in`[end].toInt() and 0x03) shl 4] // SUPPRESS CHECKSTYLE
                out[index++] = '='.toByte()
                out[index++] = '='.toByte()
            }

            2 -> {
                out[index++] =
                    MAP[(`in`[end].toInt() and 0xff) shr 2] // SUPPRESS CHECKSTYLE
                out[index++] = MAP[((`in`[end].toInt() and 0x03) shl 4) // SUPPRESS CHECKSTYLE
                        or ((`in`[end + 1].toInt() and 0xff) shr 4)] // SUPPRESS CHECKSTYLE
                out[index++] =
                    MAP[((`in`[end + 1].toInt() and 0x0f) shl 2)] // SUPPRESS CHECKSTYLE
                out[index++] = '='.toByte()
            }
        }
        if (charsetName != null) {
            return String(out, 0, index, charset(charsetName))
        }
        return String(out, 0, index, charset("UTF-8"))
    }
}

