package com.halomobi.ssp.base.utils

import android.Manifest
import android.annotation.SuppressLint
import android.content.Context
import android.content.pm.PackageManager
import android.os.Environment
import android.provider.Settings
import android.text.TextUtils
import com.halomobi.ssp.base.utils.SystemUtils.androidID
import com.halomobi.ssp.base.utils.SystemUtils.getIMEI
import com.halomobi.ssp.base.utils.Utils.context
import com.halomobi.ssp.bussiness.module_sdk.view.premission.RequestPermissionManager.Companion.checkSelfPermission
import org.json.JSONException
import org.json.JSONObject
import java.io.BufferedReader
import java.io.BufferedWriter
import java.io.File
import java.io.FileInputStream
import java.io.FileNotFoundException
import java.io.FileOutputStream
import java.io.IOException
import java.io.InputStreamReader
import java.io.OutputStreamWriter
import java.util.UUID

/**
 * cuid 算法工具类。
 */
object CuidUtils {
    private const val CUID = "hj_cuid_cache"
    private const val EXT_FILE = "huanju/.cuid"
    private const val AES_KEY = "stub@2014y03m11d"
    private const val KEY_SYSTEM_SETTINGS_CUID = "com.huanju.stub.cuid.20140311.v1"

    /**
     * cuId在系统设置中存储的Key(尽量复杂,避免于其他APP重名)
     */
    var cuid: String? = null
        get() {
            if (TextUtils.isEmpty(field)) {
                field = fromSp
            }

            if (TextUtils.isEmpty(field)) {
                field = cuidFromSdcard
            }

            if (TextUtils.isEmpty(field)) {
                field = cuidFromSetting
            }

            if (TextUtils.isEmpty(field)) {
                field = createCuid()
            }

            return field
        }
        private set

    private val fromSp: String?
        get() {
            // 看看SharedPreference是否有CUID.
            val sharedPreferences = context?.getSharedPreferences(CUID, 0)
            return sharedPreferences?.getString(CUID, "")
        }

    @SuppressLint("UseKtx")
    private fun setToSp(cuid: String) {
        // 看看SharedPreference是否有CUID.
        val sharedPreferences = context?.getSharedPreferences(CUID, 0)
        val editor = sharedPreferences?.edit()
        editor?.putString(CUID, cuid)
        editor?.apply()
    }

    private fun createCuid(): String {
        LogUtils.d("**** createCuid ****")
        var tmDeviceId = ""
        var androidId = ""
        val configInfo = Utils.mConfigInfo
        if (!TextUtils.isEmpty(configInfo)) {
            try {
                val jsonObject = JSONObject(configInfo)
                tmDeviceId = jsonObject.optString("did")
                androidId = jsonObject.optString("dpid")
            } catch (e: JSONException) {
                e.printStackTrace()
            }
        } else {
            tmDeviceId = getIMEI(0)
            androidId = androidID
        }

        var cuid = ""

        // ====Step3. 如果为空，需要重新生成一个，然后分别写入SystemSettings以及ExternalStorage中.====
        if (TextUtils.isEmpty(cuid)) {
            val uuid = UUID.randomUUID().toString()
            LogUtils.d("uuid: $uuid")

            // 此时IMEI和Android肯定获取过了
            cuid = KeyUtil.getMD5(tmDeviceId + androidId + uuid)
            LogUtils.d("CUID-generated: $cuid")

            // 写入SystemSettings作为备份.
            writeSystemSettings(KEY_SYSTEM_SETTINGS_CUID, cuid)

            // 写入外部存储作为备份
            externalCuid = cuid

            //写入sp中
            setToSp(cuid)
        }

        return cuid
    }

    private val cuidFromSdcard: String
        get() {
            val cuid = externalCuid

            if (!TextUtils.isEmpty(cuid)) {
                LogUtils.d("<<< Get CUID from ext : $cuid")

                // 写入SystemSettings作为备份.
                writeSystemSettings(KEY_SYSTEM_SETTINGS_CUID, cuid)
            }
            return cuid
        }

    private val cuidFromSetting: String
        get() {
            var cuid = ""
            try {
                val context = context
                cuid = Settings.System.getString(
                    context?.contentResolver, KEY_SYSTEM_SETTINGS_CUID
                )
                LogUtils.d("<<< Try to get CUID from sys : $cuid")
            } catch (e: Exception) {
//            LogUtils.d("Settings.System.getString or putString failed" + e);
            }
            return cuid
        }

    private var externalCuid: String
        /**
         * 从外部存储获取cuid
         *
         * @return 外部存储的cuid
         */
        get() {
            if (!checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
//            LogUtils.d("从外部存储获取CUID失败   没有权限");
                return ""
            }

            var cuid = ""
            val extFile = File(
                Environment.getExternalStorageDirectory(), EXT_FILE
            )
            var br: BufferedReader? = null
            try {
                br = BufferedReader(
                    InputStreamReader(
                        FileInputStream(extFile), "UTF-8"
                    )
                )
                val encodeStrBuilder = StringBuilder()
                var line: String? = null
                while ((br.readLine().also { line = it }) != null) {
                    encodeStrBuilder.append(line)
                }

                val encodeStr = encodeStrBuilder.toString()

                if (!TextUtils.isEmpty(encodeStr.trim { it <= ' ' })) {
                    cuid = String(
                        KeyUtil.decrypt(
                            AES_KEY,
                            AES_KEY,
                            Base64Utils.decode(encodeStr.toByteArray(charset("UTF-8")))
                        ), charset("UTF-8")
                    )
                }
            } catch (e: FileNotFoundException) {
                LogUtils.d("No sdcard backup found!")
            } catch (e: IOException) {
//            LogUtils.d("Read sdcard backup fail!\r\n" + e);
            } catch (e: Exception) {
//            LogUtils.d("Decode sdcard backup fail!\r\n" + e);
            } finally {
                FileUtils.close(br)
            }

            return cuid
        }
        /**
         * 将cuid写入外部存储
         *
         * @param cuid 写入的cuid
         */
        private set(cuid) {
            if (TextUtils.isEmpty(cuid)) {
                return
            }

            //		File extFile = new File(
//				android.os.NetEnvironment.getExternalStorageDirectory(), EXT_FILE);
            val dirPath = FileUtils.externalStoragePath
            var bw: BufferedWriter? = null
            try {
                if (!FileUtils.createDirs(dirPath)) {
                    return
                }
                bw = BufferedWriter(
                    OutputStreamWriter(
                        FileOutputStream(
                            File(
                                dirPath, EXT_FILE
                            ), false
                        ), "UTF-8"
                    )
                )
                val encoded = Base64Utils.encode(
                    KeyUtil.encrypt(
                        AES_KEY, AES_KEY, cuid.toByteArray(charset("UTF-8"))
                    ), "UTF-8"
                )
                LogUtils.d(">>> Write encoded ：\r\n$encoded")
                bw.write(encoded)
                bw.flush()
                FileUtils.close(bw)
            } catch (e: IOException) {
//            LogUtils.d("Write sdcard backup fail!\r\n" + e);
            } catch (e: Exception) {
//            LogUtils.d("Encode sdcard backup fail!\r\n" + e);
            } finally {
                FileUtils.close(bw)
            }
        }

    /**
     * 如果具备WriteSystemSettings权限,就将key-value写入SystemSettings.
     *
     * @param key   写入SystemSettings的Key.
     * @param value 写入SystemSettings的value.
     */
    private fun writeSystemSettings(key: String, value: String) {
        // 如具备SystemSettings写权限,就往系统中写.

        if (isPermissionAllowed(
                context, Manifest.permission.WRITE_SETTINGS
            )
        ) {
            try {
                // 写入System.Settings中
                Settings.System.putString(
                    context?.contentResolver, key, value
                )
            } catch (e: Exception) {
//                LogUtils.d("Settings.System.getString or putString failed"
//                        + e);
            }
        }
    }

    /**
     * 检查权限
     *
     * @param context    Context
     * @param permission android.Manifest.permission.xxxx
     * @return 是否具备权限
     */
    private fun isPermissionAllowed(
        context: Context?, permission: String
    ): Boolean {
        val result = context?.checkCallingOrSelfPermission(permission)
        return result == PackageManager.PERMISSION_GRANTED
    }
}
