package com.halomobi.ssp.base.utils

import android.R
import android.annotation.SuppressLint
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.PorterDuff
import android.graphics.PorterDuffXfermode
import android.graphics.Rect
import android.graphics.RectF
import android.graphics.drawable.Drawable
import android.graphics.drawable.GradientDrawable
import android.graphics.drawable.StateListDrawable
import android.view.View
import android.view.animation.Animation
import android.view.animation.AnimationUtils
import com.halomobi.ssp.base.utils.Utils.context
import kotlin.math.max
import kotlin.math.min

object DrawableUtils {
    /**
     * 创建一个图片
     *
     * @param contentColor 内部填充颜色
     * @param strokeColor  描边颜色
     * @param radius       圆角
     */
    fun createDrawable(
        contentColor: Int,
        strokeColor: Int, radius: Int
    ): GradientDrawable {
        val drawable = GradientDrawable() // 生成Shape
        drawable.gradientType = GradientDrawable.LINEAR_GRADIENT // 设置矩形
        drawable.setColor(contentColor) // 内容区域的颜色
        drawable.setStroke(
            1,
            strokeColor
        ) // 四周描边,描边后四角真正为圆角，不会出现黑色阴影。如果父窗体是可以滑动的，需要把父View设置setScrollCache(false)
        drawable.cornerRadius = radius.toFloat() // 设置四角都为圆角
        return drawable
    }

    /**
     * 创建一个图片选择器
     *
     * @param normalState  普通状态的图片
     * @param pressedState 按压状态的图片
     */
    @SuppressLint("UseKtx")
    fun createSelector(
        normalState: Drawable,
        pressedState: Drawable?
    ): StateListDrawable {
        var pressedState = pressedState
        val bg = StateListDrawable()
        if (pressedState == null) {
            pressedState = normalState.constantState?.newDrawable()
            pressedState?.setColorFilter(Color.parseColor("#30000000"), PorterDuff.Mode.MULTIPLY)
        }
        bg.addState(
            intArrayOf(
                R.attr.state_pressed,
                R.attr.state_enabled
            ), pressedState
        )
        bg.addState(intArrayOf(R.attr.state_enabled), normalState)
        bg.addState(intArrayOf(), normalState)
        return bg
    }

    fun createSelector(
        normalColor: Int,
        pressedColro: Int
    ): StateListDrawable {
        val normalState: Drawable = createDrawable(normalColor, normalColor, 0)
        val pressedState: Drawable = createDrawable(pressedColro, pressedColro, 0)
        return createSelector(normalState, pressedState)
    }

    //	/** 获取图片的大小 */
    //	@SuppressLint("NewApi")
    //	public static int getDrawableSize(Drawable drawable) {
    //		if (drawable == null) {
    //			return 0;
    //		}
    //		Bitmap bitmap = ((BitmapDrawable) drawable).getBitmap();
    //		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR1) {
    //			return bitmap.getByteCount();
    //		} else {
    //			return bitmap.getRowBytes() * bitmap.getHeight();
    //		}
    //	}
    /**
     * 获得“x”的path路径
     *
     * @param roundX 圆心的X坐标
     * @param roundY 圆心的Y坐标
     * @param roundR 圆的半径
     * @return
     */
    fun getXPath(roundX: Float, roundY: Float, roundR: Float): FloatArray {
        val point = floatArrayOf(
            roundX - roundR / 2, roundY - roundR / 2,
            roundX + roundR / 2, roundY + roundR / 2, roundX - roundR / 2,
            roundY + roundR / 2, roundX + roundR / 2, roundY - roundR / 2
        )
        return point
    }

    /**
     * 设置关闭区域
     */
    @JvmStatic
    fun getArea(v: View, r: RectF, rang: Int): RectF {
        val left = max(0.0, (r.left - rang).toDouble()).toFloat()
        val top = max(0.0, (r.top - rang).toDouble()).toFloat()
        val right = min(v.width.toDouble(), (r.right + rang).toDouble()).toFloat()
        val bottom = min(v.height.toDouble(), (r.bottom + rang).toDouble()).toFloat()
        return RectF(left, top, right, bottom)
    }


    @JvmStatic
    val displayAnim: Animation
        get() = AnimationUtils.loadAnimation(
            context,
            R.anim.fade_in
        )

    //生成圆角图片
    @SuppressLint("UseKtx")
    @JvmStatic
    fun getRoundedCornerBitmap(bitmap: Bitmap, roundPx: Float): Bitmap? {
        try {
            val output = Bitmap.createBitmap(
                bitmap.width,
                bitmap.height, Bitmap.Config.ARGB_8888
            )
            val canvas = Canvas(output)
            val paint = Paint()
            val rect = Rect(0, 0, bitmap.width, bitmap.height)
            val rectF = RectF(Rect(0, 0, bitmap.width, bitmap.height))
            paint.isAntiAlias = true
            canvas.drawARGB(0, 0, 0, 0)
            paint.color = Color.BLACK
            canvas.drawRoundRect(rectF, roundPx, roundPx, paint)
            paint.setXfermode(PorterDuffXfermode(PorterDuff.Mode.SRC_IN))
            val src = Rect(0, 0, bitmap.width, bitmap.height)
            canvas.drawBitmap(bitmap, src, rect, paint)
            return output
        } catch (e: Exception) {
            return bitmap
        }
    }
}
