package com.halomobi.ssp.base.utils

import java.io.BufferedReader
import java.io.DataOutputStream
import java.io.InputStreamReader
import java.util.concurrent.locks.ReadWriteLock
import java.util.concurrent.locks.ReentrantReadWriteLock

class ExeCommandUtils {
    //shell进程
    private var process: Process? = null

    //对应进程的3个流
    private var successResult: BufferedReader? = null
    private var errorResult: BufferedReader? = null
    private var os: DataOutputStream? = null

    //是否同步，true：run会一直阻塞至完成或超时。false：run会立刻返回
    private val bSynchronous: Boolean

    /**
     * 还没开始执行，和已经执行完成 这两种情况都返回false
     *
     * @return 是否正在执行
     */
    //表示shell进程是否还在运行
    var isRunning: Boolean = false
        private set

    //同步锁
    var lock: ReadWriteLock = ReentrantReadWriteLock()

    //保存执行结果
     val result = StringBuffer()

    /**
     * 构造函数
     *
     * @param synchronous true：同步，false：异步
     */
    constructor(synchronous: Boolean) {
        bSynchronous = synchronous
    }

    /**
     * 默认构造函数，默认是同步执行
     */
    constructor() {
        bSynchronous = true
    }

    /**
     * @return 返回执行结果
     */
    fun getResult(): String {
        val readLock = lock.readLock()
        readLock.lock()
        try {
            return String(result)
        } finally {
            readLock.unlock()
        }
    }

    /**
     * 执行命令
     *
     * @param command eg: cat /sdcard/test.txt
     * @param maxTime 最大等待时间 (ms)
     * @return this
     */
    fun run(command: String?, maxTime: Int): ExeCommandUtils {
        if (command == null || command.length == 0) {
            return this
        }

        try {
            process = Runtime.getRuntime().exec("sh")
        } catch (e: Throwable) {
            return this
        }
        isRunning = true
        successResult = BufferedReader(InputStreamReader(process?.inputStream))
        errorResult = BufferedReader(InputStreamReader(process?.errorStream))
        os = DataOutputStream(process?.outputStream)

        try {
            //向sh写入要执行的命令
            os?.write(command.toByteArray())
            os?.writeBytes("\n")
            os?.flush()

            os?.writeBytes("exit\n")
            os?.flush()

            os?.close()
            //如果等待时间设置为非正，就不开启超时关闭功能
            if (maxTime > 0) {
                //超时就关闭进程
                Thread {
                    try {
                        Thread.sleep(maxTime.toLong())
                    } catch (e: Throwable) {
                    }
                    try {
                        val ret = process?.exitValue()
                    } catch (e: Throwable) {
                        process?.destroy()
                    }
                }.start()
            }

            //开一个线程来处理input流
            val t1 = Thread {
                var line: String?
                val writeLock = lock.writeLock()
                try {
                    while ((successResult?.readLine().also { line = it }) != null) {
                        line += "\n"
                        writeLock.lock()
                        result.append(line)
                        writeLock.unlock()
                    }
                } catch (e: Throwable) {
                } finally {
                    try {
                        successResult?.close()
                    } catch (e: Throwable) {
                    }
                }
            }
            t1.start()

            //开一个线程来处理error流
            val t2 = Thread {
                var line: String?
                val writeLock = lock.writeLock()
                try {
                    while ((errorResult?.readLine().also { line = it }) != null) {
                        line += "\n"
                        writeLock.lock()
                        result.append(line)
                        writeLock.unlock()
                    }
                } catch (e: Throwable) {
                } finally {
                    try {
                        errorResult?.close()
                    } catch (e: Throwable) {
                    }
                }
            }
            t2.start()

            val t3 = Thread(object : Runnable {
                override fun run() {
                    try {
                        //等待执行完毕
                        t1.join()
                        t2.join()
                        process?.waitFor()
                    } catch (e: Throwable) {
                    } finally {
                        isRunning = false
                    }
                }
            })
            t3.start()

            if (bSynchronous) {
                t3.join()
            }
        } catch (e: Throwable) {
        }
        return this
    }
}
