package com.halomobi.ssp.base.utils

import com.halomobi.ssp.base.utils.Base64Utils.decode
import com.halomobi.ssp.base.utils.Base64Utils.encode
import java.io.UnsupportedEncodingException
import java.security.MessageDigest
import java.security.NoSuchAlgorithmException
import javax.crypto.Cipher
import javax.crypto.spec.IvParameterSpec
import javax.crypto.spec.SecretKeySpec

/**
 * AES 算法工具类。
 */
object KeyUtil {
    private const val PRIVATE_KEY = "halomobi"
    private const val IV = "2011121211143000"

    /**
     * the name of the transformation to create a cipher for.
     */
    private const val TRANSFORMATION = "AES/CBC/PKCS5Padding"
    private const val DETRANSFORMATION = "AES/CBC/PKCS7Padding"

    /**
     * 算法名称
     */
    private const val ALGORITHM_NAME = "AES"
    private val HEX_DIGITS = charArrayOf(
        '0', '1', '2', '3', '4', '5',
        '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
    )

    //AES加密

    fun encryptCurrentAES(uid: String, text: String): ByteArray {
        // 固定前缀PRIVATE_KEY + uid
        var key = PRIVATE_KEY + uid
        // 转化为16位key
        key = getMD5(key)
        val size = key.length
        key = key.substring(size / 2).toLowerCase()
        val bs = encrypt(IV, key, text.toByteArray(charset("UTF-8")))
        val encText = encode(bs)
        return bs
    }
    fun encryptAES(uid: String, text: String): String {
        // 固定前缀PRIVATE_KEY + uid

        var key = PRIVATE_KEY + uid

        // 转化为16位key
        key = getMD5(key)
        val size = key.length
        key = key.substring(size / 2)

        // AES加密
        val data = encrypt(IV, key, text.toByteArray(charset("UTF-8")))

        // base64转化
        val encText = encode(data)

        return encText
    }

    fun decryptAES(uid: String, text: String): String {
        // base64解码

        val decrypted = decode(text.toByteArray(charset("UTF-8")))

        // 固定前缀PRIVATE_KEY + uid
        var key = PRIVATE_KEY + uid

        // 转化为16位key
        key = getMD5(key)
        val size = key.length
        key = key.substring(size / 2)

        // 解密
        val data = decrypt(IV, key, decrypted)

        val realText = String(data, charset("UTF-8"))
        return realText
    }

    /**
     * aes 加密，AES/CBC/PKCS5Padding
     *
     * @param key     密钥字符串, 此处使用AES-128-CBC加密模式，key需要为16位
     * @param content 要加密的内容
     * @param cbcIv   初始化向量(CBC模式必须使用) 使用CBC模式，需要一个向量iv，可增加加密算法的强度
     * @return 加密后原始二进制字符串
     */
    fun encrypt(cbcIv: String, key: String, content: ByteArray?): ByteArray {
        val sksSpec = SecretKeySpec(
            key.toByteArray(charset("UTF-8")),
            ALGORITHM_NAME
        )

        val cipher = Cipher.getInstance(TRANSFORMATION)

        val iv = IvParameterSpec(cbcIv.toByteArray(charset("UTF-8")))

        cipher.init(Cipher.ENCRYPT_MODE, sksSpec, iv)

        val encrypted = cipher.doFinal(content)

        return encrypted
    }

    /**
     * aes 解密，AES/CBC/PKCS5Padding
     *
     * @param key       密钥, 此处使用AES-128-CBC加密模式，key需要为16位
     * @param encrypted 密文
     * @param cbcIv     初始化向量(CBC模式必须使用) 使用CBC模式，需要一个向量iv，可增加加密算法的强度
     * @return 明文
     * @throws Exception 异常
     */
    fun decrypt(cbcIv: String, key: String, encrypted: ByteArray?): ByteArray {
        val skeSpect = SecretKeySpec(
            key.toByteArray(charset("UTF-8")),
            ALGORITHM_NAME
        )

        val cipher = Cipher.getInstance(DETRANSFORMATION)

        val iv = IvParameterSpec(cbcIv.toByteArray(charset("UTF-8")))

        cipher.init(Cipher.DECRYPT_MODE, skeSpect, iv)

        val decrypted = cipher.doFinal(encrypted)

        return decrypted
    }

    @JvmStatic
    fun getMD5(originalString: String?): String {
        try {
            val digester = MessageDigest.getInstance("MD5")
            try {
                originalString?.toByteArray(charset("UTF-8"))?.let { digester.update(it) }
            } catch (e: UnsupportedEncodingException) {
                e.printStackTrace()
            }
            val b = digester.digest()
            return bytesToHexString(b)
        } catch (e: NoSuchAlgorithmException) {
            throw RuntimeException(e)
        }
    }

    @JvmStatic
    fun bytesToHexString(b: ByteArray): String {
        val CONSTANT_NUMBER0XF0 = 0xf0
        val CONSTANT_NUMBER0X0F = 0x0f
        val CONSTANT_NUMBER4 = 4
        val builder = StringBuilder(b.size * 2)
        for (i in b.indices) {
            builder.append(HEX_DIGITS[(b[i].toInt() and CONSTANT_NUMBER0XF0) ushr CONSTANT_NUMBER4])
            builder.append(HEX_DIGITS[b[i].toInt() and CONSTANT_NUMBER0X0F])
        }
        val string = builder.toString()
        return string
    }
}
