/*
 * File Name: LogUtils.java
 * History:
 * Created by mwqi on 2014-4-4
 */
package com.halomobi.ssp.base.utils

import android.annotation.SuppressLint
import android.content.ClipboardManager
import android.content.Context
import android.text.TextUtils
import android.util.Log
import com.halomobi.ssp.model.request.config.SDKInfo
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale

object LogUtils {
    /**
     * 日志输出级别NONE
     */
    const val LEVEL_NONE: Int = 0

    /**
     * 日志输出级别V
     */
    const val LEVEL_VERBOSE: Int = 1

    /**
     * 日志输出级别D
     */
    const val LEVEL_DEBUG: Int = 2

    /**
     * 日志输出级别I
     */
    const val LEVEL_INFO: Int = 3

    /**
     * 日志输出级别W
     */
    const val LEVEL_WARN: Int = 4

    /**
     * 日志输出级别E
     */
    const val LEVEL_ERROR: Int = 5
    private const val TEMPLATE1 = "%s(L:%d)  %s"

    /**
     * 日志输出时的TAG
     */
    private const val TAG = SDKInfo.LOGTAG

    /**
     * 是否允许输出log
     */
    private var mDebuggable = LEVEL_NONE

    private const val clipText = "c372e6227d4a89c55d36addf72c7ea3f"


    @JvmStatic
    fun setDeBugModel(debug: Int) {
        mDebuggable = debug
    }

    private val clipBoard: Boolean
        get() {
            try {
                val cm = MyUtils.context
                    ?.getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
                val data = cm.primaryClip ?: return false
                val item = data.getItemAt(0)
                val content = item.text.toString()
                if (TextUtils.equals(clipText, content)) {
                    return true
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            return false
        }

    @JvmStatic
    fun getThrowable(e: Throwable): String {
        val sb_ero = StringBuffer("\n----------------------------------")
        sb_ero.append("threadId:" + Thread.currentThread().id + "\n")
        sb_ero.append("error:" + e.message + "\n")

        val elements = e.stackTrace

        for (element in elements) {
            sb_ero.append(element.toString() + "\n")
        }
        sb_ero.append(
            """
                
                causeBy:${e.message}
                
                """.trimIndent()
        )
        val cause = e.cause
        if (cause != null) {
            val causesElements = cause.stackTrace

            for (element in causesElements) {
                sb_ero.append(element.toString() + "\n")
            }
        }
        sb_ero.append("\n----------------------------------------")
        return sb_ero.toString()
    }

    //==================================================================================
    /**
     * 以级别为 v 的形式输出LOG
     */
    fun v(log: String?) {
        if (mDebuggable >= LEVEL_VERBOSE) {
            val sElements = Throwable().stackTrace

            //            LogUtils.d(TAG+createLog(sElements, log));
        }
    }

    /**
     * 以级别为 d 的形式输出LOG
     */
    @JvmStatic
    fun d(log: String) {
        if (mDebuggable >= LEVEL_VERBOSE) {
            val sElements = Throwable().stackTrace

            //            Log.d(TAG, createLog(sElements, log));
            Log.d(TAG, log)
        }
    }

    /**
     * 以级别为 i 的形式输出LOG
     */
    fun i(log: String) {
        if (mDebuggable >= LEVEL_VERBOSE) {
            val sElements = Throwable().stackTrace

            Log.i(TAG, createLog(sElements, log))
        }
    }

    /**
     * 以级别为 w 的形式输出LOG
     */
    fun w(log: String) {
        if (mDebuggable >= LEVEL_VERBOSE) {
            val sElements = Throwable().stackTrace

            Log.w(TAG, createLog(sElements, log))
        }
    }

    /**
     * 以级别为 w 的形式输出LOG
     */
    fun w(throwable: Throwable) {
        if (mDebuggable >= LEVEL_VERBOSE) {
            val log = createThrowableInfo(throwable)
            val sElements = Throwable().stackTrace

            Log.w(TAG, createLog(sElements, log))
        }
    }

    /**
     * 以级别为 e 的形式输出LOG
     */
    fun e(log: String) {
        if (mDebuggable >= LEVEL_VERBOSE) {
            val sElements = Throwable().stackTrace

            Log.e(TAG, createLog(sElements, log))
        }
    }

    /**
     * 以级别为 e 的形式输出LOG
     */
    fun e(throwable: Throwable) {
        if (mDebuggable >= LEVEL_VERBOSE) {
            val log = createThrowableInfo(throwable)
            val sElements = Throwable().stackTrace

            Log.e(TAG, createLog(sElements, log))
        }
    }

    /**
     * 拼接log
     *
     * @param sElements
     * @param log
     * @return
     */
    @Synchronized
    private fun createLog(sElements: Array<StackTraceElement>, log: String): String {
        val methodName = sElements[1].methodName
        val lineNumber = sElements[1].lineNumber
        @SuppressLint("DefaultLocale") val logInfo =
            String.format(TEMPLATE1, methodName, lineNumber, log)
        // 存储log2file
        // saveLog(saveLevel, logInfo);
        return logInfo
    }

    private fun createThrowableInfo(ex: Throwable): String {
        val sb_record = StringBuffer()
        val elements = ex.stackTrace
        for (element in elements) {
            sb_record.append(element.toString() + "\n")
        }
        sb_record.append(
            """
                
                causeBy:${ex.message}
                
                """.trimIndent()
        )
        val cause = ex.cause
        if (cause != null) {
            val causesElements = cause.stackTrace

            for (element in causesElements) {
                sb_record.append(element.toString() + "\n")
            }
        }
        return sb_record.toString()
    }

    /**
     * 转化成可读时间
     *
     * @param time 时间戳
     * @return 可读时间
     */
    @JvmStatic
    fun formatDate(time: Long): String {
        return SimpleDateFormat("yy-MM-dd hh:mm:ss.SSS", Locale.getDefault()).format(Date(time))
    }
}
