package com.halomobi.ssp.base.utils

import android.annotation.SuppressLint
import android.content.Context
import android.content.SharedPreferences
import android.content.res.Configuration
import android.content.res.Resources
import android.net.ConnectivityManager
import android.os.Handler
import android.os.Looper
import android.os.Looper.getMainLooper
import android.text.TextUtils
import android.view.WindowManager
import com.halomobi.ssp.base.utils.LogUtils.LEVEL_ERROR
import com.halomobi.ssp.base.utils.LogUtils.LEVEL_NONE
import com.halomobi.ssp.base.utils.LogUtils.setDeBugModel
import java.security.MessageDigest
import java.security.NoSuchAlgorithmException


@SuppressLint("StaticFieldLeak")
object MyUtils {
    var context: Context? = null
        private set
    private const val SP_KEY_NAME = "huanju_fusion_info"

    @get:Synchronized
    var uiHandler: Handler? = null
        private set

    @Synchronized
    fun init(cx: Context) {
        context = cx.applicationContext
        if (uiHandler == null) {
            uiHandler = Handler(getMainLooper())
        }
    }

    @Synchronized
    fun init(cx: Context, isDebug: Boolean) {
        context = cx.applicationContext
        if (uiHandler == null) {
            uiHandler = Handler(getMainLooper())
        }
        setDeBugModel(if (isDebug) LEVEL_ERROR else LEVEL_NONE)
    }

    fun runInMainThread(runnable: Runnable) {
        if (isRunInMainThread) {
            runnable.run()
        } else {
            post(runnable)
        }
    }

    val sp: SharedPreferences?
        get() {
            val context = context
            if (context != null) {
                return context.getSharedPreferences(SP_KEY_NAME, 0)
            }
            return null
        }

    @SuppressLint("UseKtx")
    fun putLong(name: String?, value: Long) {
        sp?.edit()?.putLong(name, value)?.apply()
    }

    fun getLong(name: String?, value: Long): Long? {
        return sp?.getLong(name, value)
    }

    val isRunInMainThread: Boolean
        get() = Thread.currentThread().id == Looper.getMainLooper().thread
            .id

    fun post(runnable: Runnable) {
        uiHandler?.post(runnable)
    }

    val resources: Resources?
        get() = context?.resources
    val isPortrait: Boolean
        get() {
            val mConfiguration = context?.resources
                ?.configuration // 获取设置的配置信息
            val ori = mConfiguration?.orientation // 获取屏幕方向

            if (ori == Configuration.ORIENTATION_LANDSCAPE) {
                // 横屏
                return false
            } else if (ori == Configuration.ORIENTATION_PORTRAIT) {
                // 竖屏
                return true
            }
            return false
        }

    /**
     * 获取屏幕尺寸信息
     *
     * @return
     */
    @Suppress("deprecation")
    fun getScreenSize(cx: Context?, matchScreen: Boolean): IntArray {
        var cx = cx
        try {
            if (cx == null) {
                cx = context
            }
            val wm = cx?.getSystemService(
                Context.WINDOW_SERVICE
            ) as WindowManager
            val display = wm.defaultDisplay
            return if (!matchScreen || isPortrait) {
                intArrayOf(display.width, display.height)
            } else {
                intArrayOf(display.height, display.width)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return intArrayOf(0, 0)
    }

    fun getScreenWidthDip(context: Context): Int {
        return px2dip(context, getScreenSize(context, true)[0].toFloat())
    }

    fun getScreenHeightDip(context: Context): Int {
        return px2dip(context, getScreenSize(context, true)[1].toFloat())
    }

    fun px2dip(context: Context, pxValue: Float): Int {
        val scale = context.resources.displayMetrics.density
        return (pxValue / scale + 0.5f).toInt()
    }

    fun WifiConnected(context: Context): Boolean {
        val connectivityManager =
            context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        val activeNetInfo = connectivityManager.activeNetworkInfo
        if (activeNetInfo != null && activeNetInfo.type == ConnectivityManager.TYPE_WIFI) {
            return true
        }
        return false
    }

    fun md5(string: String): String {
        if (TextUtils.isEmpty(string)) {
            return ""
        }
        var md5: MessageDigest? = null
        try {
            md5 = MessageDigest.getInstance("MD5")
            val bytes = md5.digest(string.toByteArray())
            val result = StringBuilder()
            for (b in bytes) {
                var temp = Integer.toHexString(b.toInt() and 0xff)
                if (temp.length == 1) {
                    temp = "0$temp"
                }
                result.append(temp)
            }
            return result.toString()
        } catch (e: NoSuchAlgorithmException) {
            e.printStackTrace()
        }
        return ""
    }
}
