package com.halomobi.ssp.base.utils

import android.Manifest
import android.annotation.SuppressLint
import android.bluetooth.BluetoothAdapter
import android.content.ActivityNotFoundException
import android.content.Context
import android.content.Intent
import android.content.pm.PackageInfo
import android.content.pm.PackageManager
import android.content.res.Configuration
import android.graphics.Point
import android.location.Location
import android.location.LocationManager
import android.net.Uri
import android.net.wifi.WifiManager
import android.os.Build
import android.telephony.TelephonyManager
import android.text.TextUtils
import android.util.DisplayMetrics
import android.view.WindowManager
import android.webkit.WebSettings
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.base.utils.Utils.context
import com.halomobi.ssp.base.utils.Utils.resources
import com.halomobi.ssp.base.utils.Utils.sp
import com.halomobi.ssp.bussiness.module_sdk.view.premission.RequestPermissionManager.Companion.checkSelfPermission
import org.json.JSONException
import org.json.JSONObject
import java.io.ByteArrayOutputStream
import java.io.File
import java.io.IOException
import java.io.InputStreamReader
import java.io.LineNumberReader
import java.io.RandomAccessFile
import java.net.NetworkInterface
import java.text.DecimalFormat
import java.text.ParseException
import java.text.SimpleDateFormat
import java.util.Collections
import java.util.Date
import java.util.Locale
import java.util.Objects
import java.util.zip.GZIPOutputStream

object SystemUtils {
    var ProvidersName: Int = 0
    private const val unknown = ""
    private var sUpdataMark = ""
    private var sBookMark = "0"
    private var location: Location? = null
    private var isLoadLocation = false

    val oSVersion: String
        /**
         * 获取android系统版本号
         */
        get() = Build.VERSION.RELEASE // android系统版本号

    val sDK_INT: Int
        /**
         * 获取android系统版本号
         */
        get() = Build.VERSION.SDK_INT

    val vendor: String
        /**
         * 获取厂商名称
         */
        get() = Build.BRAND

    /**
     * 判断是否包含SIM卡
     */
    fun hasSimCard(telMgr: TelephonyManager): Boolean {
        return telMgr.simState > TelephonyManager.SIM_STATE_ABSENT
    }

    val deviceModel: String
        /**
         * 获取手机型号
         */
        get() = Build.MODEL

    @JvmStatic
    fun getIMEI(time: Int): String {
        var imei = ""
        imei = getInfoByJson("did")
        return imei
    }

    @JvmStatic
    val operatorsName: Int
        /**
         * 返回手机服务商名字
         */
        get() {
            if (ProvidersName > 0) return ProvidersName
            try {
                // IMSI号前面3位460是国家，紧接着后面2位00 02是中国移动，01是中国联通，03是中国电信。
                val tm =
                    context?.applicationContext?.getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
                val operator = tm.simOperator
                if ("46001" == operator || "46006" == operator || "46009" == operator) {
                    ProvidersName = 2
                } else if ("46000" == operator || "46002" == operator || "46004" == operator || "46007" == operator) {
                    ProvidersName = 1
                } else if ("46003" == operator || "46005" == operator || "46011" == operator) {
                    ProvidersName = 3
                }
            } catch (e: Exception) {
//            LogUtils.d("获取手机服务商名字 失败");
                e.printStackTrace()
            }
            return ProvidersName
        }


    @JvmStatic
    val cellularId: String
        /**
         * 获取运营商基站ID
         *
         * @return 如果没有权限，则返回默认值 0
         */
        get() {
            val cellularId = ""
            return if (TextUtils.isEmpty(cellularId)) ConstantPool.DEFAULT_CELLULAR_ID else cellularId
        }

    @JvmStatic
    val lac: String
        /**
         * 获取基站位置区域码
         *
         * @return 如果没有权限，则返回默认值 0
         */
        get() {
            val lac = ""
            return if (TextUtils.isEmpty(lac)) ConstantPool.DEFAULT_LAC else lac
        }

    @JvmStatic
    val mcc: String
        get() {
            var mcc = "460"
            try {
                val tm = context?.getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
                val operator = tm.networkOperator
                if (operator != null && operator.length > 3) {
                    mcc = operator.substring(0, 3)
                }
            } catch (e: Exception) {
//            LogUtils.d("get Mcc 失败");
                e.printStackTrace()
            }
            return mcc
        }

    @JvmStatic
    val bssId: String
        /**
         * 获取wifi mac地址
         */
        get() {
            var mBss_id = "0"
            try {
                val wifiManager =
                    context?.applicationContext?.getSystemService(Context.WIFI_SERVICE) as WifiManager
                val wifiInfo = wifiManager.connectionInfo
                if (wifiInfo != null) {
                    mBss_id = wifiInfo.bssid
                }
            } catch (e: Exception) {
//            LogUtils.d("获取手机mac地址 失败");
                e.printStackTrace()
            }
            return mBss_id
        }

    fun getMacAddress(context: Context?): String {
        return if (Build.VERSION.SDK_INT >= 23) {
            wifiMacAddress
        } else {
            getMacInNormal(context)
        }
    }

    val wifiMacAddress: String
        get() {
            try {
                val interfaceName = "wlan0"
                val interfaces: List<NetworkInterface> =
                    Collections.list(NetworkInterface.getNetworkInterfaces())
                for (intf in interfaces) {
                    if (!intf.name.equals(interfaceName, ignoreCase = true)) {
                        continue
                    }
                    val mac = intf.hardwareAddress ?: return ""
                    val buf = StringBuilder()
                    for (aMac in mac) {
                        buf.append(String.format("%02X:", aMac))
                    }
                    if (buf.length > 0) {
                        buf.deleteCharAt(buf.length - 1)
                    }
                    return buf.toString()
                }
            } catch (ex: Exception) {
            }
            return ""
        }

    @SuppressLint("HardwareIds", "MissingPermission")
    private fun getMacInNormal(context: Context?): String {
        var mac = ""
        if (null != context) {
            try {
                val wm =
                    context.applicationContext.getSystemService(Context.WIFI_SERVICE) as WifiManager
                val info = wm.connectionInfo
                mac = info.macAddress
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
        return mac
    }

    @JvmStatic
    val androidID: String
        get() {
            val mAndroidID = getInfoByJson("dpid")
            return if (!TextUtils.isEmpty(mAndroidID)) {
                mAndroidID
            } else {
                ""
            }
            //        return Settings.System.getString(getContext().getContentResolver(), Secure.ANDROID_ID);
        }

    @SuppressLint("MissingPermission")
    fun getLocation(): Location? {
        if (!checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION)) {
//            LogUtils.d("获取经纬度失败  没有权限");
            return null
        }

        try {
            //获取地理位置管理器
            val locationManager =
                context?.getSystemService(Context.LOCATION_SERVICE) as LocationManager
            //获取所有可用的位置提供器
            val providers = locationManager.getProviders(true)
            var locationProvider: String? = null
            if (providers.contains(LocationManager.GPS_PROVIDER)) {
                //如果是GPS
                locationProvider = LocationManager.GPS_PROVIDER
            } else if (providers.contains(LocationManager.NETWORK_PROVIDER)) {
                //如果是Network
                locationProvider = LocationManager.NETWORK_PROVIDER
            }
            //            LogUtils.d("locationProvider   -   " + locationProvider);
            location = locationManager.getLastKnownLocation(LocationManager.PASSIVE_PROVIDER)
            isLoadLocation = true
            return location
        } catch (e: Exception) {
//            LogUtils.d("获取经纬度失败");
            e.printStackTrace()
        }
        return null
    }

    @JvmStatic
    val latitude: Double
        get() {
            var currentLocation: Location? = null
            if (location == null && !isLoadLocation) {
                currentLocation = getLocation()
            } else {
                if (location != null) {
                    currentLocation = location
                }
            }
            if (currentLocation != null) {
                return formatFloat(currentLocation.latitude)
            }
            return 0.0
        }

    @JvmStatic
    val longitude: Double
        get() {
            var currentLocation: Location? = null
            if (location == null && !isLoadLocation) {
                currentLocation = getLocation()
            } else {
                if (location != null) {
                    currentLocation = location
                }
            }
            if (currentLocation != null) {
                return formatFloat(currentLocation.longitude)
            }
            return 0.0
        }

    private fun formatFloat(number: Double): Double {
        val format = DecimalFormat("#.00000")
        val formatStr = format.format(number)
        return formatStr.toDouble()
    }

    val deviceType: Int
        /**
         * 判断当前设备是手机还是平板，代码来自 Google I/O App for Android
         *
         * @return 平板返回 True，手机返回 False
         */
        get() = if (((resources?.configuration?.screenLayout?.and(Configuration.SCREENLAYOUT_SIZE_MASK))
                ?: 0) >= Configuration.SCREENLAYOUT_SIZE_LARGE
        ) 2 else 1

    val device: String
        /**
         * 获取设备机型信息
         *
         * @return 机型信息
         */
        get() = Build.MANUFACTURER.replace("_", "-") + "_" + Build.MODEL.replace("_", "-") // 机型信息

    private fun encrypt(content: String): String? {
        var content = content
        try { // 加密渠道号
            content = KeyUtil.encryptAES(CuidUtils.cuid.toString(), content)
        } catch (e: Exception) {
//            LogUtils.d("encrypt ChannelID error." + e);
        }
        return content
    }

    val clientID: String?
        get() = encrypt(getIMEI(0))

    val deviceID: String?
        get() = encrypt(NetworkUtils.macAddress)

    /**
     * 跳转系统浏览器
     *
     * @param url 目标页地址
     */
    @JvmStatic
    fun loopWeb(url: String?): Boolean {
        try {
            val intent = Intent(Intent.ACTION_VIEW)
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            intent.setData(Uri.parse(url))
            context?.startActivity(intent)
            return true
        } catch (e: ActivityNotFoundException) {
//            LogUtils.d("跳转系统浏览器 失败");
            e.printStackTrace()
        }
        return false
    }

    val dPI: Float
        get() {
            val wm = context?.getSystemService(Context.WINDOW_SERVICE) as WindowManager
            val metric = DisplayMetrics()
            wm.defaultDisplay.getMetrics(metric)
            //		int densityDpi = metric.densityDpi; // 屏幕密度DPI（120 / 160 / 240）
            return metric.density // 屏幕密度（0.75 / 1.0 / 1.5）
        }

    val deviceDPI: Double
        get() {
            val wm = context?.getSystemService(Context.WINDOW_SERVICE) as WindowManager
            val metric = DisplayMetrics()
            wm.defaultDisplay.getMetrics(metric)
            return metric.densityDpi.toDouble() // 屏幕密度DPI（120 / 160 / 240）
        }

    val appVer: String
        /**
         * 获取App版本
         *
         * @return App版本
         */
        get() {
            try {
                val pm = context?.getPackageManager()
                val pi = pm?.getPackageInfo(
                    context?.packageName.toString(), PackageManager.GET_CONFIGURATIONS
                )

                return pi?.versionName.toString()
            } catch (e: Exception) {
//            LogUtils.d("AppVersionInfo 获取失败");
                e.printStackTrace()
            }
            return "nul"
        }

    /**
     * 获取系统默认的UA
     */
    fun initUserAgent() {
        if (context == null) {
            return
        }
        val sp = sp
        var userAgentString: String? = ""
        if (!TextUtils.isEmpty(userAgentString)) {
            sp?.edit()?.putString(ConstantPool.UA_KEY, userAgentString)?.apply()
            return
        }
        try {
            if (sp?.getString(ConstantPool.UA_KEY, null) != null) {
                return
            }
            //            由于手动上报的UA需要和HttpURLConnection自带的UA相同，所以在1 .10 .3 版本中修改为获取系统UA
            userAgentString = WebSettings.getDefaultUserAgent(context)
            sp?.edit()?.putString(ConstantPool.UA_KEY, userAgentString)?.apply()
        } catch (e: Exception) {
//            LogUtils.d("UserAgent 获取失败");
            e.printStackTrace()
        }
    }

    /**
     * 外层使用ByteArrayEntity包装压缩内容。如使用HttpEntity(即StringEntity)会导致服务器端解析压缩内容出错
     * (压缩格式不匹配) 采用byteArray，经测试与服务器端匹配。
     */
    fun gZip(data: ByteArray?): ByteArray? {
        var b: ByteArray? = null
        try {
            val bos = ByteArrayOutputStream()
            val gzip = GZIPOutputStream(bos)
            gzip.write(data)
            gzip.finish()
            gzip.close()
            b = bos.toByteArray()
            bos.close()
        } catch (ex: Exception) {
//            LogUtils.d("GZip压缩失败");
            ex.printStackTrace()
            return b
        }
        return b
    }

    val language: String
        /**
         * 获取系统语言
         *
         * @return 默认返回nul
         */
        get() {
            try {
                return resources?.configuration?.locale?.language.toString()
            } catch (e: Exception) {
//            LogUtils.d("系统语言 获取失败");
                e.printStackTrace()
                return "nul"
            }
        }

    @get:SuppressLint("NewApi")
    val resolution: String
        /**
         * 获取手机分辨率
         */
        get() {
            try {
                val wm = context?.getSystemService(Context.WINDOW_SERVICE) as WindowManager
                val display = wm.defaultDisplay
                val size = Point()
                display.getSize(size)
                return size.x.toString() + "_" + size.y
            } catch (e: Exception) {
//            LogUtils.d("手机分辨率 获取失败");
                e.printStackTrace()
            }
            return "nul"
        }

    val isRoot: Boolean
        /**
         * 检测是否root
         *
         * @return 是否root
         */
        get() {
            var f: File
            val kSuSearchPaths = arrayOf(
                "/system/bin/", "/system/xbin/", "/system/sbin/", "/sbin/", "/vendor/bin/"
            )
            try {
                for (kSuSearchPath in kSuSearchPaths) {
                    f = File(kSuSearchPath + "su")
                    if (f.exists()) {
                        return true
                    }
                }
            } catch (e: Exception) {
//            LogUtils.d("root检测失败");
                e.printStackTrace()
            }

            return false
        }

    val bluetoothMac: String
        /**
         * 获取蓝牙MAC地址
         */
        get() {
            try {
                val bluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
                @SuppressLint("MissingPermission", "HardwareIds") val mac = bluetoothAdapter.address
                LogUtils.d("蓝牙 mac  ：  $mac")
                return mac
            } catch (e: Exception) {
//            LogUtils.d("蓝牙Mac地址获取失败");
                e.printStackTrace()
                return "nul"
            }
        }

    @get:SuppressLint("MissingPermission", "HardwareIds")
    val serial: String
        /**
         * pseudo-unique ID
         * API >=9:通过“Build.SERIAL”这个属性来保证ID的独一无二。
         * API 9 以上的Android设备目前市场占有率在99.5%
         */
        get() {
            try {
                return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P && checkSelfPermission(
                        Manifest.permission.READ_PHONE_STATE
                    )
                ) {
                    Build.getSerial()
                } else {
                    Build.SERIAL
                }
            } catch (e: Exception) {
//            LogUtils.d("pseudo-unique ID 获取失败");
                e.printStackTrace()
            }
            return "nul"
        }

    val cpuModel: String
        /**
         * 获取android CPU类型
         *
         * @return String CPU类型
         */
        get() {
            try {
                val cpu_model: String

                val `in` = cPUInfo

                cpu_model =
                    if ((`in`?.mCPUType?.and(CPUInfo.CPU_TYPE_ARMV5TE)) == CPUInfo.CPU_TYPE_ARMV5TE) "armv5"
                    else if ((`in`?.mCPUType?.and(CPUInfo.CPU_TYPE_ARMV6)) == CPUInfo.CPU_TYPE_ARMV6) "armv6"
                    else if ((`in`?.mCPUType?.and(CPUInfo.CPU_TYPE_ARMV7)) == CPUInfo.CPU_TYPE_ARMV7) "armv7"
                    else "unknown"
                return cpu_model
            } catch (e: Exception) {
//            LogUtils.d("CPU 类型获取失败");
                e.printStackTrace()
                return "nul"
            }
        }

    private val cPUInfo: CPUInfo?
        get() {
            var strInfo: String
            try {
                val bs = ByteArray(1024)
                val reader = RandomAccessFile("/proc/cpuinfo", "r")
                var ret = ""
                if (reader.read(bs) != -1) {
                    ret = String(bs, charset("UTF-8"))
                }
                val index = ret.indexOf(0.toChar())
                strInfo = if (index != -1) {
                    ret.substring(0, index)
                } else {
                    ret
                }
            } catch (ex: IOException) {
                strInfo = ""
                //            LogUtils.d("CPU info 获取失败");
                ex.printStackTrace()
            }

            return parseCPUInfo(strInfo)
        }

    private fun parseCPUInfo(cpuInfo: String?): CPUInfo? {
        if (cpuInfo == null || "" == cpuInfo) {
            return null
        }

        val ci = CPUInfo()
        ci.mCPUType = CPUInfo.CPU_TYPE_UNKNOWN
        ci.mCPUFeature = CPUInfo.CPU_FEATURE_UNKNOWS
        ci.mCPUCount = 1

        if (cpuInfo.contains("ARMv5")) {
            ci.mCPUType = CPUInfo.CPU_TYPE_ARMV5TE
        } else if (cpuInfo.contains("ARMv6")) {
            ci.mCPUType = CPUInfo.CPU_TYPE_ARMV6
        } else if (cpuInfo.contains("ARMv7")) {
            ci.mCPUType = CPUInfo.CPU_TYPE_ARMV7
        }

        if (cpuInfo.contains("neon")) {
            ci.mCPUFeature = ci.mCPUFeature or CPUInfo.CPU_FEATURE_NEON
        }

        if (cpuInfo.contains("vfpv3")) {
            ci.mCPUFeature = ci.mCPUFeature or CPUInfo.CPU_FEATURE_VFPV3
        }

        if (cpuInfo.contains(" vfp")) {
            ci.mCPUFeature = ci.mCPUFeature or CPUInfo.CPU_FEATURE_VFP
        }

        val items = cpuInfo.split("\n".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()

        for (item in items) {
            if (item.contains("CPU variant")) {
                val index = item.indexOf(": ")
                if (index >= 0) {
                    val value = item.substring(index + 2)
                    try {
                        ci.mCPUCount = Integer.decode(value)
                        ci.mCPUCount = if (ci.mCPUCount == 0) 1 else ci.mCPUCount
                    } catch (e: NumberFormatException) {
                        ci.mCPUCount = 1
                    }
                }
            }
        }

        return ci
    }

    val cPUSerial: String
        /**
         * 获取CPU序列号
         */
        get() {
            var str: String
            val strCPU: String
            var cpuAddress = "0000000000000000"
            var input: LineNumberReader? = null
            try {
                //读取CPU信息
                val pp = Runtime.getRuntime().exec("cat /proc/cpuinfo")
                input = LineNumberReader(
                    InputStreamReader(
                        pp.inputStream, "UTF-8"
                    )
                )
                //查找CPU序列号
                for (i in 1..99) {
                    str = input.readLine()
                    if (str != null) {
                        //查找到序列号所在行
                        if (str.contains("Serial")) {
                            //提取序列号
                            strCPU = str.substring(
                                str.indexOf(":") + 1, str.length
                            )
                            //去空格
                            cpuAddress = strCPU.trim { it <= ' ' }
                            break
                        }
                    } else {
                        //文件结尾
                        break
                    }
                }
            } catch (ex: Exception) {
                //赋予默认值
//            LogUtils.d("CPU 序列号 获取失败");
                ex.printStackTrace()
            } finally {
                FileUtils.close(input)
            }
            return cpuAddress
        }

    @JvmStatic
    fun getInfoByJson(key: String?): String {
        if (!TextUtils.isEmpty(Utils.mConfigInfo)) {
            try {
                val jsonObject = JSONObject(Utils.mConfigInfo)
                return jsonObject.optString(key)
            } catch (e: JSONException) {
                e.printStackTrace()
            }
        }
        return ""
    }

    @JvmStatic
    fun formatCurrentData(): String {
        try {
            val currentData = Date(System.currentTimeMillis())
            val simpleDateFormat = SimpleDateFormat("yyyyMMdd", Locale.CHINESE)
            return simpleDateFormat.format(currentData)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return ""
    }

    @JvmStatic
    val update: String
        get() {
            try {
                if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
                    return ""
                }
                if (unknown != sUpdataMark) {
                    return sUpdataMark.trim { it <= ' ' }
                }
                val str = ExeCommandUtils().run("cd /data/data;stat  .;", 10000).result
                val atime = str.substring(str.indexOf("Access:") + 7, str.lastIndexOf("Modify"))
                val atime1 = if (atime.contains("+")) {
                    atime.substring(atime.indexOf("Access:") + 7, atime.lastIndexOf("+"))
                } else {
                    atime.substring(atime.indexOf("Access:") + 7)
                }
                val atime3 =
                    atime1.split("\\.".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
                val result = dateToStamp(atime3[0]) + "." + atime3[1]
                sUpdataMark = result
                return result.trim { it <= ' ' }
            } catch (e: Throwable) {
                return ""
            }
        }

    @JvmStatic
    val boot: String
        get() {
            try {
                if ("0" == sBookMark) {
                    sBookMark =
                        ExeCommandUtils().run("cat /proc/sys/kernel/random/boot_id", 10000).result.toString()
                    if (sBookMark.contains("kernel")) {
                        sBookMark = ""
                        return sBookMark
                    }
                }
                return sBookMark.trim { it <= ' ' }
            } catch (e: Throwable) {
                e.printStackTrace()
            }
            return ""
        }

    @SuppressLint("SimpleDateFormat")
    private fun dateToStamp(s: String): String {
        try {
            val res: String
            val simpleDateFormat = SimpleDateFormat("yyyy-MM-dd HH:mm:ss")
            val date = simpleDateFormat.parse(s)
            val ts = date?.time
            res = (ts?.div(1000)).toString()
            return res
        } catch (_: ParseException) {
        }
        return ""
    }

    @JvmStatic
    val appStoreVersion: String
        get() {
            val pi: PackageInfo?
            var versionCode = -1
            var appStorePackagename = ""
            val vendor = vendor
            try {
                if (vendor == "Xiaomi") {
                    appStorePackagename = "com.xiaomi.market"
                } else if (vendor == "HUAWEI") {
                    appStorePackagename = "com.huawei.appmarket"
                } else if (vendor == "OPPO") {
                    appStorePackagename = if ((Build.VERSION.SDK_INT >= 29)) {
                        "com.heytap.market"
                    } else {
                        "com.oppo.market"
                    }
                } else if (vendor == "vivo") {
                    appStorePackagename = "com.bbk.appstore"
                } else if (vendor == "Lenovo") {
                    appStorePackagename = "com.lenovo.leos.appstore"
                } else if (vendor == "samsung") {
                    appStorePackagename = "com.sec.android.app.samsungapps"
                } else if (vendor == "zte") {
                    appStorePackagename = "zte.com.market"
                } else if (vendor == "OnePlus") {
                    appStorePackagename = "com.oneplus.market"
                }
                if (TextUtils.isEmpty(appStorePackagename)) return ""
                pi = context?.getPackageManager()?.getPackageInfo(appStorePackagename, 0)
                if (pi != null) {
                    versionCode = pi.versionCode
                }
            } catch (e: Throwable) {
                e.printStackTrace()
            }
            return versionCode.toString() + ""
        }

    private class CPUInfo {
        var mCPUType: Int = 0
        var mCPUCount: Int = 0
        var mCPUFeature: Int = 0

        companion object {
            const val CPU_TYPE_UNKNOWN: Int = 0x00000000
            const val CPU_TYPE_ARMV5TE: Int = 0x00000001
            const val CPU_TYPE_ARMV6: Int = 0x00000010
            const val CPU_TYPE_ARMV7: Int = 0x00000100
            const val CPU_FEATURE_UNKNOWS: Int = 0x00000000
            const val CPU_FEATURE_VFP: Int = 0x00000001
            const val CPU_FEATURE_VFPV3: Int = 0x00000010
            const val CPU_FEATURE_NEON: Int = 0x00000100
        }
    }
}
