package com.halomobi.ssp.base.utils

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.ApplicationInfo
import android.content.pm.PackageManager
import android.content.res.Configuration
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.drawable.AdaptiveIconDrawable
import android.graphics.drawable.BitmapDrawable
import android.net.ConnectivityManager
import android.net.Proxy
import android.net.Uri
import android.os.BatteryManager
import android.os.Build
import android.os.LocaleList
import android.os.Process
import android.telephony.TelephonyManager
import android.text.TextUtils
import android.view.Surface
import android.view.WindowManager
import androidx.core.content.FileProvider
import java.io.BufferedReader
import java.io.File
import java.io.FileReader
import java.io.IOException
import java.io.InputStreamReader
import java.math.BigDecimal
import java.net.NetworkInterface
import java.security.MessageDigest
import java.util.Calendar
import java.util.Collections
import java.util.Date
import java.util.Locale

/**
 * Created on 2017/6/19.
 */
object ToolsUtils {
    private const val TYPE_NONE = 0
    private const val TYPE_UNKNOW = 1
    private const val TYPE_2G = 2
    private const val TYPE_3G = 3
    private const val TYPE_4G = 4
    private const val TYPE_WIFI = 10
    private const val LOCAL_USER_ID_KEY = "u_id"
    private const val LOCAL_DEVICE_ID_KEY = "d_id"
    var IDFA: String? = null
    private val device_id: String? = null
    private const val user_id = ""

    /**
     * 当参数时null 是 返回 false
     *
     * @param context     上下文
     * @param packageName 包名
     * @return true:已经安装对应包名的应用;false:未安装对应包名的应用
     */
    fun isAppInstalled(context: Context, packageName: String): Boolean {
        try {
            context.packageManager.getPackageInfo(packageName, 0)
            return true
        } catch (e: Exception) {
        }
        return false
    }


    /**
     * 获取应用签名文件的MD5
     * 当context 是空时返回""
     *
     * @param context 上下文
     * @return 应用签名文件的MD5串
     */
    fun getSignature(context: Context): String {
        try {
            val packageInfo = context.packageManager.getPackageInfo(
                context.packageName,
                PackageManager.GET_SIGNATURES
            )
            val signs = packageInfo.signatures
            return getMD5(signs[0].toCharsString())
        } catch (e: Exception) {
        }
        return ""
    }

    /**
     * 获取对应内容的md5
     * 当data为空时,或者出现异常时返回 空串
     *
     * @param data 要获取MD5 的字符串
     * @return 返回对用内容的MD5串
     */
    fun getMD5(data: String): String {
        try {
            val md = MessageDigest.getInstance("MD5")
            md.update(data.toByteArray())
            val buf = StringBuffer()
            val bits = md.digest()
            for (i in bits.indices) {
                var a = bits[i].toInt()
                if (a < 0) a += 256
                if (a < 16) buf.append("0")
                buf.append(Integer.toHexString(a))
            }
            return buf.toString()
        } catch (e: Exception) {
        }
        return ""
    }

    /**
     * 第一次安装时间
     * 当context 为空时,或其他异常,未达到安装时间时,返回当前时间戳
     *
     * @param context 上下文
     * @return 返回安装时间 单位ms
     */
    fun getFirstInstallTime(context: Context): Long {
        var installTime = System.currentTimeMillis()
        try {
            val info = context.packageManager.getPackageInfo(context.packageName, 0)
            installTime = info.firstInstallTime
        } catch (e: Exception) {
        }
        return installTime
    }

    /**
     * 打开 其它app
     * context 或者 packageName 为null 时,不进行操作
     * 当应用内不存在对应包名,不进行操作
     *
     * @param context     上下文
     * @param packageName 要打开app 的包名
     */
    fun openApp(context: Context?, packageName: String?) {
        if (context == null || packageName == null) return
        val intent = context.packageManager.getLaunchIntentForPackage(packageName) ?: return
        context.startActivity(intent)
    }

    /**
     * context 为null 不跳转
     * 当url 不合规时,不做跳转
     *
     * @param context
     * @param url     跳转到应用外 打开浏览器对应的URL的页面
     */
    fun gotoUrl(context: Context?, url: String?) {
        if (context == null) return
        try {
            val intent = Intent(Intent.ACTION_VIEW, Uri.parse(url))
            context.startActivity(intent)
        } catch (e: Exception) {
            //
        }
    }

    /**
     * 判断当前是否有挂代理
     * context == null 时 返回false
     *
     * @param context 上下文
     * @return 未挂代理 返回false;挂代理返回true
     */
    fun hasSetProxy(context: Context?): Boolean {
        if (context == null) {
            return false
        }
        val is_ics_or_later = Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH
        val proxyAddress: String?
        val proxyPort: Int
        if (is_ics_or_later) {
            proxyAddress = System.getProperty("http.proxyHost")
            val portStr = System.getProperty("http.proxyPort")
            proxyPort = (portStr ?: "-1").toInt()
        } else {
            proxyAddress = Proxy.getHost(context)
            proxyPort = Proxy.getPort(context)
        }
        return (!TextUtils.isEmpty(proxyAddress)) && (proxyPort != -1)
    }


    /**
     * 如果context 是null,不进行分享
     * 或者为针对7.0以上版本进行相关配置,则无法分享
     * 分享当前应用 android 7.0 以上与以下走不通获取uri逻辑
     *
     * @param context 上下文
     */
    fun shareApk(context: Context) {
        try {
            val pm = context.packageManager
            val info = pm.getApplicationInfo(context.packageName, 0)
            val apkFile = File(info.sourceDir)
            val intent = Intent()
            intent.setAction(Intent.ACTION_SEND)
            intent.setType("*/*")
            val fileUri = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                FileProvider.getUriForFile(
                    context,
                    context.applicationContext.packageName + ".provider",
                    apkFile
                )
            } else {
                Uri.fromFile(apkFile)
            }
            intent.putExtra(Intent.EXTRA_STREAM, fileUri)
            context.startActivity(intent)
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    /**
     * 手机是否正在充电
     * 如果context 是null 时返回 false
     *
     * @param context 上下文
     * @return true: 正在充电; false:没有在充电
     */
    fun isPowerConnected(context: Context?): Boolean {
        if (context == null) return false
        val localIntentFilter = IntentFilter(Intent.ACTION_BATTERY_CHANGED)
        val i = context.registerReceiver(null, localIntentFilter)?.getIntExtra("status", -1)
        return (i == BatteryManager.BATTERY_STATUS_CHARGING) || (i == BatteryManager.BATTERY_STATUS_FULL)
    }

    /**
     * 手机网络是否可用
     * 如果context 是null 时返回 false
     *
     * @param context 上下文
     * @return false:网络不可用 true:联网
     */
    fun isNetworkAvailable(context: Context?): Boolean {
        if (context == null) return false
        val connectivityManager =
            context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
                ?: return false
        val networkInfo = connectivityManager.activeNetworkInfo
        if (networkInfo == null || !networkInfo.isAvailable) return false
        return true
    }

    val macAddress: String?
        /**
         * 网络接口获取mac地址
         * 当设备没有网网路接口的设备返回 null;
         * android 系统限制不让获取Mac 地址 返回"00000000000"
         *
         * @return mac 地址
         */
        get() {
            try {
                val all: List<NetworkInterface> =
                    Collections.list(NetworkInterface.getNetworkInterfaces())
                for (nif in all) {
                    if (!nif.name.equals("wlan0", ignoreCase = true)) continue

                    val macBytes = nif.hardwareAddress ?: return null
                    val res1 = StringBuilder()
                    for (b in macBytes) {
                        res1.append(String.format("%02X:", b))
                    }
                    if (res1.length > 0) {
                        res1.deleteCharAt(res1.length - 1)
                    }
                    return res1.toString()
                }
            } catch (ex: Exception) {
            }
            return "00000000000"
        }

    /**
     * 获取网络类型
     * 当context == null 时,返回 0
     *
     * @param context
     * @return 返回网路状态, 没有网络或者获取失败 返回0;默认返回 1; 2G 返回2; 3G 返回3; 4G 返回4; wifi 返回10;
     */
    fun getNetWorkType(context: Activity?): Int {
        if (context == null) {
            return 0
        }
        val manager = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        if (manager != null) {
            val networkInfo = manager.activeNetworkInfo
            if (networkInfo != null && networkInfo.isConnected) {
                val type = networkInfo.typeName
                if (type.equals("WIFI", ignoreCase = true)) {
                    return TYPE_WIFI
                } else if (type.equals("MOBILE", ignoreCase = true)) {
                    val mobileInfo = manager.getNetworkInfo(ConnectivityManager.TYPE_MOBILE)
                    if (mobileInfo != null) {
                        when (mobileInfo.type) {
                            ConnectivityManager.TYPE_MOBILE -> return when (mobileInfo.subtype) {
                                TelephonyManager.NETWORK_TYPE_UMTS, TelephonyManager.NETWORK_TYPE_EVDO_0, TelephonyManager.NETWORK_TYPE_EVDO_A, TelephonyManager.NETWORK_TYPE_HSDPA, TelephonyManager.NETWORK_TYPE_HSUPA, TelephonyManager.NETWORK_TYPE_HSPA, TelephonyManager.NETWORK_TYPE_EVDO_B, TelephonyManager.NETWORK_TYPE_EHRPD, TelephonyManager.NETWORK_TYPE_HSPAP -> TYPE_3G
                                TelephonyManager.NETWORK_TYPE_CDMA, TelephonyManager.NETWORK_TYPE_GPRS, TelephonyManager.NETWORK_TYPE_EDGE, TelephonyManager.NETWORK_TYPE_1xRTT, TelephonyManager.NETWORK_TYPE_IDEN -> TYPE_2G
                                TelephonyManager.NETWORK_TYPE_LTE -> TYPE_4G
                                else -> TYPE_UNKNOW
                            }
                        }
                    }
                }
            }
        }
        return TYPE_NONE
    }


    /**
     * 是不是通过debug形式构建的apk
     * context == null 返回false
     *
     * @param context 上下文
     * @return false 是release 包,true 是debug 包
     */
    fun isDebuggable(context: Context?): Boolean {
        if (context == null) return false
        val applicationInfo = context.applicationInfo
        return (applicationInfo.flags and ApplicationInfo.FLAG_DEBUGGABLE) != 0
    }


    /**
     * 获取当前进程名
     * context == null 返回 当前主进程
     *
     * @param context
     * @return
     */
    fun getCurrentProcessName(context: Context): String {
        try {
            val file = File("/proc/" + Process.myPid() + "/" + "cmdline")
            val mBufferedReader = BufferedReader(FileReader(file))
            val processName = mBufferedReader.readLine().trim { it <= ' ' }
            mBufferedReader.close()
            return processName
        } catch (e: Exception) {
            return context.packageName
        }
    }

    /**
     * 是否匹配进程,判断是否在指定的进程中
     * processName == null ,匹配不成功
     * context == null 会那主进程与当前processName 进行匹配
     *
     * @param context
     * @param processName 需要匹配的进程名称
     * @return true:当前进程是processName 进程,false:当前进程不是processName 进程
     */
    fun inProcess(context: Context, processName: String): Boolean {
        val name = getCurrentProcessName(context)
        if (name == processName) {
            return true
        }
        return false
    }

    val sysVersion: String
        /**
         * 获取系统版本
         *
         * @return 系统版本 如果 8.1.0;3.4b5;
         */
        get() = Build.VERSION.RELEASE

    val phoneMode: String
        /**
         * 获取手机版本
         *
         * @return "vivo X20Plus A"
         */
        get() = Build.MODEL

    val sysCountry: String
        /**
         * 获取系统本地国家编码
         *
         * @return ISO 3166 2-letter code 国家代码 "zh"
         */
        get() = Locale.getDefault().country

    val sysLanguage: String
        /**
         * 获取手机系统语言
         *
         * @return ISO 639 规则 "zhs","zht"
         */
        get() {
            var lan = Locale.getDefault().language
            val country = Locale.getDefault().country
            if (lan == "zh") {
                lan = if (country == "CN") {
                    lan + "s"
                } else {
                    lan + "t"
                }
            }
            return lan
        }

    val systemLanguage: String
        /**
         * 获取手机系统语言（和TGA源码一致）
         * @return
         */
        get() {
            val locale = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                LocaleList.getDefault()[0]
            } else Locale.getDefault()

            return locale.language
        }


    /**
     * 获取状态栏高度
     * context = null 返回 0
     *
     * @param context
     * @return 状态栏的高度 像素
     */
    @SuppressLint("InternalInsetResource")
    fun getStatusBarHeight(context: Context?): Int {
        if (context == null) {
            return 0
        }
        var result = 0
        val resourceId = context.resources.getIdentifier("status_bar_height", "dimen", "android")
        if (resourceId > 0) {
            result = context.resources.getDimensionPixelSize(resourceId)
        }
        return result
    }

    /**
     * 获取手机旋转的角度
     * context = null 返回 0
     *
     * @param activity
     * @return 旋转 0°,90°,180°,270° 分别对应 0,1,2,3
     */
    fun getRotaionMessage(activity: Activity?): Int {
        if (activity == null) {
            return 0
        }
        val angle =
            (activity.getSystemService(Context.WINDOW_SERVICE) as WindowManager).defaultDisplay.rotation
        return when (angle) {
            Surface.ROTATION_0 -> 0
            Surface.ROTATION_90 -> 1
            Surface.ROTATION_180 -> 2
            Surface.ROTATION_270 -> 3
            else -> 0
        }
    }

    /**
     * 判断类存在不存在
     * clazz = null 是 返回false
     *
     * @return true:存在  false:不存在
     */
    fun canGetClass(clazz: String): Boolean {
        var b = false
        try {
            Class.forName(clazz)
            b = true
        } catch (e: Exception) {
        }
        return b
    }

    /**
     * 获取App 的名字
     * context == null 返回 " "
     *
     * @return 返回appName
     */
    @Synchronized
    fun getAppName(context: Context): String {
        try {
            val packageManager = context.packageManager
            val packageInfo = packageManager.getPackageInfo(
                context.packageName, 0
            )
            val labelRes = packageInfo.applicationInfo.labelRes
            return context.resources.getString(labelRes)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return " "
    }

    /**
     * * 根据手机的分辨率从 dp 的单位 转成为 px(像素)
     * context == null 返回 0
     *
     * @param context
     * @param dpValue
     * @return 对应的px值
     */
    fun dp2px(context: Context?, dpValue: Float): Int {
        if (context == null) {
            return 0
        }
        val scale = context.resources.displayMetrics.density
        return (dpValue * scale + 0.5f).toInt()
    }

    /**
     * 根据手机的分辨率从 px(像素) 的单位 转成为 dp
     *
     * @param context
     * @param pxValue
     * @return
     */
    fun px2dp(context: Context, pxValue: Float): Int {
        val scale = context.resources.displayMetrics.density
        return (pxValue / scale + 0.5f).toInt()
    }

    /**
     * 判断手机现在屏幕是都是竖屏
     *
     * @param activity
     * @return true 是竖屏,false 是横屏
     */
    fun isPortrait(activity: Activity?): Boolean {
        if (activity == null) {
            return true
        }
        val configuration = activity.resources.configuration //获取设置的配置信息
        return configuration.orientation == Configuration.ORIENTATION_PORTRAIT
    }

    /**
     * 获取sdk 插件版本
     * 当activity == null 时返回 1.0.0
     *
     * @param activity
     * @return 返回引入插件的版本
     */
    @JvmStatic
    fun getPluginVersion(activity: Context): String {
        var puginVersion = try {
            activity.packageManager
                .getApplicationInfo(activity.packageName, PackageManager.GET_META_DATA)
                .metaData.getString("vvvvvvvv", "1.0.0")
        } catch (e: Exception) {
            "1.0.0"
        }
        return puginVersion
    }

    /**
     * 获取sdk 的版本号
     * 当activity == null 时返回 1
     *
     * @param context
     * @return 返回插件的版本
     */
    fun getPluginVersionCode(context: Context): Int {
        var pluginVersionCode = try {
            context.packageManager
                .getApplicationInfo(context.packageName, PackageManager.GET_META_DATA)
                .metaData.getInt("cccccccc", 1)
        } catch (e: Exception) {
            1
        }
        return pluginVersionCode
    }

    fun getConfigUrl(context: Context, name: String?): String {
        var url = try {
            context.packageManager
                .getApplicationInfo(context.packageName, PackageManager.GET_META_DATA)
                .metaData.getString(name, "")
        } catch (e: Exception) {
            ""
        }
        return url
    }

    fun <T> getMetaData(context: Context, name: String?): T? {
        try {
            val ai = context.packageManager.getApplicationInfo(
                context.packageName,
                PackageManager.GET_META_DATA
            )

            if (ai.metaData != null) {
                return ai.metaData[name] as T?
            }
        } catch (e: Exception) {
        }
        return null
    }


    /**
     * 获取项目的版本号
     * 当activity == null 时返回 ""
     *
     * @param activity
     * @return 返回项目的版本号 例如:"1.2.6"
     */
    fun getGameVersionCode(activity: Context?): String {
        var version = ""
        if (activity != null) {
            val packageName = activity.packageName
            //版本
            try {
                val pi = activity.packageManager.getPackageInfo(packageName, 0)
                version = pi.versionName
            } catch (e: Exception) {
            }
        }
        return version
    }

    /**
     * 获取项目的build号
     * activity == null 返回0
     *
     * @param activity
     * @return 返回build号
     */
    fun getGameBuildCode(activity: Context): Int {
        var version = 0
        val packageName = activity.packageName
        //版本
        try {
            val pi = activity.packageManager.getPackageInfo(packageName, 0)
            version = pi.versionCode
        } catch (e: Exception) {
        }
        return version
    }

    /**
     * 获取两个时间间隔的时差 天
     * 当start == 0 是 返回 0
     * 当 start 或 long 小于等于0  代表日期是1970-01-01
     *
     * @param start 开始时间戳 ms
     * @param end   截止时间戳
     * @return 两个时间戳转换成日期, 相隔的天数
     */
    fun getDataGapCount(start: Long, end: Long): Int {
        if (start == 0L) {
            return 1
        }
        val startDate = Date(start)
        val endDate = Date(end)
        val fromCalendar = Calendar.getInstance()
        fromCalendar.time = startDate
        fromCalendar[Calendar.HOUR_OF_DAY] = 0
        fromCalendar[Calendar.MINUTE] = 0
        fromCalendar[Calendar.SECOND] = 0
        fromCalendar[Calendar.MILLISECOND] = 0
        val toCalendar = Calendar.getInstance()
        toCalendar.time = endDate
        toCalendar[Calendar.HOUR_OF_DAY] = 0
        toCalendar[Calendar.MINUTE] = 0
        toCalendar[Calendar.SECOND] = 0
        toCalendar[Calendar.MILLISECOND] = 0
        return ((toCalendar.time.time - fromCalendar.time.time) / (1000 * 60 * 60 * 24)).toInt()
    }


    /**
     * 清单文件中是否包含某一个字段
     * activity 或  str 为空时返回的是 false
     *
     * @param activity
     * @param str
     * @return true:包含字段; false:不包含字段
     */
    fun getMetaData(activity: Activity, str: String?): Boolean {
        var appInfo: ApplicationInfo? = null
        try {
            appInfo = activity.packageManager.getApplicationInfo(
                activity.packageName,
                PackageManager.GET_META_DATA
            )
            val msg = appInfo.metaData.getString(str)
            if (!TextUtils.isEmpty(msg)) {
                return true
            }
        } catch (e: PackageManager.NameNotFoundException) {
            e.printStackTrace()
        }
        return false
    }

    /**
     * 获取游戏的icon 图标
     * 当activity == null 时,返回 null
     * adnroid 8.0 有AdaptiveIconDrawable
     *
     * @param activity
     * @return icon对用的Bitmap对象
     */
    fun getIconBitmap(activity: Activity): Bitmap? {
        try {
            val icon = activity.applicationInfo.loadIcon(activity.packageManager) ?: return null
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O && icon is AdaptiveIconDrawable) {
                val bitmap = Bitmap.createBitmap(
                    icon.getIntrinsicWidth(),
                    icon.getIntrinsicHeight(),
                    Bitmap.Config.ARGB_8888
                )
                val canvas = Canvas(bitmap)
                icon.setBounds(0, 0, canvas.width, canvas.height)
                icon.draw(canvas)
                return bitmap
            } else {
                return (icon as BitmapDrawable).bitmap
            }
        } catch (e: Exception) {
            return null
        }
    }

    fun double2float(d: Double): Float {
        val b = BigDecimal(d.toString())
        return b.toFloat()
    }

    fun float2double(f: Float): Double {
        val b = BigDecimal(f.toString())
        return b.toDouble()
    }


    //获取启动activity的类全名称
    fun getLaunchClassName(context: Context): String {
        try {
            val launchIntentForPackage =
                context.packageManager.getLaunchIntentForPackage(context.packageName)
            val component = launchIntentForPackage?.component
            return component?.className.toString()
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return ""
    }

    val isHarmonyOS: Boolean
        /**
         * 是否为鸿蒙系统
         *
         * @return true为鸿蒙系统
         */
        get() = !TextUtils.isEmpty(exec("getprop hw_sc.build.platform.version"))

    /**
     * 执行命令获取对应内容
     *
     * @param command 命令
     * @return 命令返回内容
     */
    fun exec(command: String?): String? {
        var ir: InputStreamReader? = null
        var input: BufferedReader? = null
        try {
            val process = Runtime.getRuntime().exec(command)
            ir = InputStreamReader(process.inputStream)
            input = BufferedReader(ir)
            var line: String?
            val stringBuilder = StringBuilder()
            while ((input.readLine().also { line = it }) != null) {
                stringBuilder.append(line)
            }
            return stringBuilder.toString()
        } catch (e: Throwable) {
        } finally {
            if (input != null) {
                try {
                    input.close()
                } catch (e: Throwable) {
                }
            }
            if (ir != null) {
                try {
                    ir.close()
                } catch (e: IOException) {
                }
            }
        }
        return null
    }

    val isFireOS: Boolean
        /**
         * 判断手机是是不是amazon 设备
         */
        get() = phoneBrand.contains("amazon") || phoneBrand.contains("Amazon")

    val deviceOS: String
        //获取操作系统标识
        get() {
            if (isHarmonyOS) {
                return "Harmony"
            } else if (isFireOS) {
                return "Amazon"
            }
            return "Android"
        }

    val phoneBrand: String
        get() {
            val manufacturer = Build.MANUFACTURER
            return if (manufacturer != null && manufacturer.length > 0) {
                manufacturer.toLowerCase()
            } else {
                "unknown"
            }
        }


    val currentTimeZone: Int
        /**
         * 获取当前时区
         * +0800        SimpleDateFormat("Z").format(Calendar.getInstance(TimeZone.getTimeZone("GMT"),Locale.getDefault()).time)
         * GMT+08:00    TimeZone.getDefault().getDisplayName(false, TimeZone.SHORT)——会存在PST、WIB直接别名的时区，后端不好过滤，赞、暂不采用
         * GMT+08:00    参考TimeZone方法createGmtOffsetString自定义方式实现
         *
         * @return GMT+08:00
         */
        get() {
            val cal =
                Calendar.getInstance(Locale.getDefault())

            val zoneOffset = cal[Calendar.ZONE_OFFSET]

            val zone = zoneOffset / 60 / 60 / 1000 // 时区，东时区数字为正，西时区为负
            return zone
        }

}
