package com.halomobi.ssp.base.utils

import android.annotation.SuppressLint
import android.app.Activity
import android.app.AppOpsManager
import android.content.ClipboardManager
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.content.res.Configuration
import android.content.res.Resources
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Rect
import android.graphics.drawable.AdaptiveIconDrawable
import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.Drawable
import android.graphics.drawable.LayerDrawable
import android.net.Uri
import android.os.Build
import android.os.Handler
import android.os.Looper.getMainLooper
import android.text.TextUtils
import android.util.DisplayMetrics
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import android.widget.Toast
import androidx.core.app.NotificationManagerCompat
import com.github.gzuliyujiang.oaid.DeviceIdentifier
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.framework.Abstract.AbstractInitAd
import com.halomobi.ssp.framework.schedule.ThreadManager.shortPool
import com.halomobi.ssp.framework.schedule.ThreadManager.singlePool
import com.halomobi.ssp.model.request.config.SDKInfo
import org.json.JSONObject
import java.io.BufferedInputStream
import java.io.BufferedReader
import java.io.InputStream
import java.io.InputStreamReader
import java.net.URL
import java.util.zip.GZIPInputStream
import java.util.zip.InflaterInputStream
import kotlin.math.max
import kotlin.math.min

@SuppressLint("StaticFieldLeak")
object Utils {
    private const val SP_KEY_NAME = SDKInfo.ADTAG + "_info"

    @get:Synchronized
    var uiHandler: Handler? = null
        private set
    var context: Context? = null
        private set
    private var lastClickTime: Long = 0

    /**
     * @param debug
     */
    private const val clipText = "c372e6227d4a89c55d36addf72c7ea3f"

    var isDelayed = true

    @JvmField
    var mConfigInfo: String = ""

    @Synchronized
    fun init(cx: Context, configInfo: String) {
        context = cx.applicationContext
        mConfigInfo = configInfo
    }

    @Synchronized
    fun init(cx: Context, isDebug: Boolean, abstractInitAd: AbstractInitAd) {
        context = cx.applicationContext
        if (uiHandler == null) {
            uiHandler = Handler(getMainLooper())
        }
        shortPool.execute {
            try {
                if (SpUtil.getString(context, "initConfig").isNotEmpty()) {
                    mConfigInfo = SpUtil.getString(context, "initConfig")
                    isDelayed = false
                } else {
                    isDelayed = true
                    // 要延迟执行的代码
                    Thread.sleep(1000) // 延迟1000毫秒（1秒）
                    val jsonObject = JSONObject()
                    if (TextUtils.isEmpty(abstractInitAd.deviceOaid)) { //判断如果传递oaid为空 sdk内自动获取
                        jsonObject.put("oaId", DeviceIdentifier.getOAID(context)) //当前设备oaId
                    } else {
                        jsonObject.put("oaId", abstractInitAd.deviceOaid) //当前设备oaId
                    }
                    jsonObject.put("did", DeviceIdentifier.getIMEI(context)) //当前设备imei
                    jsonObject.put("dpid", DeviceIdentifier.getAndroidID(context)) //当前设备androidId
                    jsonObject.put("mac", SystemUtils.getMacAddress(context)) //当前设备的mac地址
                    mConfigInfo = jsonObject.toString()
                    SpUtil.putString(context, "initConfig", mConfigInfo)
                }
            } catch (exception: Exception) {
                exception.printStackTrace()
            }
        }
        LogUtils.setDeBugModel(if (isDebug) LogUtils.LEVEL_ERROR else LogUtils.LEVEL_NONE)
    }

    val clipBoard: Boolean
        get() {
            try {
                val cm =
                    context?.getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
                val data = cm.primaryClip ?: return false
                val item = data.getItemAt(0)
                val content = item.text.toString()
                if (TextUtils.equals(clipText, content)) {
                    return true
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            return false
        }


    @JvmStatic
    val sp: SharedPreferences?
        get() = context?.getSharedPreferences(
            SP_KEY_NAME,
            0
        )

    fun getSp(context: Context): SharedPreferences {
        return context.getSharedPreferences(SP_KEY_NAME, 0)
    }

    @JvmStatic
    fun runInMainThread(runnable: Runnable) {
        if (isRunInMainThread) {
            runnable.run()
        } else {
            post(runnable)
        }
    }

    @JvmStatic
    fun post(runnable: Runnable) {
        uiHandler?.post(runnable)
    }

    fun postDelayed(runnable: Runnable, delay: Long) {
        uiHandler?.postDelayed(runnable, delay)
    }

    fun removeCallbacks(runnable: Runnable) {
        uiHandler?.removeCallbacks(runnable)
    }

    private val isRunInMainThread: Boolean
        get() = Thread.currentThread().id == getMainLooper().thread
            .id

    @JvmStatic
    val resources: Resources?
        get() = context?.resources

    @JvmStatic
    fun showToastSafe(text: String?) {
        runInMainThread(Runnable {
            Toast.makeText(
                context,
                text,
                Toast.LENGTH_SHORT
            ).show()
        })
    }

    /**
     * 获取屏幕尺寸信息
     *
     * @return
     */
    @Suppress("deprecation")
    fun getScreenSize(mContext: Context?, macthScreen: Boolean): IntArray {
        var cx = mContext
        try {
            if (cx == null) {
                cx = context
            }

            val dm: DisplayMetrics? = cx?.resources?.displayMetrics
            val wm = cx?.getSystemService(
                Context.WINDOW_SERVICE
            ) as WindowManager
            val display = wm.defaultDisplay

            return if (!macthScreen || isPortrait) {
                intArrayOf(dm?.widthPixels ?: display.width, dm?.heightPixels ?: display.height)
            } else {
                intArrayOf(dm?.heightPixels?:display.height, dm?.widthPixels?:display.width)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return intArrayOf(0, 0)
    }

    /**
     * 获取屏幕尺寸信息
     *
     * @param matchScreen 是否自动适应屏幕
     * @return
     */
    @JvmStatic
    fun getScreenSize(matchScreen: Boolean): IntArray {
        return getScreenSize(null, matchScreen)
    }

    val isPortrait: Boolean
        /**
         * 是否是竖屏状态
         *
         * @return
         */
        get() {
            val mConfiguration =
                context?.resources
                    ?.configuration // 获取设置的配置信息
            val ori = mConfiguration?.orientation // 获取屏幕方向

            if (ori == Configuration.ORIENTATION_LANDSCAPE) {
                // 横屏
                return false
            } else if (ori == Configuration.ORIENTATION_PORTRAIT) {
                // 竖屏
                return true
            }
            return false
        }

    @JvmStatic
    val isFastDoubleClick: Boolean
        get() {
            val time = System.currentTimeMillis()
            val timeD = time - lastClickTime
            if (timeD in 1..499) {
                return true
            }
            lastClickTime = time
            return false
        }
    var adViewArea: Float? = null

    /**
     * 检测View是否被遮挡
     *
     * @param view       被检测视图
     * @param shieldRate 遮挡率
     * @return true 遮挡 false 未遮挡
     */
    @SuppressLint("UseKtx")
    @JvmStatic
    fun isViewCovered(view: View?, shieldRate: Float): Boolean {
        var shieldRate = shieldRate
        try {
            shieldRate = if (shieldRate < 0) 0f else if (shieldRate > 1) 1f else shieldRate
            adViewArea = (view?.measuredHeight?.times(view.measuredWidth))?.toFloat()
            var currentView = view
            val currentViewRect = Rect()
            val onScreenVisible = currentView?.getGlobalVisibleRect(currentViewRect)
            //            LogUtils.d("currentViewRect:" + currentViewRect.toShortString());
            if (onScreenVisible != true) {
                return true
            }

            val visibleArea = ((currentViewRect.bottom - currentViewRect.top)
                    * (currentViewRect.right - currentViewRect.left)).toFloat()
            if ((adViewArea?.let { visibleArea.div(it) })?.compareTo(shieldRate) == -1) {
                return true
            }

            while (currentView?.parent is ViewGroup) {
                val currentParent = currentView.parent as ViewGroup
                if (currentParent.visibility != View.VISIBLE) {
                    return true
                }
                var start = 0
                while (start < currentParent.childCount) {
                    if (currentParent.getChildAt(start) === currentView) break
                    start++
                }

                for (i in start + 1 until currentParent.childCount) {
                    val viewRect = Rect()
                    view?.getGlobalVisibleRect(viewRect)
                    val otherView = currentParent.getChildAt(i)
                    val otherViewRect = Rect()
                    otherView.getGlobalVisibleRect(otherViewRect)
                    if (otherView.visibility == View.VISIBLE
                        && Rect.intersects(viewRect, otherViewRect)
                    ) {
                        if (otherView.background != null
                            || otherView.isClickable
                        ) {
                            var mixArea = 0f
                            val left = max(
                                viewRect.left.toDouble(),
                                otherViewRect.left.toDouble()
                            ).toInt()
                            val top =
                                max(viewRect.top.toDouble(), otherViewRect.top.toDouble()).toInt()
                            val right = min(
                                viewRect.right.toDouble(),
                                otherViewRect.right.toDouble()
                            ).toInt()
                            val bottom = min(
                                viewRect.bottom.toDouble(),
                                otherViewRect.bottom.toDouble()
                            ).toInt()
                            if (left < right && top < bottom) {
                                mixArea = ((right - left) * (bottom - top)).toFloat()
                            }
                            val f = adViewArea?.let { mixArea.div(it) }
                            if (f?.compareTo(shieldRate) == 1) {
                                return true
                            }
                        }
                    }
                }
                currentView = currentParent
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return false
    }

    @JvmStatic
    fun removeSelf(adView: View?) {
        if (adView != null) {
            val parent = adView.parent
            if (parent != null && parent is ViewGroup) {
                parent.removeView(adView)
            }
        }
    }

    @JvmStatic
    val sDKVer: String
        get() = SDKInfo.SDK_VERSION

    val pkgName: String
        /**
         * 获取SDK包名
         *
         * @return sdk包名
         */
        get() {
            try {
                return Utils::class.java.name.replace("\\.base.+".toRegex(), "")
            } catch (e: Exception) {
            }
            return ""
        }

    /**
     * 检测距离上次发送是否已超过设定时间
     *
     * @param key          SP key
     * @param value        设定的值
     * @param defaultValue 默认值
     * @return true 可以发送  false 不发送
     */
    fun checkIntervalTime(key: String?, value: Long, defaultValue: Long): Boolean {
        return System.currentTimeMillis() - (sp?.getLong(key, defaultValue)
            ?: defaultValue) >= value
    }

    fun decodeInputStream(inputStream: InputStream?, contentEncoding: String): BufferedInputStream {
        return if (TextUtils.isEmpty(contentEncoding)) {
            BufferedInputStream(inputStream)
        } else if (contentEncoding.toLowerCase() == "gzip") {
            BufferedInputStream(GZIPInputStream(inputStream))
        } else if (contentEncoding.toLowerCase() == "deflate") {
            BufferedInputStream(InflaterInputStream(inputStream))
        } else {
            BufferedInputStream(inputStream)
        }
    }

    fun parseInputString(inputStream: BufferedInputStream?): String? {
        var response: String? = null
        if (inputStream != null) {
            var reader: BufferedReader? = null
            try {
                val sb = StringBuilder()
                reader = BufferedReader(
                    InputStreamReader(
                        inputStream, "UTF-8"
                    )
                )
                var line: String? = null
                while ((reader.readLine().also { line = it }) != null) {
                    sb.append(line)
                }

                response = String(sb)
            } finally {
                FileUtils.close(inputStream)
                FileUtils.close(reader)
            }
        }
        return response
    }

    @JvmStatic
    fun Drawable2BitMap(drawable: Drawable): Bitmap? {
        try {
            if (Build.VERSION.SDK_INT >= 26) {
                if (drawable is BitmapDrawable) {
                    return drawable.bitmap
                } else if (drawable is AdaptiveIconDrawable) {
                    val backgroundDr = drawable.background
                    val foregroundDr = drawable.foreground

                    val drr = arrayOfNulls<Drawable>(2)
                    drr[0] = backgroundDr
                    drr[1] = foregroundDr

                    val layerDrawable = LayerDrawable(drr)

                    val width = layerDrawable.intrinsicWidth
                    val height = layerDrawable.intrinsicHeight

                    val bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888)

                    val canvas = Canvas(bitmap)

                    layerDrawable.setBounds(0, 0, canvas.width, canvas.height)
                    layerDrawable.draw(canvas)

                    return bitmap
                }
            } else {
                (drawable as BitmapDrawable).bitmap
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return null
    }

    /**
     * 隐藏底部虚拟按键
     */
    fun hideNavKey(context: Context) {
        try {
            //for new api versions.
            val decorView = (context as Activity).window.decorView
            val uiOptions = (View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    or View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY)
            decorView.systemUiVisibility = uiOptions
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    /**
     * 显示虚拟按键
     *
     * @param context
     */
    fun showNavKey(context: Context) { //getWindow().getDecorView().getSystemUiVisibility() 传入0也可以
        try {
            (context as Activity).window.decorView.systemUiVisibility = 0
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    var isCacheSuccess: Boolean = false

    @JvmStatic
    fun preCache(preUrl: String?): Boolean {
        singlePool.execute(Runnable {
            try {
                var inputStream: InputStream? = null
                val url = URL(preUrl)
                inputStream = url.openStream()
                val bufferSize = 1024
                val buffer = ByteArray(bufferSize)
                var length = 0
                //                    LogUtils.d("RewardVideo cache1:");
                while ((inputStream.read(buffer).also { length = it }) != -1) {
                    //Since we just need to kick start the prefetching, dont need to do anything here
                    //  or we can use ByteArrayOutputStream to write down the data to disk
                    isCacheSuccess = true
                }
                //                    LogUtils.d("RewardVideo cache2:");
            } catch (e: Exception) {
                isCacheSuccess = false
                //                    LogUtils.d("RewardVideo cacheFailed:" + e.getMessage());
                e.printStackTrace()
            }
        })
        return isCacheSuccess
    }

    @SuppressLint("ApplySharedPref")
    fun putSpInt(type: Int) {
        val sharedPreferences = sp
        val editor = sharedPreferences?.edit()
        editor?.putInt(ConstantPool.AD_TYPE, type)
        editor?.apply()
    }


    @SuppressLint("ObsoleteSdkInt")
    @JvmStatic
    fun areNotificationsEnabled(): Boolean? {
        if (context == null) {
            return true;
        }
        if (Build.VERSION.SDK_INT >= 24) {
            return context?.let { NotificationManagerCompat.from(it).areNotificationsEnabled() }
        } else if (Build.VERSION.SDK_INT >= 19) {
            val appOps =
                context?.getSystemService(Context.APP_OPS_SERVICE) as AppOpsManager
            val appInfo = context?.applicationInfo
            val pkg = context?.applicationContext?.packageName
            val uid = appInfo?.uid
            try {
                val appOpsClass = Class.forName(AppOpsManager::class.java.name)
                val checkOpNoThrowMethod = appOpsClass.getMethod(
                    "checkOpNoThrow", Integer.TYPE,
                    Integer.TYPE,
                    String::class.java
                )
                val opPostNotificationValue = appOpsClass.getDeclaredField("OP_POST_NOTIFICATION")
                val value = opPostNotificationValue[Int::class.java] as Int
                return (checkOpNoThrowMethod.invoke(appOps, value, uid, pkg) as Int
                        == AppOpsManager.MODE_ALLOWED)
            } catch (e: Exception) {
                return true;
            }
        } else {
            return true;
        }
    }

    @JvmStatic
    fun gotoSet() {
        val context = context
        val intent = Intent()
        if (Build.VERSION.SDK_INT >= 26) {
            // android 8.0引导
            intent.setAction("android.settings.APP_NOTIFICATION_SETTINGS")
            intent.putExtra("android.provider.extra.APP_PACKAGE", context?.packageName)
        } else if (Build.VERSION.SDK_INT >= 21) {
            // android 5.0-7.0
            intent.setAction("android.settings.APP_NOTIFICATION_SETTINGS")
            intent.putExtra("app_package", context?.packageName)
            intent.putExtra("app_uid", context?.applicationInfo?.uid)
        } else {
            // 其他
            intent.setAction("android.settings.APPLICATION_DETAILS_SETTINGS")
            intent.setData(Uri.fromParts("package", context?.packageName, null))
        }
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        context?.startActivity(intent)
    }
}
