package com.halomobi.ssp.base.utils

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Bitmap
import android.os.Build
import android.text.TextUtils
import android.view.SoundEffectConstants.CLICK
import android.webkit.DownloadListener
import android.webkit.WebChromeClient
import android.webkit.WebSettings
import android.webkit.WebView
import android.webkit.WebViewClient
import com.halomobi.ssp.base.common.ConstantPool.TrackerType
import com.halomobi.ssp.base.utils.KeyUtil.getMD5
import com.halomobi.ssp.bussiness.module_download.DownLoadManager
import com.halomobi.ssp.bussiness.module_download.bean.AppInfo
import com.halomobi.ssp.bussiness.module_download.bean.DownloadItem
import com.halomobi.ssp.bussiness.module_download.listener.DownloadStateListener
import com.halomobi.ssp.bussiness.module_report.track.ReportTrackerManager
import com.halomobi.ssp.framework.listener.TrackerTaskListener
import com.halomobi.ssp.model.request.bean.AdBean


class WebViewUtils {
    private val dUrls = HashSet<String>()
    private var mContext: Context? = null

    @SuppressLint("SetJavaScriptEnabled")
    fun initWebView(context: Context, wView: WebView?, mAd: AdBean?) {
        mContext = context
        val webSettings = wView?.settings
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            webSettings?.mixedContentMode = WebSettings.MIXED_CONTENT_ALWAYS_ALLOW
        }

        webSettings?.setSupportZoom(true)
        webSettings?.javaScriptEnabled = true
        webSettings?.javaScriptCanOpenWindowsAutomatically = true
        //        webSettings.setBuiltInZoomControls(true);// support zoom
//        webSettings.setDisplayZoomControls(false);
        webSettings?.useWideViewPort = true // 这个很关键
        webSettings?.loadWithOverviewMode = true
        webSettings?.blockNetworkImage = false
        WebView.setWebContentsDebuggingEnabled(true)

        webSettings?.domStorageEnabled = true
        webSettings?.setAppCacheMaxSize((1024 * 1024 * 20).toLong())
        val appCachePath = context.cacheDir.absolutePath
        webSettings?.setAppCachePath(appCachePath)
        webSettings?.allowFileAccess = true
        webSettings?.setAppCacheEnabled(true)
        val mWebViewClient: WebViewClient = object : WebViewClient() {
            override fun onPageStarted(view: WebView, url: String, favicon: Bitmap) {
                super.onPageStarted(view, url, favicon)
            }

            @Deprecated("Deprecated in Java")
            override fun shouldOverrideUrlLoading(view: WebView, url: String): Boolean {
//                LogUtils.d("fza" + " RewardVideo: shouldOverrideUrlLoading" + url);
                if (mAd != null && mAd.interaction_type == 1 && !mAd.isSubmitClkTrk) {
                    if (mAd.getTracks(CLICK).isNotEmpty()) {
                        ReportTrackerManager.instance
                            ?.reportTrack(CLICK, object : TrackerTaskListener {
                                override fun onTrackersReturn() {
                                }
                            }, mAd.getTracks(CLICK))
                    }
                    mAd.trackerGroup?.remove(CLICK)
                }

                return super.shouldOverrideUrlLoading(view, url)
            }
        }
        wView?.webViewClient = mWebViewClient
        val mWebChromeClient: WebChromeClient = object : WebChromeClient() {
            override fun onProgressChanged(view: WebView, newProgress: Int) {
                super.onProgressChanged(view, newProgress)
            }
        }

        wView?.webChromeClient = mWebChromeClient
        // 取消滚动条
        wView?.isHorizontalScrollBarEnabled = false //水平不显示
        wView?.isVerticalScrollBarEnabled = false //垂直不显示
        wView?.setDownloadListener(object : DownloadListener {
            var appInfo: AppInfo? = null

            override fun onDownloadStart(
                url: String,
                userAgent: String,
                contentDisposition: String,
                mimetype: String,
                contentLength: Long
            ) {
//                LogUtils.d("fza" + "downloadUrl:" + url);
                if (TextUtils.isEmpty(url) || contentLength <= 0) {
                    return
                }

                if (dUrls.contains(url)) {
//                    Utils.showToastSafe("正在下载中...");
                    return
                }

                if (appInfo != null && url == appInfo?.downloadUrl) {
                    val packageName = appInfo?.packageName
                    if (DownLoadManager.getInstance(mContext)
                            .isAppInstalled(packageName)
                    ) { //检测是否已经安装
                        if (DownLoadManager.getInstance(mContext).openApp(packageName, -1)) {
                        // LogUtils.d("WebView 打开 已安装 app--->" + packageName);
                        }
                    } else {
                        dUrls.add(url)

                        //DownLoadManager.getInstance(mContext).downloadWithShow(createDownloadItem(url, mAd));
                        appInfo = AppInfo(url, contentLength)
                    }
                } else {
                    dUrls.add(url)

                    //DownLoadManager.getInstance(mContext).downloadWithShow(createDownloadItem(url, mAd));
                    appInfo = AppInfo(url, contentLength)
                }
            }
        })
    }


    private fun createDownloadItem(url: String, mAd: AdBean): DownloadItem {
        val item = DownloadItem()
        item.name = getMD5(url) //设置下载项目名称
        item.downLoadUrl = url.trim { it <= ' ' } //下载地址
        if (mAd.ctop == 4L) {
            //替换点击位置
            item.clickTracker = mAd.getTracks(TrackerType.CLICK).toTypedArray().contentToString()
        }
        item.setDownloadStartTrackerTracker(
            mAd.getTracks(TrackerType.S_DOWNLOAD).toTypedArray().contentToString()
        )
        item.downloadedTracker =
            mAd.getTracks(TrackerType.DOWNLOADED).toTypedArray().contentToString()
        item.installStartTracker =
            mAd.getTracks(TrackerType.INSTALL_START).toTypedArray().contentToString()
        item.installedTracker =
            mAd.getTracks(TrackerType.INSTALLED).toTypedArray().contentToString()
        item.openTracker = mAd.getTracks(TrackerType.OPEN).toTypedArray().contentToString()
        item.listener = object : DownloadStateListener {
            override fun onDownloadError(
                item: DownloadItem?,
                errorCode: Int,
                errorMsg: String?
            ) {
                //                LogUtils.d("fza" + "onDownloadError:");
                dUrls.remove(url)
            }

            override fun onDownloadStart(item: DownloadItem?, ad: AdBean?) {
                //                LogUtils.d("fza" + "onDownloadStart:");
            }

            override fun onDownloaded(item: DownloadItem?, ad: AdBean?) {
                //                LogUtils.d("fza" + "onDownloaded:" + url);
                dUrls.remove(url)
            }

            override fun onInstalled(item: DownloadItem?, ad: AdBean?) {
                //                LogUtils.d("fza" + "onInstalled:" + url);
            }

            override fun onOpened(item: DownloadItem?, ad: AdBean?) {
                //                LogUtils.d("fza" + "onOpened:" + url);
            }
        }

        return item
    }
}
