package com.halomobi.ssp.bussiness.module_Exception.sentry.android.ndk;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Arrays;
import java.util.List;

import com.halomobi.ssp.bussiness.module_Exception.sentry.SentryLevel;
import com.halomobi.ssp.bussiness.module_Exception.sentry.SentryOptions;
import com.halomobi.ssp.bussiness.module_Exception.sentry.android.core.IDebugImagesLoader;
import com.halomobi.ssp.bussiness.module_Exception.sentry.android.core.SentryAndroidOptions;
import com.halomobi.ssp.bussiness.module_Exception.sentry.protocol.DebugImage;
import com.halomobi.ssp.bussiness.module_Exception.sentry.util.Objects;


final class DebugImagesLoader implements IDebugImagesLoader {

  private final @NotNull SentryOptions options;

  private final @NotNull NativeModuleListLoader moduleListLoader;

  private static @Nullable List<DebugImage> debugImages;

  /** we need to lock it because it could be called from different threads */
  private static final @NotNull Object debugImagesLock = new Object();

  DebugImagesLoader(
      final @NotNull SentryAndroidOptions options,
      final @NotNull NativeModuleListLoader moduleListLoader) {
    this.options = Objects.requireNonNull(options, "The SentryAndroidOptions is required.");
    this.moduleListLoader =
        Objects.requireNonNull(moduleListLoader, "The NativeModuleListLoader is required.");
  }

  /**
   * Returns the list of debug images loaded by sentry-native.
   *
   * @return the list or null.
   */
  @Override
  public @Nullable List<DebugImage> loadDebugImages() {
    synchronized (debugImagesLock) {
      if (debugImages == null) {
        try {
          final DebugImage[] debugImagesArr = moduleListLoader.loadModuleList();
          if (debugImagesArr != null) {
            debugImages = Arrays.asList(debugImagesArr);
            options
                .getLogger()
                .log(SentryLevel.DEBUG, "Debug images loaded: %d", debugImages.size());
          }
        } catch (Exception e) {
          options.getLogger().log(SentryLevel.ERROR, e, "Failed to load debug images.");
        }
      }
    }
    return debugImages;
  }

  /** Clears the caching of debug images on sentry-native and here. */
  @Override
  public void clearDebugImages() {
    synchronized (debugImagesLock) {
      try {
        moduleListLoader.clearModuleList();

        options.getLogger().log(SentryLevel.INFO, "Debug images cleared.");
      } catch (Exception e) {
        options.getLogger().log(SentryLevel.ERROR, e, "Failed to clear debug images.");
      }
      debugImages = null;
    }
  }


  @Nullable
  List<DebugImage> getCachedDebugImages() {
    return debugImages;
  }
}
