package com.halomobi.ssp.bussiness.module_Exception.sentry.exception;

import org.jetbrains.annotations.NotNull;

import com.halomobi.ssp.bussiness.module_Exception.sentry.protocol.Mechanism;
import com.halomobi.ssp.bussiness.module_Exception.sentry.util.Objects;

/**
 * A throwable decorator that holds an {@link Mechanism} related to the decorated
 * {@link Throwable}.
 */

public final class ExceptionMechanismException extends RuntimeException {
  private static final long serialVersionUID = 142345454265713915L;

  private final @NotNull Mechanism exceptionMechanism;
  private final @NotNull Throwable throwable;
  private final @NotNull Thread thread;
  private final boolean snapshot;

  /**
   * A {@link Throwable} that decorates another with a Sentry {@link Mechanism}.
   *
   * @param mechanism The {@link Mechanism}.
   * @param throwable The {@link Throwable}.
   * @param thread The {@link Thread}.
   * @param snapshot if the captured {@link Thread}'s stacktrace is a snapshot.
   */
  public ExceptionMechanismException(
      final @NotNull Mechanism mechanism,
      final @NotNull Throwable throwable,
      final @NotNull Thread thread,
      final boolean snapshot) {
    exceptionMechanism = Objects.requireNonNull(mechanism, "Mechanism is required.");
    this.throwable = Objects.requireNonNull(throwable, "Throwable is required.");
    this.thread = Objects.requireNonNull(thread, "Thread is required.");
    this.snapshot = snapshot;
  }

  /**
   * A {@link Throwable} that decorates another with a Sentry {@link Mechanism}.
   *
   * @param mechanism The {@link Mechanism}.
   * @param throwable The {@link Throwable}.
   * @param thread The {@link Thread}.
   */
  public ExceptionMechanismException(
      final @NotNull Mechanism mechanism,
      final @NotNull Throwable throwable,
      final @NotNull Thread thread) {
    this(mechanism, throwable, thread, false);
  }

  /**
   * Returns the encapsulated Mechanism
   *
   * @return the Mechanism
   */
  public @NotNull Mechanism getExceptionMechanism() {
    return exceptionMechanism;
  }

  /**
   * Returns the encapsulated Throwable
   *
   * @return the Throwable
   */
  public @NotNull Throwable getThrowable() {
    return throwable;
  }

  /**
   * Returns the encapsulated Thread
   *
   * @return the Thread
   */
  public @NotNull Thread getThread() {
    return thread;
  }

  /**
   * Returns true if its a snapshot or false otherwise
   *
   * @return true or false
   */
  public boolean isSnapshot() {
    return snapshot;
  }
}
