package com.halomobi.ssp.bussiness.module_report.track

import android.annotation.SuppressLint
import android.content.ContentValues
import android.database.Cursor
import android.database.sqlite.SQLiteDatabase
import android.provider.BaseColumns
import android.text.TextUtils
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.base.utils.FileUtils
import com.halomobi.ssp.base.utils.LogUtils.d
import com.halomobi.ssp.base.utils.LogUtils.w
import com.halomobi.ssp.bussiness.module_report.db.DatabaseManager
import com.halomobi.ssp.framework.listener.TrackerTaskListener


class ReportTrackerManager {

    private val dManager: DatabaseManager = DatabaseManager.getInstance() // 数据库的帮助类对象
    /**
     * 上报监播
     *
     * @param tracks 监播集合
     */
    fun reportTrack(tracks: Set<String?>?) {
        if (tracks?.isEmpty() == true) {
            return
        }
        ReportTrackerProcessor(tracks, null, false).process()
    }

    /**
     * 上报监播
     *
     * @param tracks 监播集合
     */
    fun reportTrack(trackType: Int, listener: TrackerTaskListener?, tracks: Set<String?>?) {
        //Banner 需要接收展示监播回调触发轮播机制。
        if (trackType == ConstantPool.TrackerType.DISPLAY) {
            ReportTrackerProcessor(tracks, listener, false).process()
            return
        }

        reportTrack(tracks)
    }

    /**
     * 上传历史失败监播
     */
    fun reportTrackerCache() {
        ReportTrackerProcessor(trackerCaches, null, true).process();
    }

    
    /**
     * 读取缓存监播数据
     *
     * @return 监播缓存
     */
    private val trackerCaches: Set<String>
        /**
         * 读取缓存监播数据
         *
         * @return 监播缓存
         */
        get() {
            val trackers: MutableSet<String> =
                HashSet()
            val validTime =
                System.currentTimeMillis() - EXPIRATION_TIME
            var cursor: Cursor? = null
            synchronized(DatabaseManager::class.java) {
                try {
                    val db: SQLiteDatabase = dManager.openDatabase()
                    cursor = db.query(
                        TABLE,
                        arrayOf(
                            BaseColumns._ID,
                            COLUMN_TRACK_URL
                        ),
                        "$COLUMN_TRACK_TIME >= ?",
                        arrayOf(validTime.toString() + ""),
                        null,
                        null,
                        COLUMN_TRACK_TIME
                    )
                    if (cursor?.moveToFirst() == true) {
                        do {
                            cursor?.getColumnIndex(COLUMN_TRACK_URL)?.let { it ->
                                cursor?.getString(it)
                                    ?.let { trackers.add(it) }
                            }
                        } while (cursor?.moveToNext()==true)
                    }
                    //删除所有监播（包含过期的）
                    db.delete(
                        TABLE,
                        null,
                        null
                    )
                } catch (e: Exception) {
                    e.printStackTrace()
                } finally {
                    FileUtils.close(cursor)
                    dManager.closeDatabase()
                }
                return trackers
            }
        }


    /**
     * 监播发送失败时，本地记录
     *
     * @param track 监播地址
     */
    fun addTracker(track: String) {
        synchronized(DatabaseManager::class.java) {
            //设置写锁
            var db: SQLiteDatabase? = null
            try {
                if (TextUtils.isEmpty(track)) {
                    w("监播为空")
                    return
                }
                db = dManager.openDatabase()
                db.beginTransaction()
                d("track  : $track")
                val values = ContentValues()
                values.put(COLUMN_TRACK_URL, track)
                values.put(COLUMN_TRACK_TIME, System.currentTimeMillis())
                db.insert(TABLE, null, values)
                db.setTransactionSuccessful()
            } catch (e: Exception) {
                e.printStackTrace()
            } finally {
                if (db != null) {
                    if (db.inTransaction()) {
                        db.endTransaction()
                    }
                    dManager.closeDatabase()
                }
            }
        }
    }

    companion object {
        const val TABLE: String = "trackers"

        // Columns
        const val COLUMN_TRACK_URL: String = "url"
        const val COLUMN_TRACK_TIME: String = "last_request_time"
        /**
         * 监播过期时间24h
         */
        var EXPIRATION_TIME: Long = (60 * 60 * 24 * 1000).toLong()
        private var mInstance: ReportTrackerManager? = null

        @JvmStatic
        @get:Synchronized
        val instance: ReportTrackerManager?
            get() {
                if (mInstance == null) {
                    mInstance = ReportTrackerManager()
                }
                return mInstance
            }
    }
}
