package com.halomobi.ssp.bussiness.module_report.track

import com.halomobi.ssp.base.common.ConstantPool.EroType
import com.halomobi.ssp.framework.listener.TrackerTaskListener
import com.halomobi.ssp.model.net.absnet.AbsNetProcessor
import com.halomobi.ssp.model.net.absnet.AbsNetTask
import com.halomobi.ssp.model.net.http.HttpResult

internal class ReportTrackerProcessor(
    trackUrls: Set<String?>?,
    private val mListener: TrackerTaskListener?,
    isHistory: Boolean
) :
    AbsNetProcessor() {
    private var isSuccess = true

    private val mTask: ReportTrackerTask

    /**
     * 是否是上报之前失败的监播历史数据
     */
    private var isReportHistory = false

    init {
        isReportHistory = isHistory

        mTask = ReportTrackerTask(trackUrls)
    }

    /**
     * 创建TASK
     *
     * @return 错误上报NetTask
     */
    override fun createNetTask(): AbsNetTask {
        return mTask
    }

    /**
     * 网络请求成功返回
     */
    override fun onDataReceived(result: HttpResult?) {
        if ((if (result != null) result.code else 0) ?.compareTo(300)==-1) {
            isSuccess = isSuccess and true
        } else {
            reportError(
                EroType.BAD_TRACK,
                "监播发送失败 , " + "  httpCode：" + result?.code + " , URL：" + mTask.url
            )
            isSuccess = false
        }

        if (!mTask.isLastRequest) {
            //如果不是一轮监播中的最后一条，直接返回
            return
        }

        if (isSuccess) {
            //所有监播都发送完成且没有失败才会触发banner轮播，如果mListener != null 说明是展示监播。
            onTrackersReturn()
        }

        //每一轮监播发送成功后检测是否有失败监播可以发送
        if (!isReportHistory) {//避免陷入无限循环，只触发一次上报历史
            ReportTrackerManager.instance?.reportTrackerCache();//检测是否有失败监播可以发送
        }
    }

    /**
     * 网络请求错误返回
     */
    override fun onErrorReceived(eroMsg: String?, eroCode: Int) {
        reportError(
            EroType.BAD_TRACK, """监播发送失败, error_msg : $eroMsg  URL：${mTask.url}"""
        )
        //Listener是为了回调到Banner类触发轮播的，一系列监播中任何一次发生错误，就停止触发Banner轮播，
        isSuccess = false

        //每一轮监播发送成功后检测是否有失败监播可以发送
        if (!isReportHistory && mTask.isLastRequest) {//避免陷入无限循环，只触发一次上报历史
            ReportTrackerManager.instance?.reportTrackerCache();//检测是否有失败监播可以发送
        }
    }

    /**
     * 网络错误
     */
    override fun onNetworkError() {
        reportError(EroType.BAD_TRACK, "监播发送失败,网络错误，URL：" + mTask.url)
        //Listener是为了回调到Banner类触发轮播的，一系列监播中任何一次发生错误，就停止触发Banner轮播
        isSuccess = false
    }

    /**
     * 回调到Manager
     */
    private fun onTrackersReturn() {
        if (mListener == null) {
            return
        }

        mListener.onTrackersReturn()
    }

    /**
     * 监播发送失败，上报错误信息
     *
     * @param error_code 错误码
     * @param error_msg  错误信息
     */
    private fun reportError(error_code: Int, error_msg: String) {
        ReportTrackerManager.instance?.addTracker(mTask.url.toString())
    }
}
