package com.halomobi.ssp.bussiness.module_sdk

import android.app.Application
import android.content.Context
import android.text.TextUtils
import android.util.Log
import com.danikula.videocache.HttpProxyCacheServer
import com.github.gzuliyujiang.oaid.DeviceIdentifier
import com.halomobi.ssp.base.common.ConfigPool
import com.halomobi.ssp.base.utils.LogUtils
import com.halomobi.ssp.base.utils.MyUtils

import com.halomobi.ssp.base.utils.Utils
import com.halomobi.ssp.bussiness.module_Exception.manager.SentryManger
import com.halomobi.ssp.bussiness.module_download.DownLoadManager
import com.halomobi.ssp.bussiness.module_download.receiver.DownloadReceiver
import com.halomobi.ssp.bussiness.module_sdk.normal.HmRewardVideoManager.Companion.instance
import com.halomobi.ssp.framework.Abstract.AbstractInitAd
import com.halomobi.ssp.framework.schedule.ThreadManager
import com.halomobi.ssp.framework.schedule.ThreadManager.shortPool
import com.halomobi.ssp.model.request.iptask.RequestIpManager

object HlSdk {
    var proxy: HttpProxyCacheServer? = null

    /**
     * 初始化方法
     *
     * @param context 上下文
     * @param appId   媒体id
     * @return 是否初始化成功
     */
    fun init(
        context: Context?,
        appId: String,
        isDebug: Boolean,
        abstractInitAd: AbstractInitAd
    ): Boolean {
        if (context == null || TextUtils.isEmpty(appId)) {
            Log.d("HaloMobiSDK", "init failed,parameter cannot be empty!")
            return false
        }
        DeviceIdentifier.register(context as Application?)
        init(context, appId, isDebug,abstractInitAd)
        return true
    }


    /**
     * @param app_id  广告appID
     * @param context 上下文Context
     * @param isDebug 是否开启Debug
     */
    private fun init(context: Context, app_id: String, isDebug: Boolean,abstractInitAd:AbstractInitAd) {
        MyUtils.init(context, isDebug)
        Utils.init(context, isDebug,abstractInitAd)
        LogUtils.d("init:app_id:$app_id")
        ConfigPool.init(app_id)
        SentryManger.initSentry(context)
        shortPool.execute{Runnable{initManager(context)}}
        LogUtils.d("initSuccess")
    }

    private fun newProxy(context: Context?): HttpProxyCacheServer {
        return HttpProxyCacheServer.Builder(context)
            .maxCacheFilesCount(5)
            .maxCacheSize((1024 * 1024 * 1024).toLong())
            .build()
    }

    @JvmStatic
    fun getProxy(context: Context?,url: String?): HttpProxyCacheServer? {
        if (proxy == null) {
            proxy = newProxy(context)
        }
        proxy?.registerCacheListener(instance?.mCacheListener, url)
        return proxy
    }


    /**
     * 初始化功能模块
     */
    private fun initManager(context: Context?) {
        //请求IP对应表
        RequestIpManager.instance?.requestIp()
        //注册 监听app安装 广播
        DownloadReceiver.registerReceiver(context)
        //初始化下载模块，执行未完成任务
        DownLoadManager.getInstance(context)
    }


    /**
     * app退出回调
     */
    fun onDestroy() {
        if (!isAdInit) { // 防止未初始化就调用了此方法
            return
        }
        DownloadReceiver.unregisterReceiver(Utils.context)
        ThreadManager.onDestroy()
        if (proxy != null) {
            proxy?.unregisterCacheListener(instance?.mCacheListener)
        }
    }

    val isAdInit: Boolean
        /**
         * 是否初始化
         */
        get() = Utils.context != null && !TextUtils.isEmpty(
            ConfigPool.mAppId
        )
}
