package com.halomobi.ssp.bussiness.module_sdk.normal

import android.app.Activity
import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.hardware.Sensor
import android.hardware.SensorManager
import android.util.Log
import android.view.Gravity
import android.widget.ImageView
import android.widget.TextView
import com.halomobi.ssp.base.common.ConfigPool
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.base.utils.ToolsUtils
import com.halomobi.ssp.base.utils.Utils
import com.halomobi.ssp.framework.Abstract.AbsNormalAd
import com.halomobi.ssp.framework.listener.ClickAdStateChangListener
import com.halomobi.ssp.model.request.config.SDKInfo
import com.ssp.sdk.R
import java.util.Timer
import java.util.TimerTask
import kotlin.math.max

class HmBannerAd(context: Activity?, adSlotId: String?) :
    AbsNormalAd(context, adSlotId, ConstantPool.AdType.BANNER),
    ClickAdStateChangListener {
    private var mTimer: Timer? = null

    private var mTask: TimerTask? = null
    private var textView: TextView? = null
    private var imageView: ImageView? = null
    private var textView1: TextView? = null


    /**
     * 循环轮播任务
     */
    private val recycleTask = Runnable {
        if (getAdView<AbstractAdView>()?.isAttachToWindow == true) {
            requestAd(mContext?.let { BannerAdView(it).viewSize })
            try {
                if (ad != null) {
                    ad?.isShow = true
                    adInfo.isShowNow = true
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }

    init {
        Utils.putSpInt(ConstantPool.AdType.BANNER.type)
        setClickAdListener(this)
    }

    fun setRefresh(refreshCount: Int) {
        if (refreshCount in 30..120) {
            mTimer = Timer()
            mTask = object : TimerTask() {
                override fun run() {
                    Utils.postDelayed(recycleTask, 0)
                }
            }
            mTimer?.schedule(mTask, (refreshCount * 1000).toLong(), (refreshCount * 1000).toLong())
        }
    }


    fun stopRefresh() {
        if (mTask != null) {
            mTask?.cancel()
        }
    }

    val ecpm: Int?
        get() {
            if (ad == null) return 0
            return ad?.price
        }

    override fun onAdError(errorMsg: String?, errorCode: Int) {
        // 优化，无业务错误或者无广告情况下，默认延迟轮播
        if (!canReloadAd(errorCode)) {
            super.onAdError(errorMsg, errorCode)
            return
        }
        if (mAdListener != null) {
            mAdListener?.onAdError(errorMsg, errorCode)
        }
        //当自刷新计时器为空时，默认延迟轮播
        if (mTask == null) {
            Utils.postDelayed(
                recycleTask,
                max(ConfigPool.BANNER_REQUEST_INTERVAL.toDouble(), (60 * 1000).toDouble()).toLong()
            )
        }
    }

    /**
     * 判断是否可以轮播
     *
     * @param errorCode 错误码
     * @return boolean 是否可以轮播广告
     */
    private fun canReloadAd(errorCode: Int): Boolean {
        return errorCode == 6 // 无广告
                || errorCode == ConstantPool.EroType.WITHOUT_NET || errorCode == ConstantPool.EroType.LOAD_AD_TIMEOUT || errorCode == ConstantPool.EroType.RENDER_AD_TIMEOUT || errorCode == ConstantPool.EroType.RENDER_FAILED
    }

    override fun onAdClose() {
        removeSelf()
    }

    override fun createAdView(context: Context?): BannerAdView? {
        return context?.let { BannerAdView(it) }
    }

    override fun onClickAdStateChang(clickAdState: Int) {
        when (clickAdState) {
            ClickAdStateChangListener.STATE_WEB_OPEN, ClickAdStateChangListener.STATE_APP_DOWNLOAD_START -> Utils.removeCallbacks(
                recycleTask
            )

            ClickAdStateChangListener.STATE_WEB_CLOSE, ClickAdStateChangListener.STATE_APP_DOWNLOAD_OK, ClickAdStateChangListener.STATE_APP_DOWNLOAD_FAILED -> Utils.postDelayed(
                recycleTask, 1000
            )

            else -> {}
        }
    }

    //设置是否取消关闭按钮
    fun setCloseBtnVisible(isShow: Boolean) {
        getAdView<AbstractAdView>()?.setCloseBtnVisible(isShow)
    }

    fun release() {
        getAdView<AbstractAdView>()?.removeAllViews()
    }

    /**
     * 条形广告视图
     */
    inner class BannerAdView internal constructor(context: Context) : AbsAdView(context) {
        init {

            //logo
            imageView = ImageView(context)
            imageView?.setImageResource(R.drawable.hj_logo)
            val params = LayoutParams(100, 50)
            params.gravity = Gravity.BOTTOM or Gravity.END
            params.bottomMargin = 10
            params.rightMargin = 20
            addView(imageView, params)

            //点击样式
            this.setOnClickListener {
                mAdController?.clickAd(ad)
            }



            //跳过按钮
            textView = TextView(context)
            val params1 = LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT)
            params1.gravity = Gravity.TOP or Gravity.END
            params1.rightMargin = 20
            textView?.text = SDKInfo.CLO_SIGN
            textView?.setTextColor(Color.WHITE)
            textView?.gravity = Gravity.CENTER
            addView(textView, params1)
            textView?.setOnClickListener {
                closeView()
                mAdController?.closeAd(ad) //上报关闭事件
            }

        }

        override val viewSize: IntArray
            get() = intArrayOf(ToolsUtils.dp2px(context, 320F), ToolsUtils.dp2px(context, 50F))

        override fun drawView(canvas: Canvas?) {
        }

        override fun onAttachedToWindow() {
            super.onAttachedToWindow()
        }

        override fun onDetachedFromWindow() {
            Utils.removeCallbacks(recycleTask)
            super.onDetachedFromWindow()
            if (mTask != null) {
                mTask?.cancel()
            }
        }
    }
}
