package com.halomobi.ssp.bussiness.module_sdk.normal

import android.app.Activity
import android.app.AlertDialog
import android.content.Context
import android.content.DialogInterface
import android.content.res.Configuration
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Point
import android.hardware.Sensor
import android.hardware.SensorEvent
import android.hardware.SensorEventListener
import android.hardware.SensorManager
import android.os.Looper
import android.util.DisplayMetrics
import android.view.Gravity
import android.view.View
import android.view.WindowManager
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.RelativeLayout
import android.widget.TextView
import android.widget.Toast
import com.halomobi.ssp.base.common.ConfigPool
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.base.utils.ToolsUtils
import com.halomobi.ssp.base.utils.Utils
import com.halomobi.ssp.framework.Abstract.AbsNormalAd
import com.halomobi.ssp.framework.listener.ClickAdStateChangListener
import com.halomobi.ssp.model.request.config.SDKInfo
import com.ssp.sdk.R
import kotlin.math.abs


class HmInsertAd(activity: Activity?, adSlotId: String?) :
    AbsNormalAd(activity, adSlotId, ConstantPool.AdType.INSERT), ClickAdStateChangListener {
    private var mActivity: Activity? = null
    private val adDialog: AlertDialog?

    private var sensorManager: SensorManager? = null
    private var accelerometer: Sensor? = null

    private var lastX = 0f
    private var lastY = 0f
    private var lastZ = 0f
    private var lastUpdate: Long = 0


    private var isFirstImage = true
    private var imageView1: ImageView? = null
    private var imageView2: ImageView? = null
    private var imageView3: ImageView? = null

    private var textView: TextView? = null
    private var textView1: TextView? = null
    private var lastClickTime: Long = 0


    val ecpm: Int?
        get() {
            if (ad == null) return 0
            return ad?.price
        }

    override fun createAdView(context: Context?): InsertAdView? {
        return context?.let { InsertAdView(it) }
    }

    override fun onAdViewShow() {
        if (mActivity != null && this.getAdView<AbstractAdView?>() != null && mActivity?.isFinishing == false) {
            adDialog?.show()
            getAdView<AbstractAdView>()?.let { adDialog?.setContentView(it) }
            adDialog?.setCancelable(true)
        }
        super.onAdViewShow()
    }


    override fun onAdClose() {
        try {
            if (adDialog != null && adDialog.isShowing) {
                adDialog.dismiss()
            }
            removeSelf()
            if (sensorManager != null) {
                sensorManager?.unregisterListener(sensorEventListener) //广告关闭时摇一摇监听关闭
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    fun showAd() {
        if (Thread.currentThread().id == Looper.getMainLooper().thread.id) {
            if (adInfo.isReady) {
                adInfo.isDismiss = false
                ad?.isShow = getAdView<AbstractAdView>()?.showAdView() == true
                checkView()
                adInfo.isReady = false
            } else {
                if (mAdListener != null) {
                    mAdListener?.onAdError(
                        "请先调用InsertAd.loadAd()", ConstantPool.EroType.AD_SHOW_FAILED
                    )
                }
            }
        }
    }

    fun clearAd() {
        onAdClose()
    }

    override fun onClickAdStateChang(clickAdState: Int) {
        when (clickAdState) {
            ClickAdStateChangListener.STATE_WEB_OPEN, ClickAdStateChangListener.STATE_OPTION_DIALOG_OPEN -> {}
            ClickAdStateChangListener.STATE_WEB_LOAD_DEEPLINK -> {
            }

            ClickAdStateChangListener.STATE_WEB_CLOSE -> {
            }

            ClickAdStateChangListener.STATE_OPTION_DIALOG_CANCEL, ClickAdStateChangListener.STATE_OPTION_DIALOG_CONFIRM -> {}
            else -> {}
        }
    }

    //设置是否取消关闭按钮
    fun setCloseBtnVisible(isShow: Boolean) {
        getAdView<AbstractAdView>()?.setCloseBtnVisible(isShow)
    }

    /**
     * 插屏广告视图
     */
    inner class InsertAdView(context: Context) : AbsAdView(context) {


        init {
            imageView1 = ImageView(context)
            imageView2 = ImageView(context)
            imageView1?.setImageResource(R.drawable.hj_shake_one)
            imageView2?.setImageResource(R.drawable.hj_shake_two)
            val params = LayoutParams(260, 260)
            params.gravity = Gravity.CENTER
            addView(imageView1, params)
            addView(imageView2, params)


            //点击样式
            textView = TextView(context)
            val params1 = LayoutParams(660, 100)
            params1.gravity = Gravity.BOTTOM or Gravity.CENTER_HORIZONTAL
            params1.bottomMargin=70
            params1.leftMargin=20
            textView?.setBackgroundResource(R.drawable.hj_corner_grey)
            textView?.text = "点击跳转至网页或第三方应用"
            textView?.textSize = 12F
            textView?.setTextColor(Color.WHITE)
            textView?.gravity = Gravity.CENTER
            addView(textView, params1)
            textView?.setOnClickListener {
                mAdController?.clickAd(ad)
            }


            //logo
            imageView3 = ImageView(context)
            imageView3?.setImageResource(R.drawable.hj_logo)
            val params2 = LayoutParams(100, 50)
            params2.gravity = Gravity.BOTTOM or Gravity.END
            params2.bottomMargin = 10
            params2.rightMargin = 20
            addView(imageView3, params2)


            //跳过按钮
            textView1 = TextView(context)
            val params3 = LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT)
            params3.gravity = Gravity.TOP or Gravity.END
            params3.rightMargin = 30
            textView1?.text = SDKInfo.CLO_SIGN
            textView1?.textSize=20F
            textView1?.setTextColor(Color.WHITE)
            textView1?.gravity = Gravity.CENTER
            addView(textView1, params3)
            textView1?.setOnClickListener {
                closeView()
                mAdController?.closeAd(ad) //上报关闭事件
            }


            sensorManager = context.getSystemService(Context.SENSOR_SERVICE) as SensorManager
            accelerometer = sensorManager?.getDefaultSensor(Sensor.TYPE_ACCELEROMETER)
            sensorManager?.registerListener(
                sensorEventListener, accelerometer, SensorManager.SENSOR_DELAY_NORMAL
            )

        }

        override fun drawView(canvas: Canvas?) {

        }

        //设置插屏实际尺寸 根据屏幕尺寸动态调节
        override val viewSize: IntArray
            get() = intArrayOf(
                (screenSizes[0] * 0.7).toInt(),if (resources.configuration.orientation == Configuration.ORIENTATION_LANDSCAPE) (screenSizes[0] * 0.9).toInt() else (screenSizes[1] * 0.45).toInt()
            )
    }

    private val sensorEventListener: SensorEventListener = object : SensorEventListener {
        override fun onSensorChanged(event: SensorEvent) {
            val curTime = System.currentTimeMillis()
            if ((curTime - lastUpdate) > TIME_THRESHOLD) {
                val diffTime = (curTime - lastUpdate)
                lastUpdate = curTime

                val x = event.values[0]
                val y = event.values[1]
                val z = event.values[2]

                val speed =
                    (abs((x + y + z - lastX - lastY - lastZ).toDouble()) / diffTime * 10000).toFloat()
                if (speed >= 0) { // 摇动的判定条件
                    if (isFirstImage) {
                        imageView1?.visibility = View.VISIBLE
                        imageView2?.visibility = View.GONE
                        isFirstImage = false
                    } else {
                        imageView1?.visibility = View.GONE
                        imageView2?.visibility = View.VISIBLE
                        isFirstImage = true
                    }
                }
                if (speed > FORCE_THRESHOLD && mActivity?.isFinishing == false && mActivity?.window
                        ?.decorView?.windowVisibility == View.VISIBLE && !isQuickClick()
                ) {
                    mAdController?.clickAd(ad)
                }

                lastX = x
                lastY = y
                lastZ = z
            }
        }

        override fun onAccuracyChanged(sensor: Sensor, accuracy: Int) {
            // 可以选择忽略这个方法，因为我们不关心传感器精度变化。
        }
    }

    fun isQuickClick(): Boolean {
        val currentTime: Long = System.currentTimeMillis()
        if (currentTime - lastClickTime > 1000) {
            lastClickTime = currentTime;
            return false
        }
        return true
    }

    init {
        Utils.putSpInt(ConstantPool.AdType.INSERT.type)
        setClickAdListener(this)
        mActivity = activity
        adDialog = AlertDialog.Builder(mActivity).create()
        adDialog.setCancelable(false)
        adDialog.setOnDismissListener(DialogInterface.OnDismissListener {
            closeView()
        })
    }

    companion object {
        private const val FORCE_THRESHOLD = 170
        private const val TIME_THRESHOLD = 100
    }
}
