package com.halomobi.ssp.bussiness.module_sdk.normal

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.res.Configuration
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Typeface
import android.hardware.Sensor
import android.hardware.SensorEvent
import android.hardware.SensorEventListener
import android.hardware.SensorManager
import android.text.TextUtils
import android.view.Gravity
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.base.utils.Utils
import com.halomobi.ssp.framework.Abstract.AbsNormalAd
import com.halomobi.ssp.framework.listener.ClickAdStateChangListener
import com.halomobi.ssp.model.request.config.SDKInfo
import com.ssp.sdk.R
import kotlin.math.abs


class HmSplashAd(activity: Activity?, paramClass: Class<*>?, adSlotId: String?) :
    AbsNormalAd(activity, adSlotId, ConstantPool.AdType.SPLASH),
    ClickAdStateChangListener {
    private var loopIntent: Intent? = null
    private var isJumpTargetWhenFail = true //加载失败是否自动跳转
    var isShowCountDown: Boolean = false //是否显示倒计时
    private var showAdTime: Long = 3
    private var isLoop = false
    private val closeTask = CloseTask() //关闭任务
    private val downCountTask = DownCountTask() //自动关闭任务
    private var isClick = false

    private var webIsClose = true//控制开屏跳转链接后倒计时结束是否跳转到MainActivity


    private var sensorManager: SensorManager? = null
    private var accelerometer: Sensor? = null

    private var lastX = 0f
    private var lastY = 0f
    private var lastZ = 0f
    private var lastUpdate: Long = 0


    private var isFirstImage = true
    private var imageView1: ImageView? = null
    private var imageView2: ImageView? = null
    private var imageView3: ImageView? = null

    private var textView: TextView? = null
    private var textView1: TextView? = null
    private var mActivity: Activity? = null

    private var lastClickTime: Long = 0
    fun setIntent(intent: Intent?) {
        this.loopIntent = intent
    }

    /**
     * 加载失败是否自动跳转
     *
     * @param isJump 是否自动跳转
     */
    fun setIsJumpTargetWhenFail(isJump: Boolean) {
        isJumpTargetWhenFail = isJump
    }

    /**
     * 设置广告展示时间(有效范围3*1000-99*1000)
     *
     * @param showAdTime 广告展示时间
     */
    fun setShowAdTime(showAdTime: Long) {
        if (showAdTime > 3 * 1000 && showAdTime < 100 * 1000) {
            this.showAdTime = showAdTime / 1000
        }
    }

    /**
     * 设置是否显示倒计时效果
     *
     * @param isShowCountDown 否显示倒计时
     */
    fun showCountDown(isShowCountDown: Boolean) {
        this.isShowCountDown = isShowCountDown
    }

    override fun onStart() {
    }

    val ecpm: Int?
        get() {
            if (ad == null) return 0
            return ad?.price
        }

    override fun onAdError(errorMsg: String?, errorCode: Int) {
        Utils.removeCallbacks(closeTask)
        super@HmSplashAd.onAdError(errorMsg, errorCode)
    }

    public override fun onAdViewShow() {
        if (!isLoop) {
            Utils.removeCallbacks(closeTask)
            if (isShowCountDown) {
                Utils.postDelayed(downCountTask, 1000)
            } else {
                Utils.postDelayed(closeTask, showAdTime * 1000)
            }
        }
        super.onAdViewShow()
    }

    override fun onClickAdStateChang(clickAdState: Int) {
        when (clickAdState) {
            ClickAdStateChangListener.STATE_WEB_OPEN -> {
                webIsClose = false
            }

            ClickAdStateChangListener.STATE_OPTION_DIALOG_OPEN -> {
                webIsClose = false
            }

            ClickAdStateChangListener.STATE_WEB_LOAD_DEEPLINK -> {
//                isClick = true
            }

            ClickAdStateChangListener.STATE_WEB_CLOSE -> {
                webIsClose = true
                Utils.removeCallbacks(closeTask)
                Utils.postDelayed(closeTask, 0)
            }

            ClickAdStateChangListener.STATE_OPTION_DIALOG_CANCEL, ClickAdStateChangListener.STATE_OPTION_DIALOG_CONFIRM -> {
                webIsClose = true
                Utils.removeCallbacks(closeTask)
                Utils.postDelayed(closeTask, 0)
            }

            else -> {}
        }
    }

    //设置是否取消关闭按钮
    fun setCloseBtnVisible(isShow: Boolean) {
        getAdView<AbstractAdView>()?.setCloseBtnVisible(isShow)
    }

    override fun onAdClose() {
        if (isJumpTargetWhenFail && !isLoop) {
            if (loopIntent != null && mContext is Activity) {
                val activity = mContext as Activity
                val componentName = loopIntent?.component
                if (componentName != null) {
                    // 兼容无闪屏APP
                    if (!TextUtils.equals(
                            activity.javaClass.name, componentName.className
                        )
                    ) {
                        activity.startActivity(loopIntent)
                        activity.finish()
                        val enterAnim = android.R.anim.fade_in
                        val exitAnim = android.R.anim.fade_out
                        activity.overridePendingTransition(enterAnim, exitAnim) //Activity切换动画

                    }
                }

            }
            isLoop = true
        }
        if (sensorManager != null) {
            sensorManager?.unregisterListener(sensorEventListener) //广告关闭时摇一摇监听关闭
        }
    }

    override fun createAdView(context: Context?): SplashAdView? {
        return context?.let { SplashAdView(it) }
    }

    private inner class CloseTask : Runnable {
        override fun run() {
            if (showAdTime == 0L && webIsClose) {
                closeView()
            }
        }
    }

    private inner class DownCountTask : Runnable {
        override fun run() {
            if (!adInfo.isDismiss) {
                if (showAdTime <= 0) {
                    closeTask.run()
                } else {
                    showAdTime--
                    if (!isClick) {
                        getAdView<AbstractAdView>()?.innerView?.view?.invalidate()
                        Utils.postDelayed(this, 1000)
                    }
                }
            }
        }
    }


    /**
     * 开机全屏广告视图
     */
    @SuppressLint("SetTextI18n")
    inner class SplashAdView(context: Context) : AbsAdView(context) {
        init {
            //摇一摇样式
            screenSizes = Utils.getScreenSize(false)
            imageView1 = ImageView(context)
            imageView2 = ImageView(context)
            imageView1?.setImageResource(R.drawable.hj_shake_one)
            imageView2?.setImageResource(R.drawable.hj_shake_two)
            val params = LayoutParams(360, 360)
            params.gravity = Gravity.CENTER
            addView(imageView1, params)
            addView(imageView2, params)


            //点击样式
            textView = TextView(context)
            val params1 = LayoutParams(860, 130)
            params1.gravity = Gravity.BOTTOM or Gravity.CENTER_HORIZONTAL
            params1.bottomMargin = 360
            textView?.setBackgroundResource(R.drawable.hj_corner_grey)
            textView?.text = "点击跳转至网页或第三方应用"
            textView?.textSize = 14F
            textView?.setTextColor(Color.WHITE)
            textView?.gravity = Gravity.CENTER
            addView(textView, params1)
            textView?.setOnClickListener {
                mAdController?.clickAd(ad)
            }


            //logo
            imageView3 = ImageView(context)
            imageView3?.setImageResource(R.drawable.hj_logo)
            val params2 = LayoutParams(100, 50)
            params2.gravity = Gravity.BOTTOM or Gravity.END
            params2.bottomMargin = 10
            params2.rightMargin = 20
            addView(imageView3, params2)


            //跳过按钮
            textView1 = TextView(context)
            val params3 = LayoutParams(180, 90)
            params3.gravity = Gravity.TOP or Gravity.END
            params3.topMargin = 26
            params3.rightMargin = 26
            textView1?.setBackgroundResource(R.drawable.hj_corner_grey)
            textView1?.setTextColor(Color.WHITE)
            textView1?.gravity = Gravity.CENTER
            addView(textView1, params3)
            textView1?.setOnClickListener {
                closeView()
                mAdController?.closeAd(ad) //上报关闭事件
            }


            sensorManager = context.getSystemService(Context.SENSOR_SERVICE) as SensorManager
            accelerometer = sensorManager?.getDefaultSensor(Sensor.TYPE_ACCELEROMETER)
            sensorManager?.registerListener(
                sensorEventListener,
                accelerometer,
                SensorManager.SENSOR_DELAY_NORMAL
            )

        }


        override val viewSize: IntArray
            get() = intArrayOf(if (resources.configuration.orientation == Configuration.ORIENTATION_LANDSCAPE) screenSizes[1] else screenSizes[0],if (resources.configuration.orientation == Configuration.ORIENTATION_LANDSCAPE) screenSizes[0] else  screenSizes[1])


        override fun drawView(canvas: Canvas?) {
            textView1?.text = SDKInfo.CLO_TEXT + " " + (if (isShowCountDown) showAdTime else "")
        }


        override fun onVisibilityChanged(changedView: View, visibility: Int) {
            super.onVisibilityChanged(changedView, visibility)
            if (visibility == VISIBLE) {
                if (isClick) {
                    isClick = false
                    Utils.postDelayed(downCountTask, 1000)
                }
            }
        }
    }

    val sensorEventListener: SensorEventListener = object : SensorEventListener {
        override fun onSensorChanged(event: SensorEvent) {
            val curTime = System.currentTimeMillis()
            if ((curTime - lastUpdate) > TIME_THRESHOLD) {
                val diffTime = (curTime - lastUpdate)
                lastUpdate = curTime

                val x = event.values[0]
                val y = event.values[1]
                val z = event.values[2]

                val speed =
                    (abs((x + y + z - lastX - lastY - lastZ).toDouble()) / diffTime * 10000).toFloat()
                if (speed >= 0) { // 摇动的判定条件
                    if (isFirstImage) {
                        imageView1?.visibility = View.VISIBLE
                        imageView2?.visibility = View.GONE
                        isFirstImage = false
                    } else {
                        imageView1?.visibility = View.GONE
                        imageView2?.visibility = View.VISIBLE
                        isFirstImage = true
                    }
                }
                if (speed > FORCE_THRESHOLD && mActivity?.isFinishing == false && mActivity?.window?.decorView?.windowVisibility == View.VISIBLE && !isQuickClick()) {
                    mAdController?.clickAd(ad)
                }

                lastX = x
                lastY = y
                lastZ = z
            }
        }

        override fun onAccuracyChanged(sensor: Sensor, accuracy: Int) {
            // 可以选择忽略这个方法，因为我们不关心传感器精度变化。
        }
    }


    fun isQuickClick(): Boolean {
        val currentTime: Long = System.currentTimeMillis()
        if (currentTime - lastClickTime > 1000) {
            lastClickTime = currentTime;
            return false
        }
        return true
    }


    init {
        mActivity = activity
        if (paramClass != null) {
            loopIntent = Intent(activity, paramClass)
        }
        Utils.putSpInt(ConstantPool.AdType.SPLASH.type)
        setClickAdListener(this)
    }

    companion object {
        private const val FORCE_THRESHOLD = 170
        private const val TIME_THRESHOLD = 100
    }
}
