package com.halomobi.ssp.bussiness.module_sdk.normal

import android.text.TextUtils
import android.view.View
import android.widget.TextView
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.base.utils.Utils
import com.halomobi.ssp.bussiness.module_sdk.listener.BindDataProxy
import com.halomobi.ssp.bussiness.module_sdk.listener.NativeResponse
import com.halomobi.ssp.framework.Abstract.AbstractAd
import com.halomobi.ssp.model.request.bean.AdBean

class NativeResponseImpl(private val adInfo: AdBean, private val mAbstractAd: AbstractAd) :
    NativeResponse {
    private val adTitle = AdText(adInfo.title.toString())
    private val adSub_title = AdText(adInfo.des.toString())
    private val adImage: AdImage
    private val adSource: AdText
    private val price: Int
    private var imageBindDataProxy: BindDataProxy<Array<View>, Array<String?>>? = null
    private var titleBindDataProxy: BindDataProxy<View, String>? = null



    init {
        val imgUrl = adInfo.imgurl
        val imgUrls = arrayOfNulls<String>(1)
        imgUrls[0] = imgUrl
        price = adInfo.price
        adImage = AdImage(imgUrls)
        adSource = AdText(adInfo.source.toString())
    }

    override fun bindingTitle(id: Int) {
        adTitle.textViewId = id
    }

    override fun bindingSub_title(id: Int) {
        adSub_title.textViewId = id
    }

    override fun bindingSource(id: Int) {
        adSource.textViewId = id
    }

    override fun bindingImgUrl(vararg ids: Int) {
        adImage.imageViewIds = ids
    }

    override fun getTitle(): String {
        return adTitle.text
    }

    override fun getSubTitle(): String {
        return adSub_title.text
    }

    override fun getSource(): String {
        return adSource.text
    }


    override fun getImageUrlList(): Array<String?> {
        return adImage.imageUrls
    }

    val adUUid: String
        get() = adInfo.adUUID

    override fun getType(): Int {
        return adInfo.ad_type
    }

    override fun getW(): Int {
        return adInfo.w
    }

    override fun getH(): Int {
        return adInfo.h
    }

    override fun getPrice(): Int {
        return price
    }

    override fun setImageBindDataProxy(imageBindDataProxy: BindDataProxy<Array<View>, Array<String?>>) {
        this.imageBindDataProxy = imageBindDataProxy
        val views = imageBindDataProxy.BindData(adImage.imageUrls)
        if (views != null && views.size > 0) {
            val ids = IntArray(views.size)
            for (i in views.indices) {
                ids[i] = views[i].id
            }
            adImage.imageViewIds = ids
        } else {
            throw RuntimeException("绑定视图不存在")
        }
    }

    override fun setTitleBindDataProxy(titleBindDataProxy: BindDataProxy<View, String>) {
        this.titleBindDataProxy = titleBindDataProxy
        adTitle.textViewId = titleBindDataProxy.BindData(adTitle.text).id
    }

    override fun recordImpression(v: View) {
        //必填

        if (titleBindDataProxy == null) {
            adTitle.setValue(v, true)
        } else {
            checkSubView(v, adTitle.textViewId)
            titleBindDataProxy = null
        }

        if (imageBindDataProxy == null) {
            adImage.setValue(v)
        } else {
            for (subId in adImage.imageViewIds) {
                checkSubView(v, subId)
            }
            imageBindDataProxy = null
        }

        //非必填
        if (!TextUtils.isEmpty(adSub_title.text) && adSub_title.textViewId != 0) {
            adSub_title.setValue(v, false)
        }
        if (!TextUtils.isEmpty(adSource.text) && adSource.textViewId != 0) {
            adSource.setValue(v, false)
        }

        if (!adInfo.isShow) {
            adInfo.isShow = true
            mAbstractAd.checkView(v, adInfo)
        }
    }

    fun checkSubView(parent: View, subId: Int) {
        val view = parent.findViewById<View>(subId)
            ?: throw RuntimeException("view == null")
    }

    override fun handleClick(v: View, isReport: Boolean) {
        if (!Utils.isViewCovered(v, 0.5f)) {
            mAbstractAd.clickAd(adInfo)
            if (isReport) {
                mAbstractAd.reportTracker(adInfo, ConstantPool.TrackerType.CLICK)
            }
        }
    }

    override fun handleShow() {
        mAbstractAd.reportTracker(adInfo, ConstantPool.TrackerType.DISPLAY)
    }

    override fun isDownloadApp(): Boolean {
        return adInfo.interaction_type == 2
    }

    override fun loadImage(imageView: View?, imageUrls: String) {
        if (imageView != null && !TextUtils.isEmpty(imageUrls) && (imageUrls.startsWith("http://") || imageUrls.startsWith(
                "https://"
            ))
        ) {
            mAbstractAd.loadImage(imageView, imageUrls)
        }
    }

    override fun handleClose() {
        mAbstractAd.reportTracker(adInfo, ConstantPool.TrackerType.CLOSE)
    }

    override fun getAdInfo(): AdBean {
        return adInfo
    }


    internal class AdText(val text: String) {
        var textViewId: Int = 0

        fun setValue(v: View, needCheck: Boolean) {
            val textView = v.findViewById<View>(textViewId) as TextView
            textView.text = text
        }
    }

    internal inner class AdImage(imageUrl: Array<String?>) {
        var imageUrls: Array<String?> = arrayOf()
        var imageViewIds: IntArray = intArrayOf()

        init {
            this.imageUrls = imageUrl
        }

        fun setValue(v: View) {
            if (imageUrls.isNotEmpty()) {
                for (i in imageUrls.indices) {
                    val imageView = v.findViewById<View>(imageViewIds[i])
                    if (imageView != null) {
                        mAbstractAd.loadImage(imageView, imageUrls[i])
                    } else {
                        throw RuntimeException("数据和视图不匹配")
                    }
                }
            }
        }
    }
}
