package com.halomobi.ssp.bussiness.module_sdk.view.premission

import android.Manifest
import android.app.Activity
import android.content.pm.PackageManager
import android.os.Build
import com.halomobi.ssp.base.utils.LogUtils
import com.halomobi.ssp.base.utils.Utils


class RequestPermissionManager private constructor() {
    fun shouldShowRequestPermissionRationale(activity: Activity?, permission: String): Boolean {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M && activity != null) {
            return activity.shouldShowRequestPermissionRationale(permission)
        }
        return false
    }

    /**
     * 初始化时调用，请求SDK所需的所有权限
     * 位置、手机硬件信息、SD卡权限等
     */
    fun requestPermissions(activity: Activity?, requestCode: Int) {
        if (activity == null) {
            return
        }
        val permissions: MutableList<String> = ArrayList()

        //获取手机信息
        if (!checkSelfPermission(Manifest.permission.READ_PHONE_STATE)) {
            permissions.add(Manifest.permission.READ_PHONE_STATE)
        }
        //获取位置
//        if (!checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION)) {
//            permissions.add(Manifest.permission.ACCESS_FINE_LOCATION);
//        }
        //外部存储
        if (!checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
            permissions.add(Manifest.permission.WRITE_EXTERNAL_STORAGE)
        }
        requestPermissions(activity, permissions.toTypedArray<String>(), requestCode)
    }

    private fun requestPermissions(
        activity: Activity,
        permission: Array<String>?,
        requestCode: Int
    ) {
        if (permission == null || permission.size < 1) {
            LogUtils.d("没有权限需要申请")
            return
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            activity.requestPermissions(permission, requestCode)
        }
    }

    companion object {
        const val DEFAULT_REQUEST_CODE: Int = 10001

        private var mInstance: RequestPermissionManager? = null

        @get:Synchronized
        val instance: RequestPermissionManager?
            get() = if (mInstance == null) RequestPermissionManager().also {
                mInstance = it
            } else mInstance

        /**
         * 检测权限
         * Android 6.0之前版本默认返回true
         *
         * @param permission 权限名称
         * @return 是否具有该权限
         */
        @JvmStatic
        fun checkSelfPermission(permission: String): Boolean {
            return Build.VERSION.SDK_INT < Build.VERSION_CODES.M || PackageManager.PERMISSION_GRANTED == Utils.context
                ?.checkSelfPermission(permission)
        }
    }
}
