package com.halomobi.ssp.bussiness.module_video.otherads.cn.jzvd

import android.annotation.SuppressLint
import android.graphics.SurfaceTexture
import android.media.AudioManager
import android.media.MediaPlayer
import android.media.MediaPlayer.OnBufferingUpdateListener
import android.media.MediaPlayer.OnCompletionListener
import android.media.MediaPlayer.OnPreparedListener
import android.media.MediaPlayer.OnSeekCompleteListener
import android.media.MediaPlayer.OnVideoSizeChangedListener
import android.media.PlaybackParams
import android.os.Build
import android.os.Handler
import android.os.HandlerThread
import android.view.Surface

/**
 * 实现系统的播放引擎
 */
class JZMediaSystem(jzvd: Jzvd) : JZMediaInterface(jzvd),
    OnPreparedListener, OnCompletionListener, OnBufferingUpdateListener, OnSeekCompleteListener,
    MediaPlayer.OnErrorListener, MediaPlayer.OnInfoListener, OnVideoSizeChangedListener {
    var mediaPlayer: MediaPlayer? = null

    @SuppressLint("DiscouragedPrivateApi")
    override fun prepare() {
        release()
        mMediaHandlerThread = HandlerThread("JZVD")
        mMediaHandlerThread?.start()
        mMediaHandler = mMediaHandlerThread?.looper?.let { Handler(it) } //主线程还是非主线程，就在这里
        handler = Handler()

        mMediaHandler?.post {
            try {
                mediaPlayer = MediaPlayer()
                mediaPlayer?.setAudioStreamType(AudioManager.STREAM_MUSIC)
                mediaPlayer?.isLooping = jzvd.jzDataSource?.looping == true
                mediaPlayer?.setOnPreparedListener(this@JZMediaSystem)
                mediaPlayer?.setOnCompletionListener(this@JZMediaSystem)
                mediaPlayer?.setOnBufferingUpdateListener(this@JZMediaSystem)
                mediaPlayer?.setScreenOnWhilePlaying(true)
                mediaPlayer?.setOnSeekCompleteListener(this@JZMediaSystem)
                mediaPlayer?.setOnErrorListener(this@JZMediaSystem)
                mediaPlayer?.setOnInfoListener(this@JZMediaSystem)
                mediaPlayer?.setOnVideoSizeChangedListener(this@JZMediaSystem)
                val clazz = MediaPlayer::class.java
                //如果不用反射，没有url和header参数的setDataSource函数
                val method = clazz.getDeclaredMethod(
                    "setDataSource",
                    String::class.java,
                    MutableMap::class.java
                )
                method.invoke(
                    mediaPlayer,
                    jzvd.jzDataSource?.currentUrl.toString(),
                    jzvd.jzDataSource?.headerMap
                )
                mediaPlayer?.prepareAsync()
                mediaPlayer?.setSurface(Surface(SAVED_SURFACE))
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }

    override fun start() {
        if (mMediaHandler != null && mediaPlayer != null) {
            mMediaHandler?.post { mediaPlayer?.start() }
        }
    }

    override fun pause() {
        if (mMediaHandler != null && mediaPlayer != null) {
            mMediaHandler?.post { mediaPlayer?.pause() }
        }
    }

    override val isPlaying: Boolean
        get() {
            return mediaPlayer?.isPlaying == true

        }

    override fun seekTo(time: Long) {
        if (mMediaHandler != null && mediaPlayer != null) {
            mMediaHandler?.post {
                try {
                    mediaPlayer?.seekTo(time.toInt())
                } catch (e: IllegalStateException) {
                    e.printStackTrace()
                }
            }
        }
    }

    override fun release() {
        if (mMediaHandler != null && mMediaHandlerThread != null && mediaPlayer != null) { //不知道有没有妖孽
            val tmpHandlerThread: HandlerThread = mMediaHandlerThread as HandlerThread
            val tmpMediaPlayer: MediaPlayer = mediaPlayer as MediaPlayer
            SAVED_SURFACE = null

            mMediaHandler?.post {
                tmpMediaPlayer.setSurface(null)
                tmpMediaPlayer.release()
                tmpHandlerThread.quit()
            }
            mediaPlayer = null
        }
    }

    override val currentPosition: Long
        //TODO 测试这种问题是否在threadHandler中是否正常，所有的操作mediaplayer是否不需要thread，挨个测试，是否有问题
        get() {
            return mediaPlayer?.currentPosition?.toLong() ?: 0

        }

    override val duration: Long
        get() {
            return mediaPlayer?.duration?.toLong()?:0

        }

    override fun setVolume(leftVolume: Float, rightVolume: Float) {
        if (mMediaHandler == null) return
        mMediaHandler?.post {
            if (mediaPlayer != null) mediaPlayer?.setVolume(leftVolume, rightVolume)
        }
    }

    override fun setSpeed(speed: Float) {
        if (mediaPlayer != null) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                val pp = mediaPlayer?.playbackParams
                pp?.setSpeed(speed)
                mediaPlayer?.playbackParams = pp as PlaybackParams
            }
        }
    }

    override fun onPrepared(mediaPlayer: MediaPlayer) {
        if (handler != null) {
            handler?.post { jzvd.onPrepared() } //如果是mp3音频，走这里
        }
    }

    override fun onCompletion(mediaPlayer: MediaPlayer) {
        if (handler != null) {
            handler?.post { jzvd.onCompletion() }
        }
    }

    override fun onBufferingUpdate(mediaPlayer: MediaPlayer, percent: Int) {
        if (handler != null) {
            handler?.post { jzvd.setBufferProgress(percent) }
        }
    }

    override fun onSeekComplete(mediaPlayer: MediaPlayer) {
        if (handler != null) {
            handler?.post { jzvd.onSeekComplete() }
        }
    }

    override fun onError(mediaPlayer: MediaPlayer, what: Int, extra: Int): Boolean {
        if (handler != null) {
            handler?.post { jzvd.onError(what, extra) }
        }
        return true
    }

    override fun onInfo(mediaPlayer: MediaPlayer, what: Int, extra: Int): Boolean {
        if (handler != null) {
            handler?.post { jzvd.onInfo(what, extra) }
        }
        return false
    }

    override fun onVideoSizeChanged(mediaPlayer: MediaPlayer, width: Int, height: Int) {
        if (handler != null) {
            handler?.post { jzvd.onVideoSizeChanged(width, height) }
        }
    }

    override fun setSurface(surface: Surface?) {
        if (mediaPlayer != null) {
            mediaPlayer?.setSurface(surface)
        }
    }

    override fun onSurfaceTextureAvailable(surface: SurfaceTexture, width: Int, height: Int) {
        if (SAVED_SURFACE == null) {
            SAVED_SURFACE = surface
            prepare()
        } else {
            jzvd.textureView?.setSurfaceTexture(SAVED_SURFACE?:surface)
        }
    }

    override fun onSurfaceTextureSizeChanged(surface: SurfaceTexture, width: Int, height: Int) {
    }

    override fun onSurfaceTextureDestroyed(surface: SurfaceTexture): Boolean {
        return false
    }

    override fun onSurfaceTextureUpdated(surface: SurfaceTexture) {
    }
}
