package com.halomobi.ssp.framework.Abstract

import android.annotation.SuppressLint
import android.content.Context
import com.halomobi.ssp.base.common.ConfigPool
import com.halomobi.ssp.base.common.ConfigPool.appPackageName
import com.halomobi.ssp.base.common.ConstantPool.AdType
import com.halomobi.ssp.base.utils.Utils.post
import com.halomobi.ssp.base.utils.Utils.runInMainThread
import com.halomobi.ssp.bussiness.module_sdk.listener.AdListener

abstract class AbsNormalAd
/**
 * 限制广告位于View不可同时重复请求(如banner自刷新）
 */
    (context: Context?, adSlotId: String?, adType: AdType?) :
    AbstractAd(context, ConfigPool.mAppId, appPackageName, adSlotId, adType) {
    var mAdListener: AdListener? = null

    /**
     * 设置对外回调对象
     *
     * @param listener 广告回调对象
     */
    fun setHjAdListener(listener: AdListener?) {
        if (listener != null) {
            mAdListener = listener
        }
    }

    /**
     * 将自身从父布局移除
     */
    override fun removeSelf() {
        super.removeSelf()
    }


    /**
     * 加载并渲染广告
     */
    fun loadAndShowAd() {
        adInfo.isShowNow = true
        requestAd(getAdView<AbstractAdView>()?.viewSize)
    }

    /**
     * 加载并渲染广告
     */
    fun loadAd() {
        adInfo.isShowNow = false
        requestAd(getAdView<AbstractAdView>()?.viewSize)
    }

    override fun onAdViewShow() {
        if (mAdListener != null) {
            mAdListener?.onDisplayAd()
        }
    }


    override fun onAdReady() {
        if (mAdListener == null) {
            return
        }
        ad?.ready_time = System.currentTimeMillis()
        post(Runnable { mAdListener?.onAdReady() })
    }

    override fun onCloseAd() {
        if (mAdListener != null) {
            mAdListener?.onCloseAd()
        }
    }

    val isAdReady: Boolean
        /**
         * 广告是否已经准备就绪
         *
         * @return true 准备就绪 false 未准备好
         */
        get() = adInfo.isReady

    /**
     * 请求广告
     *
     * @param viewSize View尺寸
     */
    override fun requestAd(viewSize: IntArray?) {
        super.requestAd(viewSize)
    }

    override fun onAdError(errorMsg: String?, errorCode: Int) {
        runInMainThread(Runnable {
            if (mAdListener != null) {
                mAdListener?.onAdError(errorMsg, errorCode)
            }
        })

        super.onAdError(errorMsg, errorCode)
    }

    abstract inner class AbsAdView @SuppressLint("ClickableViewAccessibility") protected constructor(
        context: Context
    ) :
        AbstractAdView(context) {
        /**
         * 当View从其父布局中移除时系统回调该方法
         */
        override fun onDetachedFromWindow() {
            super.onDetachedFromWindow()
        }

        override fun onClickAd(type: Int) {
            if (mAdListener != null) {
                mAdListener?.onClickAd(type)
            }
        }
    }
}
