package com.halomobi.ssp.framework.Abstract

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Canvas
import android.graphics.RectF
import android.os.PowerManager
import android.view.MotionEvent
import android.view.View
import android.widget.FrameLayout
import com.halomobi.ssp.base.common.ConfigPool.CHECK_VIEW_TIME
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.base.common.ConstantPool.AdType
import com.halomobi.ssp.base.common.ConstantPool.TrackerType
import com.halomobi.ssp.base.utils.DrawableUtils.displayAnim
import com.halomobi.ssp.base.utils.LogUtils.d
import com.halomobi.ssp.base.utils.Utils.getScreenSize
import com.halomobi.ssp.base.utils.Utils.isFastDoubleClick
import com.halomobi.ssp.base.utils.Utils.isViewCovered
import com.halomobi.ssp.base.utils.Utils.post
import com.halomobi.ssp.base.utils.Utils.removeSelf
import com.halomobi.ssp.bussiness.module_sdk.listener.NativeResponse
import com.halomobi.ssp.bussiness.module_sdk.normal.HmNativeAd
import com.halomobi.ssp.bussiness.module_sdk.normal.NativeResponseImpl
import com.halomobi.ssp.bussiness.module_video.otherads.cn.jzvd.Jzvd
import com.halomobi.ssp.framework.controller.AdController.Companion.getInstance
import com.halomobi.ssp.framework.interfaces.IAdControl
import com.halomobi.ssp.framework.interfaces.IAdInnerView
import com.halomobi.ssp.framework.listener.AdInnerViewOnDrawListener
import com.halomobi.ssp.framework.listener.AdStateChangListener
import com.halomobi.ssp.framework.listener.ClickAdStateChangListener
import com.halomobi.ssp.framework.listener.TrackerTaskListener
import com.halomobi.ssp.framework.schedule.ThreadManager.checkViewPool
import com.halomobi.ssp.model.request.bean.AdBean
import com.halomobi.ssp.model.request.bean.AdParameter
import com.halomobi.ssp.model.request.config.SDKInfo

abstract class AbstractAd protected constructor(
    protected var mContext: Context?,
    app_id: String,
    packageName: String,
    adSlotId: String?,
    adType: AdType?
) : AdStateChangListener {
    var adInfo: AdInfo
        protected set

    var ad: AdBean? = null
        protected set

    private var mAdParameter: AdParameter


    private var mAdView: AbstractAdView? = null

    var mAdController: IAdControl?


    init {
        adInfo = AdInfo()
        mAdParameter = AdParameter(app_id, adSlotId, packageName, adType)
        mAdController = getInstance(mContext)
    }

    protected abstract fun createAdView(context: Context?): AbstractAdView?

    protected abstract fun onAdClose()

    @Synchronized
    fun <T : AbstractAdView?> getAdView(): T? {
        if (mAdView == null) {
            mAdView = createAdView(mContext)
        }
        return mAdView as T?
    }

    fun requestAd(adParameter: AdParameter?, adStateListener: AdStateChangListener?) {
        mAdController?.requestAd(adParameter, adStateListener)
    }

    //针对原生广告上报
    fun reportTracker(adInfo: AdBean?, type: Int) {
        mAdController?.reportTracker(adInfo, type)
    }

    fun clickAd(adInfo: AdBean?) {
        mAdController?.clickAd(adInfo)
    }

    fun setTrackerTaskListener(trackerListener: TrackerTaskListener?) {
        mAdController?.setTrackerTaskListener(trackerListener)
    }

    fun setClickAdListener(listener: ClickAdStateChangListener?) {
        mAdController?.setClickAdListener(listener)
    }

    fun loadImage(imageView: View?, string: String?) {
        mAdController?.loadImage(imageView, string)
    }

    /**
     * 将自身从父布局移除
     */
    protected open fun removeSelf() {
        removeSelf(getAdView<AbstractAdView>())
    }


    protected abstract fun onAdReady()


    /**
     * 请求广告
     *
     * @param viewSize View尺寸
     */
    protected open fun requestAd(viewSize: IntArray?) {
        synchronized(AbstractAd::class.java) {
            mAdParameter.adViewSize = viewSize
            mAdController?.requestAd(mAdParameter, this)
        }
    }

    override fun onStart() {

    }

    override fun onAdError(errorMsg: String?, errorCode: Int) {

    }


    override fun onNativeAdReach(adInfoList: List<AdBean?>?) {
        if (adInfoList != null) {
            nativeResponseSuccess(parseData(adInfoList))
        }
    }

//    override fun onFloatAdReach(adInfoList: List<AdBean?>?) {
//        floatAdResponseSuccess(adInfoList)
//    }
//
//
//    protected fun floatAdResponseSuccess(adInfoList: List<AdBean?>?) {
//        //do nothing 浮窗类型自己实现
//    }

    override fun onAdReach(adInfo: AdBean?) {
        this.ad = adInfo
        if ((getAdView<AbstractAdView>()?.renderAdView(this.ad).also {
                this.adInfo.isReady = it == true
            }) != true) {
            //渲染广告
            val errorMsg = if (adInfo?.creative_type == AdBean.CRE_TYPE_MIX) {
                """广告渲染失败，createType:${adInfo.creative_type}ImgUrl : ${adInfo.mix?.imgurl} title : ${adInfo.mix?.title}"""
            } else {
                """广告渲染失败，createType:${adInfo?.creative_type}"""
            }
            onAdError(errorMsg, ConstantPool.EroType.RENDER_FAILED)
            return
        }
        adInfo?.ready_time = System.currentTimeMillis()
        if (!this.adInfo.isShowNow) {
            onAdReady()
            return
        }
        adInfo?.isShow = true
        this.adInfo.isDismiss = false

        post(Runnable {
            try {
                adInfo?.isShow = getAdView<AbstractAdView>()?.showAdView() == true //展示广告
            } catch (e: Exception) {
                e.stackTrace
            }
        })

        //检测广告是否展示
        checkView()
    }

    private val isScreenOn: Boolean
        /**
         * 检测屏幕是否亮着,为防止意外情况，默认返回true
         *
         * @return
         */
        get() {
            val pm = mContext?.getSystemService(Context.POWER_SERVICE) as PowerManager
            return pm.isScreenOn
        }

    protected fun checkView() {
        checkView(getAdView<AbstractAdView>(), ad)
    }

    /**
     * 检测广告是否展示
     */
    fun checkView(view: View?, adInfo: AdBean?) {
        checkViewPool.execute(object : Runnable {
            override fun run() {
                if (!isScreenOn) {
                    val msg = "手机屏幕没有点亮"
                    return
                }

                val time = System.currentTimeMillis()
                do {
                    if (!isViewCovered(view, 0.5f)) { //检测是否被遮挡
                        showCloseBtn(view) //显示关闭按钮
                        return
                    } else if (System.currentTimeMillis() - time < CHECK_VIEW_TIME) {
                        try {
                            Thread.sleep(500)
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                    } else {
                        val msg = "广告被遮挡"
                        adInfo?.isShow = false
                        d("AdType:" + adInfo?.ad_type + "::AdShowFailed:" + msg)
                        return
                    }
                } while (adInfo?.isShow == true)
                val msg = "AdShowFailed:mAdInfo.isShow = " + adInfo?.isShow
                onAdError(msg, ConstantPool.EroType.AD_SHOW_FAILED)
            }
        })
    }

    /**
     * 显示关闭按钮
     */
    private fun showCloseBtn(view: View?) {
        if (this is HmNativeAd) {
            return  //原生没有关闭按钮
        }
        (view as AbstractAdView).showCloseBtn()
    }

    protected open fun nativeResponseSuccess(list: List<NativeResponse?>?) {

    }

    /**
     * 解析原生数据
     */
    private fun parseData(adInfoList: List<AdBean?>): List<NativeResponse?> {
        val list = ArrayList<NativeResponse?>(adInfoList.size)
        for (adInfo in adInfoList) {
            if (adInfo != null) {
                val response: NativeResponse = NativeResponseImpl(adInfo, this)
                list.add(response)
            }
        }
        return list
    }

    /**
     * 当广告视图可见回调，主线程执行
     */
    protected abstract fun onAdViewShow()

    protected abstract fun onCloseAd()

    /**
     * 关闭广告View
     */
    protected fun closeView() {
        post(Runnable {
            if (adInfo.isDismiss) {
                return@Runnable
            }
            onCloseAd()
            onAdClose()
            adInfo.isDismiss = true //控制是否停止倒计时
            Jzvd.releaseAllVideos()//关闭播放视频
            if (mContext != null) { //防止内存泄露
                mContext = null
            }
        })
    }

    abstract inner class AbstractAdView @SuppressLint("ClickableViewAccessibility") protected constructor(
        context: Context
    ) : FrameLayout(context), View.OnClickListener {
        protected var screenSizes: IntArray = getScreenSize(true)
        var innerView: IAdInnerView?
        private var closeArea: RectF? = null

        private var clickArea: RectF? = null


        init {
            setOnTouchListener(null)
            setOnClickListener(null)
            visibility = GONE
            val mViewSize = viewSize
            innerView =
                mViewSize?.get(0)?.let { mAdController?.getAdInnerView(context, it, mViewSize[1]) }
            innerView?.setOnDrawListener(object : AdInnerViewOnDrawListener {
                override fun onAdDraw(canvas: Canvas?) {
                    this@AbstractAdView.drawView(canvas)
                }
            })
            this.addView(
                innerView?.view, LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT)
            )
        }

        val isAttachToWindow: Boolean
            get() = adInfo.mAttach

        /**
         * 画view
         */
        protected abstract fun drawView(canvas: Canvas?)


        /**
         * 获取广告的宽高
         *
         * @return 广告的宽高
         */
        abstract val viewSize: IntArray?


        final override fun setOnClickListener(l: OnClickListener?) {
            super.setOnClickListener(this)
        }

        override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
            var mWidthMeasureSpec = widthMeasureSpec
            var mHeightMeasureSpec = heightMeasureSpec
            var width = 0
            var height = 0
            val viewSize = viewSize

            /**
             * 设置宽度
             */
            var specMode = MeasureSpec.getMode(widthMeasureSpec)
            var specSize = MeasureSpec.getSize(widthMeasureSpec)
            if (specMode == MeasureSpec.EXACTLY) {
                width = paddingLeft + paddingRight + specSize
            } else {
                width = paddingLeft + paddingRight + (viewSize?.get(0) ?: 0)
                mWidthMeasureSpec = MeasureSpec.makeMeasureSpec(width, MeasureSpec.EXACTLY)
            }

            /**
             * 设置高度
             */
            specMode = MeasureSpec.getMode(heightMeasureSpec)
            specSize = MeasureSpec.getSize(heightMeasureSpec)
            if (specMode == MeasureSpec.EXACTLY) {
                height = paddingTop + paddingBottom + specSize
            } else {
                height = paddingTop + paddingBottom + (viewSize?.get(1) ?: 0)
                mHeightMeasureSpec = MeasureSpec.makeMeasureSpec(height, MeasureSpec.EXACTLY)
            }
            setMeasuredDimension(width, height)
            ad?.show_w = width
            ad?.show_h = height
            super.onMeasure(mWidthMeasureSpec, mHeightMeasureSpec)
        }


        @SuppressLint("ClickableViewAccessibility")
        final override fun setOnTouchListener(l: OnTouchListener?) {
            super.setOnTouchListener(object : OnTouchListener {
                override fun onTouch(v: View, event: MotionEvent): Boolean {
                    when (event.action) {
                        MotionEvent.ACTION_DOWN -> {
                            ad?.down_x = event.x
                            ad?.down_y = event.y
                        }

                        MotionEvent.ACTION_UP -> {
                            ad?.up_x = event.x
                            ad?.up_y = event.y
                            if (ad?.opted == 2) {
                                ad?.opted = 1
                                mAdController?.clickAd(ad)
                                return true
                            }
                        }

                        else -> {}
                    }
                    return false
                }
            })
        }

        /**
         * 渲染广告
         *
         * @param ad 广告对象
         */
        fun renderAdView(ad: AdBean?): Boolean {
            try {
                return innerView?.renderAdView(ad) == true
            } catch (e: Exception) {
                e.printStackTrace()
                return false
            }
        }

        /**
         * 当View被添加到父容器中时系统回调该方法
         */
        override fun onAttachedToWindow() {
            adInfo.mAttach = true
            super.onAttachedToWindow()
        }

        /**
         * 当View从其父布局中移除时系统回调该方法
         */
        override fun onDetachedFromWindow() {
            super.onDetachedFromWindow()
            d("AdType:" + ad?.ad_type + "::AdClosed")
            adInfo.mAttach = false
        }

        /**
         * 展示广告
         */
        fun showAdView(): Boolean {
            if (adInfo.isDismiss || !adInfo.isReady || innerView?.showView() != true) {
                d("AdType:" + ad?.ad_type + "::AdShowFailed:isReady" + adInfo.isReady)
                return false
            }
            try {
                startAnimation(displayAnim) //设置动画
                mAdController?.reportTracker(ad, TrackerType.DISPLAY) //发送展示监播 针对banner 开屏 插屏展示上报
                onAdViewShow() //展示广告
                d("AdType:" + ad?.ad_type + "::AdShowSuccess")
                return true
            } catch (e: Exception) {
                e.printStackTrace()
            }
            return false
        }

        /**
         * 显示广告
         */
        fun showCloseBtn() {
            if (adInfo.isCloseBtnVisible) {
                innerView?.view?.postInvalidate()
            }
        }


        override fun onClick(v: View) {
            if (isFastDoubleClick) { //忽略500毫秒之内的第二次点击
                return
            }
            if (SDKInfo.isMonkeyTest) {
                closeView()
                return
            }

            if (!isViewCovered(getAdView<AbstractAdView>(), 0.5f)) {
                mAdController?.reportTracker(ad, TrackerType.DISPLAY) //发送展示监播
            }

            ad?.interaction_type?.let { onClickAd(it) }
            if (ad?.ad_type == 1) { //当广告类型为banner时 点击广告跳转
                mAdController?.clickAd(ad) //控制背景广告点击跳转
            }
        }

        protected abstract fun onClickAd(type: Int)

        /**
         * 智能适配尺寸
         */
        protected fun getSmartSize(size: Int): Int {
            return ((screenSizes[0] / 640.0) * size).toInt()
        }

        /**
         * 设置是否显示关闭按钮
         */
        fun setCloseBtnVisible(visible: Boolean) {
            adInfo.isCloseBtnVisible = visible
        }
    }

}
