package com.halomobi.ssp.framework.Abstract

import android.content.Context
import android.content.Intent
import android.text.TextUtils
import com.halomobi.ssp.base.common.ConfigPool
import com.halomobi.ssp.base.common.ConfigPool.appPackageName
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.base.common.ConstantPool.AdType
import com.halomobi.ssp.base.utils.LogUtils.d
import com.halomobi.ssp.base.utils.Utils.getScreenSize
import com.halomobi.ssp.base.utils.Utils.post
import com.halomobi.ssp.base.utils.Utils.preCache
import com.halomobi.ssp.bussiness.module_sdk.HlSdk.getProxy
import com.halomobi.ssp.bussiness.module_sdk.listener.RewardVideoListener
import com.halomobi.ssp.bussiness.module_sdk.normal.HmRewardVideoManager.Companion.instance
import com.halomobi.ssp.bussiness.module_sdk.normal.RewardVideoActivity
import com.halomobi.ssp.framework.controller.AdController
import com.halomobi.ssp.framework.interfaces.IAdControl
import com.halomobi.ssp.framework.listener.AdStateChangListener
import com.halomobi.ssp.model.request.bean.AdBean
import com.halomobi.ssp.model.request.bean.AdParameter

abstract class AbstractVideoAd protected constructor(
    context: Context?,
    adSlotId: String?,
    adType: AdType,
    listener: RewardVideoListener?
) :
    AdStateChangListener {
    private var mContext: Context? = null
    var mAdController: IAdControl? = null
    protected var mAdParameter: AdParameter? = null
    private var mAd: AdBean? = null
    protected var mRewardVideoListener: RewardVideoListener? = null


    init {
        if (!checkParams(context, adSlotId, listener)) {
            this.mRewardVideoListener = listener
            this.mContext = context
            mAdParameter = AdParameter(ConfigPool.mAppId, adSlotId, appPackageName, adType)
            mAdParameter?.adViewSize = getScreenSize(true)
            mAdController = AdController.getInstance(mContext)
        }
    }

    override fun onStart() {
    }

    override fun onAdError(errorMsg: String?, errorCode: Int) {
        mRewardVideoListener?.onError(errorCode, errorMsg)
        d("$TAG:errMsg : $errorMsg")
    }

    override fun onAdReach(adInfo: AdBean?) {
        this.mAd = adInfo
        if (instance != null) {
            instance?.mAd = mAd
            instance?.isReady = true
        }
        post(Runnable { mRewardVideoListener?.onSuccess() })
        if (mAd?.reward_video != null && mAd?.reward_video?.prefetch == true) {
            cacheVideo()
        } else {
            mRewardVideoListener?.onCacheFailed()
        }
    }

    private fun cacheVideo() {
        val proxy = getProxy(mContext, mAd?.reward_video?.videoUrl)
        var proxyUrl: String? = null
        if (proxy != null) {
            proxyUrl = proxy.getProxyUrl(mAd?.reward_video?.videoUrl)
        }
        //如果没有缓存,则进行缓存
        if (proxy != null && !proxy.isCached(mAd?.reward_video?.videoUrl)) {
            if (!preCache(proxyUrl)) {
                mRewardVideoListener?.onCacheFailed()
            }
        } else {
            //不进行缓存直接回调缓存成功
            mRewardVideoListener?.onCacheSuccess()
            mAd?.isCached = true
        }
        mAd?.reward_video?.videoUrl = proxyUrl
    }


    fun showAd() {
        if (mAd != null && instance != null && instance?.isShow != true) {
            val intent = Intent(mContext, RewardVideoActivity::class.java)
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            mContext?.applicationContext?.startActivity(intent)
            instance?.isShow = true
            instance?.isReady = false
        }
    }

    /**
     * 检查参数是否为空
     *
     * @param context  上下文
     * @param adSlot   广告位id
     * @param listener 监听器
     */
    private fun checkParams(
        context: Context?,
        adSlot: String?,
        listener: RewardVideoListener?
    ): Boolean {
        if (context == null || TextUtils.isEmpty(adSlot)) {
            listener?.onError(
                ConstantPool.EroType.PARAMS_ERO,
                "参数错误"
            )
            return true
        }
        return false
    }

    companion object {
        const val TAG: String = "RewardAd"
    }
}
