package com.halomobi.ssp.framework.controller

import android.annotation.SuppressLint
import android.app.Activity
import android.app.AlertDialog
import android.content.Context
import android.content.DialogInterface
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.Color
import android.os.Build
import android.text.TextUtils
import android.view.View
import android.widget.TextView
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.base.common.ConstantPool.TrackerType
import com.halomobi.ssp.base.utils.KeyUtil.getMD5
import com.halomobi.ssp.base.utils.LogUtils.d
import com.halomobi.ssp.base.utils.NetworkUtils
import com.halomobi.ssp.base.utils.NetworkUtils.networkType
import com.halomobi.ssp.base.utils.SystemUtils.loopWeb
import com.halomobi.ssp.base.utils.Utils.gotoSet
import com.halomobi.ssp.base.utils.Utils.showToastSafe
import com.halomobi.ssp.base.utils.Utils.sp
import com.halomobi.ssp.bussiness.module_download.DownLoadManager
import com.halomobi.ssp.bussiness.module_download.TDownloadInfoProcessor
import com.halomobi.ssp.bussiness.module_download.bean.DownloadItem
import com.halomobi.ssp.bussiness.module_download.listener.DownloadStateListener
import com.halomobi.ssp.bussiness.module_download.listener.TDownloadInfoListener
import com.halomobi.ssp.bussiness.module_report.track.ReportTrackerManager
import com.halomobi.ssp.bussiness.module_sdk.view.imageloader.ImageLoader.load
import com.halomobi.ssp.bussiness.module_sdk.view.web.BrowserView
import com.halomobi.ssp.bussiness.module_video.rewardvideo.Jzvd.goOnPlayOnPause
import com.halomobi.ssp.framework.interfaces.IAdControl
import com.halomobi.ssp.framework.listener.AdStateChangListener
import com.halomobi.ssp.framework.listener.ClickAdStateChangListener
import com.halomobi.ssp.framework.listener.TrackerTaskListener
import com.halomobi.ssp.model.request.adtask.RequestAdManager.Companion.instance
import com.halomobi.ssp.model.request.bean.AdBean
import com.halomobi.ssp.model.request.bean.AdParameter
import com.halomobi.ssp.model.request.config.SDKInfo
import com.ssp.sdk.R
import java.util.Random
import java.util.UUID


class AdController : IAdControl {
    private var trackerListener: TrackerTaskListener? = null

    var clickListener: ClickAdStateChangListener? = null

    var adBean: AdBean? = null


    @Synchronized
    override fun setTrackerTaskListener(trackerListener: TrackerTaskListener?) {
        this.trackerListener = trackerListener
    }

    override fun setClickAdListener(listener: ClickAdStateChangListener?) {
        clickListener = listener
    }

    /**
     * 请求广告
     */
    @Synchronized
    override fun requestAd(adParameter: AdParameter?, adStateListener: AdStateChangListener?) {
        if (instance != null) {
            instance?.requestAd(adParameter, adStateListener)
        }
    }

    /**
     * 关闭广告
     */
    override fun closeAd(adInfo: AdBean?) {
        reportTracker(adInfo, TrackerType.CLOSE) //发送关闭广告监播 针对banner 插屏和开屏
    }

    /**
     * 点击广告
     *
     * @param info 广告对象
     */
    override fun clickAd(adInfo: AdBean?) {
        clickAd(adInfo, null)
    }

    override fun clickAd(adInfo: AdBean?, listener: DownloadStateListener?) {
        if (SDKInfo.isMonkeyTest) {
            return
        }
        if (TextUtils.isEmpty(adInfo?.clkurl)) {
            adInfo?.clkurl = adInfo?.landing_url.toString()
        }

        val deepLink = adInfo?.dplk
        if (!TextUtils.isEmpty(deepLink) && adInfo?.interaction_type != 2) {//当有deeplink返回 且不是下载类广告
            if (gotoQuickApp(adInfo, deepLink, true)) return
        }
        if (networkType == NetworkUtils.NET_NONE) {
            d("AdType:" + adInfo?.ad_type + "::click no network")
            showToastSafe("无网络")
            return
        }
        when (adInfo?.interaction_type) {
            1 -> {
                openWebPage(adInfo, adInfo.clkurl)
            }

            2 -> {
                if (checkInstalled(adInfo, adInfo.bundle?.trim())) {
                    //已经安装过，调起应用，不继续执行下载任务
                    return
                }
                //启动下载任务
                executeDownload(adInfo, listener)
            }

            3 -> {
                reportTracker(adInfo, TrackerType.CLICK) //发送点击监播
                loopWeb(adInfo.clkurl) //跳转系统浏览器
            }

            4 -> {
                gotoQuickApp(adInfo, adInfo.clkurl, false)
            }

            else -> {}
        }
    }

    private fun showSettingDialog() {
        if (sp != null) {
            val isFirstNotification = sp?.getBoolean("isFirstNotification", false)
            if (isFirstNotification == true) {
                return
            }
            showTipsDialogDownload()
        }
    }

    @SuppressLint("SetTextI18n", "UseKtx", "UseCompatLoadingForDrawables")
    private fun showTipsDialogDownload() {
        val builder = AlertDialog.Builder(mContext, R.style.hl_dialog)
        val inflater = mContext?.layoutInflater
        val dialogLayout = inflater?.inflate(R.layout.hj_dialog_tips_download, null, false)
        val mTipTitle = dialogLayout?.findViewById<TextView>(R.id.tv_tip_title)
        mTipTitle?.text = "是否开启通知栏消息?"
        val mEnSure = dialogLayout?.findViewById<TextView>(R.id.tv_ensure)
        mEnSure?.text = "确定"
        mEnSure?.setTextColor(Color.parseColor("#FFFFFF"))
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            mEnSure?.background = mContext?.getDrawable(R.drawable.hj_corner_blue_dialog)
        }
        val mCancel = dialogLayout?.findViewById<TextView>(R.id.tv_cancel)
        mCancel?.text = "取消"
        mCancel?.setTextColor(Color.parseColor("#000000"))
        goOnPlayOnPause() //弹出是否关闭广告弹窗后暂停播放
        builder.setCancelable(false)
        builder.setView(dialogLayout)
        val alertDialog = builder.create()
        alertDialog.window?.setLayout(100, 200)
        alertDialog.show()
        mEnSure?.setOnClickListener({
            sp?.edit()?.putBoolean("isFirstNotification", true)?.apply()
            gotoSet()
            alertDialog.dismiss()
        })
        mCancel?.setOnClickListener({
            sp?.edit()?.putBoolean("isFirstNotification", true)?.apply()
            alertDialog.dismiss()
        })

    }

    private fun gotoQuickApp(ad: AdBean?, deepLink: String?, reportDeepLinkOpen: Boolean): Boolean {
        adBean = ad
        try {
            val deepLinkIntent = Intent.parseUri(deepLink, Intent.URI_INTENT_SCHEME)
            deepLinkIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            val resolveInfo = mContext?.packageManager?.resolveActivity(
                deepLinkIntent,
                PackageManager.MATCH_DEFAULT_ONLY
            )
            if (resolveInfo != null) {
                openMediaDialog(mContext, deepLinkIntent)
                if (clickListener != null) {
                    clickListener?.onClickAdStateChang(ClickAdStateChangListener.STATE_WEB_LOAD_DEEPLINK)
                }
                if (reportDeepLinkOpen) {
                    reportTracker(ad, TrackerType.CLICK)
                    reportTracker(ad, TrackerType.DEEP_LINK_OPEN) //发送deepLink激活监播
                }
                return true
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return false
    }

    //打开deeplink的activity
    private fun openMediaDialog(activity: Activity?, intent: Intent) {
        activity?.startActivityForResult(intent, 1)
    }


    /**
     * 点击广告后使用WebView打开页面
     *
     * @param info      广告对象
     * @param click_url 跳转地址
     */
    private fun openWebPage(info: AdBean, click_url: String) {
        adBean = info
        val intent = Intent(mContext, BrowserView::class.java)
        intent.putExtra("click_url", click_url)
        mContext?.startActivity(intent)
    }

    /**
     * deeplink点击广告弹出弹框提示跳转app时,点击拒绝后使用WebView打开页面
     */
    fun openWebPage() {
        val intent = Intent(mContext, BrowserView::class.java)
        intent.putExtra("click_url", adBean?.clkurl)
        mContext?.startActivity(intent)
    }

    /**
     * 下载类广告点击后启动下载任务
     *
     * @param info 广告对象
     */
    private fun executeDownload(info: AdBean, listener: DownloadStateListener?) {
        if (networkType == NetworkUtils.NET_WIFI || (sp != null && sp?.getBoolean(
                ConstantPool.SWICH_DWLCONFIRM,
                true
            ) == false)
        ) {
            //如果是wifi 或者 不需要提示二次确认
            downLoadApp(info, listener)
            return
        }

        //非Wifi且需要提示用户
        val alertDialog = AlertDialog.Builder(mContext).setTitle("提示")
            .setMessage("确认使用移动网络进行下载？")
            .setNegativeButton(
                "取消"
            ) { dialog: DialogInterface?, which: Int ->
                onClickAdStateChang(
                    ClickAdStateChangListener.STATE_OPTION_DIALOG_CANCEL
                )
            }
            .setPositiveButton(
                "确认"
            ) { dialog: DialogInterface?, which: Int ->
                onClickAdStateChang(ClickAdStateChangListener.STATE_OPTION_DIALOG_CONFIRM)
                downLoadApp(info, listener) //启动下载
            }.setOnCancelListener { dialog: DialogInterface? ->
                onClickAdStateChang(
                    ClickAdStateChangListener.STATE_OPTION_DIALOG_CANCEL
                )
            }.create()

        if (alertDialog == null) {
            return
        }

        if (!alertDialog.isShowing && !(mContext as Activity).isFinishing) {
            alertDialog.show()
            onClickAdStateChang(ClickAdStateChangListener.STATE_OPTION_DIALOG_OPEN)
        }
    }

    /**
     * 检测是否已经安装了该应用，如果已安装，打开该应用
     *
     * @param packageName 包名
     * @return true 已安装  false 未安装
     */
    private fun checkInstalled(ad: AdBean?, packageName: String?): Boolean {

        if (!DownLoadManager.getInstance(mContext).isAppInstalled(packageName)) { //检测是否已经安装
            //未安装
            return false
        }
        ad?.clkurl = ad?.landing_url.toString()
        adBean = ad
        //尝试调起该应用
        DownLoadManager.getInstance(mContext).openApp(packageName, -1)

        return true
    }

    /**
     * 创建下载任务
     *
     * @param adInfo 广告对象
     * @return 下载任务对象
     */
    private fun createDownloadItem(adInfo: AdBean, listener: DownloadStateListener?): DownloadItem {
        val item = DownloadItem()
        item.name = getMD5(adInfo.clkurl) //设置下载项目名称
        item.downLoadUrl = adInfo.clkurl.trim { it <= ' ' } //下载地址
        item.deepLink = adInfo.dplk
        item.setDownloadStartTrackerTracker(
            adInfo.getTracks(TrackerType.S_DOWNLOAD).toTypedArray().contentToString()
        )
        item.downloadedTracker =
            adInfo.getTracks(TrackerType.DOWNLOADED).toTypedArray().contentToString()
        item.installStartTracker =
            adInfo.getTracks(TrackerType.INSTALL_START).toTypedArray().contentToString()
        item.installedTracker =
            adInfo.getTracks(TrackerType.INSTALLED).toTypedArray().contentToString()
        item.openTracker =
            adInfo.getTracks(TrackerType.OPEN).toTypedArray().contentToString()
        if (!TextUtils.isEmpty(adInfo.dplk)) { //deepLink 不为空，deepLink监播才有值
            item.deepLinkTracker =
                adInfo.getTracks(TrackerType.DEEP_LINK_OPEN).toTypedArray()
                    .contentToString()
        }
        if (adInfo.ctop == 4L) {
            //替换点击位置
            item.clickTracker =
                adInfo.getTracks(TrackerType.CLICK).toTypedArray().contentToString()
        }
        item.listener = object : DownloadStateListener {
            override fun onDownloadError(
                item: DownloadItem?,
                errorCode: Int,
                errorMsg: String?
            ) {
                onClickAdStateChang(ClickAdStateChangListener.STATE_APP_DOWNLOAD_FAILED)
            }

            override fun onDownloadStart(item: DownloadItem?, ad: AdBean?) {
                onClickAdStateChang(ClickAdStateChangListener.STATE_APP_DOWNLOAD_START)
                listener?.onDownloadStart(item, ad)
            }

            override fun onDownloaded(item: DownloadItem?, ad: AdBean?) {
                onClickAdStateChang(ClickAdStateChangListener.STATE_APP_DOWNLOAD_OK)

                listener?.onDownloaded(item, ad)
            }

            override fun onInstalled(item: DownloadItem?, ad: AdBean?) {
                onClickAdStateChang(ClickAdStateChangListener.STATE_APP_INSTALLED)

                listener?.onInstalled(item, ad)
            }

            override fun onOpened(item: DownloadItem?, ad: AdBean?) {
                onClickAdStateChang(ClickAdStateChangListener.STATE_APP_OPEN)
            }
        }

        return item
    }

    /**
     * 回调点击状态
     *
     * @param clickAdState 当前状态
     */
    private fun onClickAdStateChang(clickAdState: Int) {
        if (clickListener == null) {
            return
        }
        clickListener?.onClickAdStateChang(clickAdState)
    }

    /**
     * 准备开始下载任务
     *
     * @param info 广告对象
     */
    private fun downLoadApp(info: AdBean, listener: DownloadStateListener?) {
        reportTracker(info, TrackerType.CLICK)
        if (info.isTencent) {
            info.tencentDownloadUrl
            val processor = TDownloadInfoProcessor(info.clkurl, object : TDownloadInfoListener {
                override fun onDownloadError(errorCode: Int) {
                }

                override fun onDownloadSuccess(dstlink: String?, clickid: String?) {
                    info.clkurl = dstlink.toString()
                    info.tencentClickId = clickid.toString()
                    DownLoadManager.getInstance(mContext)
                        .downloadWithShow(createDownloadItem(info, listener), info)
                }
            })
            processor.process()
        } else {
            DownLoadManager.getInstance(mContext)
                .downloadWithShow(createDownloadItem(info, listener), info)
        }
    }

    /**
     * 发送监播
     *
     * @param adInfo 广告对象
     * @param type 监播类型
     */
    @Synchronized
    override fun reportTracker(adInfo: AdBean?, type: Int) {
        if (adInfo == null) {
            return
        }

        when (type) {
            TrackerType.DISPLAY -> {
                //banner轮播不曝光
                if (adInfo.ad_type == ConstantPool.AdType.BANNER.type && isBannerLoop) {
                    if (trackerListener != null) trackerListener?.onTrackersReturn()
                    return
                }
                isBannerLoop = true
                if (adInfo.isSubDisTrk) {
                    return
                }
                adInfo.isSubDisTrk = true
                adInfo.show_time = System.currentTimeMillis()
            }

            TrackerType.CLICK -> {
                if (adInfo.isSubmitClkTrk) {
                    return
                }
                adInfo.isSubmitClkTrk = true
                adInfo.click_time = System.currentTimeMillis()
            }

            TrackerType.DEEP_LINK_OPEN -> {
                if (adInfo.isSubmitDeepLinkTrk) {
                    return
                }
                adInfo.isSubmitDeepLinkTrk = true
            }

            else -> {}
        }


        ReportTrackerManager.instance?.reportTrack(type, trackerListener, adInfo.getTracks(type))
    }

    override fun loadImage(imageView: View?, string: String?) {
        load(imageView, string)
    }

    override fun getAdInnerView(context: Context?, w: Int, h: Int): AdInnerView? {
        return context?.let { AdInnerView(it, w, h) }
    }

    companion object {
        private var isBannerLoop = false

        @SuppressLint("StaticFieldLeak")
        private var mContext: Activity? = null

        @SuppressLint("StaticFieldLeak")
        private var adController: AdController? = null


        @JvmStatic
        @Synchronized
        fun getInstance(context: Context?): AdController? {
            mContext = context as Activity?
            isBannerLoop = false
            if (adController == null) {
                adController = AdController()
            }
            return adController
        }
    }
}
