package com.halomobi.ssp.framework.controller

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.TextView
import androidx.core.graphics.toColorInt
import com.halomobi.ssp.base.common.ConfigPool
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.base.utils.FileUtils.close
import com.halomobi.ssp.base.utils.FileUtils.getByteFromStream
import com.halomobi.ssp.bussiness.module_sdk.view.gif.GifView
import com.halomobi.ssp.bussiness.module_sdk.view.imageloader.ImageLoader.adapterData
import com.halomobi.ssp.bussiness.module_sdk.view.imageloader.ImageLoader.getByteArray
import com.halomobi.ssp.bussiness.module_sdk.view.imageloader.ImageLoader.load
import com.halomobi.ssp.bussiness.module_video.otherads.cn.jzvd.JzvdStd
import com.halomobi.ssp.framework.interfaces.IAdInnerView
import com.halomobi.ssp.framework.listener.AdInnerViewOnDrawListener
import com.halomobi.ssp.model.request.bean.AdBean
import com.halomobi.ssp.model.request.bean.AdBean.Mix
import com.halomobi.ssp.model.request.config.SDKInfo
import com.ssp.sdk.R
import java.io.IOException
import java.io.InputStream
import java.util.LinkedList
import java.util.Queue


@SuppressLint("ViewConstructor")
class AdInnerView(
    mContext: Context, private val adWith: Int, private val adHeight: Int
) : FrameLayout(
    mContext
), IAdInnerView {
    private val mPaint = Paint()

    private var imageView: GifView? = null


    private val mixView: MixView? = null

    private var mixInnerView: InnerMixView? = null
    private var mixInnerViewFull: InnerMixViewFull? = null

    private var onDrawListener: AdInnerViewOnDrawListener? = null

    private var ad: AdBean? = null

    init {
        mPaint.isAntiAlias = true
        setBackgroundColor("#00ff0000".toColorInt())
    }

    override fun setOnDrawListener(adInnerViewOnDrawListener: AdInnerViewOnDrawListener?) {
        this.onDrawListener = adInnerViewOnDrawListener
    }

    /**
     * 渲染广告
     *
     * @param ad 广告信息
     */
    override fun renderAdView(ad: AdBean?): Boolean {
        if (ad == null) {
            return false
        }

        this.ad = ad
        when (ad.creative_type) {
            AdBean.CRE_TYPE_IMAGE -> {
                val imgUrl = ad.imgurl
                ad.img = getByteArray(imgUrl.toString())
                return ad.img != null && ad.img?.size?.compareTo(0) == 1
            }

            AdBean.CRE_TYPE_MIX -> {
                var `is`: InputStream? = null
                val mix = ad.mix
                try {
                    mix?.icon = getByteArray(mix?.imgurl.toString())
                    if (TextUtils.isEmpty(mix?.acimgurl)) {
                        `is` = context.assets.open(
                            String.format(
                                SDKInfo.ADTAG + "/ac_%s.png",
                                if (ad.interaction_type >= 3) 1 else ad.interaction_type
                            )
                        )
                        mix?.acicon = getByteFromStream(`is`)
                    } else {
                        mix?.acicon = getByteArray(mix?.imgurl.toString())
                    }
                } catch (e: IOException) {
                    e.printStackTrace()
                } finally {
                    close(`is`)
                }
                return mix?.icon != null && mix.icon?.size?.compareTo(0) == 1 && !TextUtils.isEmpty(
                    mix.title
                )
            }

            AdBean.CRE_TYPE_VIDEO -> {
                val videoUrl = ad.reward_video?.videoUrl
                return videoUrl != null
            }

            else -> {}
        }
        return false
    }

    /**
     * 展示广告
     */
    override fun showView(): Boolean {
        val isShow: Boolean
        val parent = this.parent as ViewGroup
        val creative_type = ad?.creative_type
        when (creative_type) {
            AdBean.CRE_TYPE_IMAGE -> {
                if (imageView == null) {
                    imageView = GifView(context)
                    imageView?.scaleType = ImageView.ScaleType.FIT_XY
                    parent.addView(
                        imageView,
                        0,
                        LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT)
                    )
                }
                val imageByte = ad?.img
                val bitmap = adapterData(imageView, imageByte)
                if (bitmap == null) {
                    try {
                        imageView?.free()
                        imageView?.setImageDrawable(null)
                        imageView?.setGifImageType(GifView.GifImageType.WAIT_FINISH)
                        imageView?.setShowDimension(adWith, adHeight)
                        imageView?.setGifImage(imageByte)
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                } else {
                    imageView?.free()
                    imageView?.setImageBitmap(bitmap)
                }
                ad?.img = null
                isShow = imageByte != null
            }

            AdBean.CRE_TYPE_MIX -> {
                if (ConfigPool.AdShowTypeName == ConstantPool.AdType.INSERTFULLSCREEN.adName) {
                    if (mixInnerViewFull == null) {
                        mixInnerViewFull = InnerMixViewFull(context)
                    }
                    parent.addView(
                        mixInnerViewFull,
                        0,
                        LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT)
                    )
                    isShow = ad?.mix?.let { mixInnerViewFull?.showMix(it) } == true
                } else {
                    if (mixInnerView == null) {
                        mixInnerView = InnerMixView(context)
                    }
                    parent.addView(
                        mixInnerView,
                        0,
                        LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT)
                    )
                    isShow = ad?.mix?.let { mixInnerView?.showMix(it) } == true
                }
            }

            AdBean.CRE_TYPE_VIDEO -> {
                val jzvdStd = JzvdStd(context, adWith, adHeight)
                if ((ConfigPool.AdShowTypeName == ConstantPool.AdType.BANNER.adName)) {
                    if (checkFieldInView(parent.getChildAt(0), "JzvdStd")) {
                        parent.removeView(parent.getChildAt(0))
                    }
                }
                parent.addView(jzvdStd, 0, LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT))
                jzvdStd.setUp(
                   ad?.reward_video?.videoUrl,
                    ""
                )
                jzvdStd.setAdBean(ad)
                jzvdStd.startVideo()
                isShow = true
            }

            else -> return false
        }

        if (imageView != null) {
            imageView?.visibility =
                if (creative_type == AdBean.CRE_TYPE_IMAGE) VISIBLE else INVISIBLE
        }
        if (mixInnerView != null) {
            mixInnerView?.visibility =
                if (creative_type == AdBean.CRE_TYPE_MIX) VISIBLE else INVISIBLE
        }
        if (mixInnerViewFull != null) {
            mixInnerViewFull?.visibility =
                if (creative_type == AdBean.CRE_TYPE_MIX) VISIBLE else INVISIBLE
        }
        parent.visibility = VISIBLE
        return isShow
    }


    private fun checkFieldInView(view: View, fieldName: String): Boolean {
        // 这里需要根据实际情况实现如何检查字段，例如通过反射或者特定的方法
        // 例如，如果字段是一个自定义属性，你可以这样检查：
        return view.toString().contains(fieldName)
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        if (imageView != null) {
            imageView?.free()
        }
    }


    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)
        if (onDrawListener != null) {
            onDrawListener?.onAdDraw(canvas)
        }
    }

    override val view: View
        get() = this

    private fun getSize(d: Double): Int {
        return (adWith * 0.14 * d).toInt()
    }

    inner class InnerMixView @SuppressLint("UseCompatLoadingForDrawables") constructor(context: Context) :
        RelativeLayout(context) {
        private var imageView: ImageView? = null
        private var title: TextView? = null
        private var subTitle: TextView? = null

        init {
            setBackgroundResource(R.drawable.hj_corner_white)
            setPadding(
                dip2px(context, 5f),
                dip2px(context, 5f),
                dip2px(context, 5f),
                dip2px(context, 5f)
            )
            initInner(context)
        }

        private fun initInner(context: Context) {
            imageView = GifView(getContext())
            val iconId = imageView.hashCode()
            imageView?.id = iconId
            imageView?.scaleType = ImageView.ScaleType.FIT_XY
            val icon_Params =
                LayoutParams(
                    LayoutParams.MATCH_PARENT, adHeight.times(0.7).toInt()
                )

            addView(imageView, icon_Params)
            title = TextView(getContext())
            title?.id = title.hashCode()
            title?.includeFontPadding = false
            title?.setSingleLine()
            title?.setTextColor(Color.BLACK)
            title?.gravity = Gravity.CENTER_VERTICAL
            title?.setTextSize(TypedValue.COMPLEX_UNIT_SP, 15f)
            val title_Params = LayoutParams(
                LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT
            )
            title_Params.addRule(BELOW, iconId)
            title_Params.topMargin = dip2px(context, 2f)
            addView(title, title_Params)

            subTitle = TextView(getContext())
            subTitle?.id = subTitle.hashCode()
            subTitle?.includeFontPadding = false
            subTitle?.setSingleLine()
            subTitle?.setTextColor(Color.BLACK)
            subTitle?.gravity = Gravity.CENTER_VERTICAL
            subTitle?.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13f)
            val subTitle_Params = LayoutParams(
                LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT
            )
            subTitle_Params.addRule(BELOW, title.hashCode())
            subTitle_Params.topMargin = dip2px(context, 2f)
            addView(subTitle, subTitle_Params)
        }

        fun showMix(mix: Mix): Boolean {
            try {
                title?.text = mix.title
                subTitle?.text = mix.sub_title
                if (!TextUtils.isEmpty(mix.imgurl)) {
                    load(imageView, mix.imgurl)
                }
                return true
            } catch (e: Exception) {
                e.printStackTrace()
            }
            return false
        }
    }

    inner class InnerMixViewFull @SuppressLint("UseCompatLoadingForDrawables") constructor(context: Context) :
        FrameLayout(context) {
        private var imageView: ImageView? = null
        private var title: TextView? = null
        private var subTitle: TextView? = null

        init {
            setBackgroundResource(R.drawable.hj_corner_white)
            initInner(context)
        }

        private fun initInner(context: Context) {
            imageView = GifView(getContext())
            val iconId = imageView.hashCode()
            imageView?.id = iconId
            imageView?.scaleType = ImageView.ScaleType.FIT_XY
            val icon_Params =
                LayoutParams(
                    LayoutParams.MATCH_PARENT, adHeight
                )
            addView(imageView, icon_Params)
            title = TextView(getContext())
            title?.id = title.hashCode()
            title?.includeFontPadding = false
            title?.setSingleLine()
            title?.setTextColor(Color.WHITE)
            title?.gravity = Gravity.CENTER_VERTICAL
            title?.setTextSize(TypedValue.COMPLEX_UNIT_SP, 15f)
            val title_Params = LayoutParams(
                LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT
            )
            title_Params.topMargin = dip2px(context, 2f)
            addView(title, title_Params)


            subTitle = TextView(getContext())
            subTitle?.id = subTitle.hashCode()
            subTitle?.includeFontPadding = false
            subTitle?.setSingleLine()
            subTitle?.setTextColor(Color.WHITE)
            subTitle?.gravity = Gravity.CENTER_VERTICAL
            subTitle?.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13f)
            val subTitle_Params = LayoutParams(
                LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT
            )
            subTitle_Params.topMargin = dip2px(context, 20f)
            addView(subTitle, subTitle_Params)
        }


        fun showMix(mix: Mix): Boolean {
            try {
                title?.text = mix.title
                subTitle?.text = mix.sub_title
                if (!TextUtils.isEmpty(mix.imgurl)) {
                    load(imageView, mix.imgurl)
                }
                return true
            } catch (e: Exception) {
                e.printStackTrace()
            }
            return false
        }
    }

    inner class MixView(context: Context?) : RelativeLayout(context) {
        private var icon: GifView? = null
        private var ac_icon: GifView? = null
        private var title: TextView? = null
        private var subtitle: TextView? = null


        init {
            init()
        }

        private fun init() {
            setBackgroundColor("#333333".toColorInt())
            initIcon()
            initAcIcon()
            initTitle()
            initSubTitle()
        }

        private fun initIcon() {
            icon = GifView(context)
            icon?.id = icon.hashCode()
            icon?.scaleType = ImageView.ScaleType.FIT_XY
            val rl_Params = LayoutParams(
                getSize(0.74), getSize(0.74)
            )
            rl_Params.leftMargin = getSize(0.2)
            rl_Params.addRule(CENTER_VERTICAL)
            icon?.layoutParams = rl_Params
            addView(icon, rl_Params)
        }

        private fun initAcIcon() {
            ac_icon = GifView(context)
            ac_icon?.id = ac_icon.hashCode()
            ac_icon?.scaleType = ImageView.ScaleType.FIT_XY
            val rl_Params = LayoutParams(getSize(0.54), getSize(0.44))
            rl_Params.addRule(ALIGN_PARENT_RIGHT)
            rl_Params.addRule(CENTER_VERTICAL)
            rl_Params.rightMargin = getSize(0.54)
            ac_icon?.layoutParams = rl_Params
            addView(ac_icon, rl_Params)
        }

        private fun initTitle() {
            title = TextView(context)
            title?.id = title.hashCode()
            title?.includeFontPadding = false
            title?.setSingleLine()
            title?.setTextColor("#ffffff".toColorInt())
            title?.setTextSize(TypedValue.COMPLEX_UNIT_PX, getSize(0.24).toFloat())
            val rl_Params = LayoutParams(
                -2, getSize(0.28)
            )
            icon?.id?.let { rl_Params.addRule(RIGHT_OF, it) }
            icon?.id?.let { rl_Params.addRule(ALIGN_TOP, it) }
            ac_icon?.id?.let { rl_Params.addRule(LEFT_OF, it) }
            rl_Params.leftMargin = getSize(0.18)
            rl_Params.rightMargin = getSize(0.4)
            title?.layoutParams = rl_Params
            addView(title, rl_Params)
        }

        private fun initSubTitle() {
            subtitle = TextView(context)
            subtitle?.id = subtitle.hashCode()
            subtitle?.includeFontPadding = false
            subtitle?.gravity = Gravity.CENTER_VERTICAL
            subtitle?.maxLines = 2
            subtitle?.text = "副标题"
            subtitle?.setTextSize(TypedValue.COMPLEX_UNIT_PX, getSize(0.16).toFloat())
            subtitle?.setTextColor("#80FFFFFF".toColorInt())
            subtitle?.setTextColor(Color.WHITE)
            val rl_Params = LayoutParams(-2, getSize(0.40))
            icon?.id?.let { rl_Params.addRule(RIGHT_OF, it) }
            ac_icon?.id?.let { rl_Params.addRule(LEFT_OF, it) }
            title?.id?.let { rl_Params.addRule(BELOW, it) }
            rl_Params.topMargin = getSize(0.1)
            rl_Params.leftMargin = getSize(0.18)
            rl_Params.rightMargin = getSize(0.4)
            subtitle?.layoutParams = rl_Params
            addView(subtitle, rl_Params)
        }


        override fun onDraw(canvas: Canvas) {
            super.onDraw(canvas)
            title?.invalidate()
            subtitle?.invalidate()
        }
    }

    companion object {
        fun dip2px(context: Context, dpValue: Float): Int {
            val scale = context.resources.displayMetrics.density
            return (dpValue * scale + 0.5f).toInt()
        }
    }
}