package com.halomobi.ssp.framework.interfaces

import android.content.Context
import android.view.View
import com.halomobi.ssp.bussiness.module_download.listener.DownloadStateListener
import com.halomobi.ssp.framework.listener.AdStateChangListener
import com.halomobi.ssp.framework.listener.ClickAdStateChangListener
import com.halomobi.ssp.framework.listener.TrackerTaskListener
import com.halomobi.ssp.model.request.bean.AdBean
import com.halomobi.ssp.model.request.bean.AdParameter

interface IAdControl {
    /**
     * 请求广告
     *
     * @param adParameter     广告请求参数
     * @param adStateListener 广告状态回调对象
     */
    fun requestAd(adParameter: AdParameter?, adStateListener: AdStateChangListener?) //兼容浮层逻辑

    /**
     * 上报监播
     *
     * @param adInfo 广告对象
     * @param type   广告类型
     */
    fun reportTracker(adInfo: AdBean?, type: Int)

    /**
     * 关闭广告
     *
     * @param adInfo 广告对象
     */
    fun closeAd(adInfo: AdBean?)

    /**
     * 点击广告
     *
     * @param adInfo 广告对象
     */
    fun clickAd(adInfo: AdBean?)

    /**
     * 点击广告
     *
     * @param adInfo   广告对象
     * @param listener 下载任务状态回调
     */
    fun clickAd(adInfo: AdBean?, listener: DownloadStateListener?)

    /**
     * 设置监播回调
     *
     * @param trackerListener 监播回调对象
     */
    fun setTrackerTaskListener(trackerListener: TrackerTaskListener?)

    /**
     * 设置点击广告回调
     *
     * @param listener 点击回调对象
     */
    fun setClickAdListener(listener: ClickAdStateChangListener?)

    /**
     * 网络图片加载
     *
     * @param imageView 图片View
     * @param string    图片地址
     */
    fun loadImage(imageView: View?, string: String?)

    /**
     * 获取广告核心视图实例
     *
     * @param context 上下文
     * @param w       宽
     * @param h       高
     * @return 广告核心View
     */
    fun getAdInnerView(context: Context?, w: Int, h: Int): IAdInnerView?
}
