package com.halomobi.ssp.model.net.http

import android.text.TextUtils
import com.halomobi.ssp.base.utils.FileUtils.close
import com.halomobi.ssp.base.utils.Utils.decodeInputStream
import com.halomobi.ssp.base.utils.Utils.parseInputString
import java.io.InputStream
import java.net.HttpURLConnection


class HttpResult(connection: HttpURLConnection?) {
    val httpResponse: HttpURLConnection? = connection
    private var mIn: InputStream? = null
    private var mStr: String? = null
    val code: Int? = connection?.responseCode


    val string: String?
        /**
         * 从结果中获取字符串，一旦获取，会自动关流，并且把字符串保存，方便下次获取
         */
        get() {
            if (!TextUtils.isEmpty(mStr)) {
                return mStr
            }
            val inputStream = inputStream

            val contentEncoding = httpResponse?.contentEncoding
            try {
                mStr = parseInputString(
                    decodeInputStream(
                        inputStream,
                        contentEncoding.toString()
                    )
                )
                if (TextUtils.isEmpty(mStr)) {
                    mStr = httpResponse?.responseMessage
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            return mStr
        }

    val length: Long?
        /**
         * 获取文件长度
         */
        get() {
            if (code in 200..399) {
                return httpResponse?.contentLength?.toLong()
            }
            return 0
        }

    val inputStream: InputStream?
        /**
         * 获取流，需要使用完毕后调用close方法关闭网络连接
         */
        get() {
            if (mIn == null && code ?.compareTo( 400)==-1) {
                try {
                    mIn = httpResponse?.inputStream
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
            return mIn
        }

    /**
     * 关闭网络连接
     */
    fun close() {
        close(mIn)
        httpResponse?.disconnect()
    }

    companion object {
        private val httpCode3XXs = arrayOf(
            "多选项", "永久移动", "临时移动",
            "参见其他", "未改动", "使用代理", "", "暂时重定向"
        )
        private val httpCode4XXs = arrayOf(
            "错误请求",
            "未授权",
            "要求付费",
            "禁止",
            "未找到",
            "不允许的方法",
            "不被采纳",
            "要求代理授权",
            "请求超时",
            "冲突",
            "过期的",
            "要求的长度",
            "前提不成立",
            "请求实例太大",
            "请求URI太大",
            "不支持的媒体类型",
            "无法满足的请求范围",
            "失败的预期"
        )
        private val httpCode5XXs = arrayOf(
            "内部服务器错误", "未被使用", "网关错误",
            "不可用的服务", "网关超时", "HTTP版本未被支持"
        )

        /**
         * @param statusCode 状态码
         */
        fun getMsgByErrorCode(statusCode: Int): String {
            var resultMsg = ""
            if (statusCode in 300..399) {
                resultMsg = "重定向:$statusCode"
                val code = statusCode - 300
                if (code < httpCode3XXs.size) {
                    resultMsg += "----->" + httpCode3XXs[code]
                }
            }
            if (statusCode in 400..499) {
                resultMsg = "客户端错误:$statusCode"
                val code = statusCode - 400
                if (code < httpCode4XXs.size) {
                    resultMsg += "----->" + httpCode4XXs[code]
                }
            } else if (500 <= statusCode) {
                resultMsg = "服务器错误:$statusCode"
                val code = statusCode - 500
                if (code < httpCode5XXs.size) {
                    resultMsg += "----->" + httpCode5XXs[code]
                }
            }
            return resultMsg
        }
    }
}
