package com.halomobi.ssp.model.request.adtask

import android.media.MediaMetadataRetriever
import android.text.TextUtils
import android.util.Log
import android.util.SparseArray
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.base.utils.LogUtils.d
import com.halomobi.ssp.base.utils.NetworkUtils
import com.halomobi.ssp.base.utils.NetworkUtils.networkType
import com.halomobi.ssp.base.utils.Utils.post
import com.halomobi.ssp.base.utils.Utils.sp
import com.halomobi.ssp.framework.listener.AdStateChangListener
import com.halomobi.ssp.model.net.absnet.AbsNetProcessor
import com.halomobi.ssp.model.net.absnet.AbsNetTask
import com.halomobi.ssp.model.net.http.HttpResult
import com.halomobi.ssp.model.request.bean.AdBean
import com.halomobi.ssp.model.request.bean.AdBean.Mix
import com.halomobi.ssp.model.request.bean.AdBean.RewardVideoAds
import com.halomobi.ssp.model.request.bean.AdParameter
import org.json.JSONException
import org.json.JSONObject

class RequestAdProcessor internal constructor(
    private val mAdParameter: AdParameter?,
    private val mListener: AdStateChangListener?
) :
    AbsNetProcessor() {
    override fun onNetworkError() {
        onAdLoadFailed("network exception", ConstantPool.EroType.WITHOUT_NET)
    }

    override fun onErrorReceived(eroMsg: String?, eroCode: Int) {
        onAdLoadFailed(eroMsg, eroCode)
    }

    override fun onDataReceived(result: HttpResult?) {
        val adms = getBeanFromJson(result?.string.toString())
        if (adms==null) {
            //没有返回广告数据集合
            onAdLoadFailed("广告返回空集合", ConstantPool.EroType.BACK_EMPTY)
            return
        }
        val ad = adms[0]
        ad.response_time = System.currentTimeMillis()
        mAdParameter?.adType?.type?.let { ad.setAdType(it) }
        if (mListener == null) {
            return
        }

        if (ad.ad_type == ConstantPool.AdType.NATIVE.type || ad.ad_type == ConstantPool.AdType.UNIFIEDNATIVE.type) {
            //原生类型广告返回（集合）
            d("AdType:" + ad.ad_type + "::AdLoadSuccess")
            mListener.onNativeAdReach(adms)
            return
        }

//        if (ad.ad_type == ConstantPool.AdType.FLOATAD.type) {
//            //浮窗广告返回（集合）
//            d("AdType:" + ad.ad_type + "::AdLoadSuccess")
//            mListener.onFloatAdReach(adms)
//            return
//        }
        d("AdType:" + ad.ad_type + "::AdLoadSuccess")
        mListener.onAdReach(ad)
    }

    override fun createNetTask(): AbsNetTask {
        return RequestAdTask(mAdParameter)
    }

    private fun onAdLoadFailed(errorMsg: String?, errorCode: Int) {
        post(Runnable {
            d("AdType:" + mAdParameter?.adType + "::onAdLoadFailed errorMsg:" + errorMsg + "::errorCode" + errorCode)
            onAdError(errorMsg, errorCode)
        })
    }

    override fun process() {
        onStart()

        if (networkType == NetworkUtils.NET_NONE) {
            onAdLoadFailed("no network", ConstantPool.EroType.WITHOUT_NET)
            d("AdType:" + mAdParameter?.adType + "::AdLoadFailed:no network:" + ConstantPool.EroType.WITHOUT_NET)
            return
        }

        if (TextUtils.isEmpty(mAdParameter?.adSlotId)) {
            onAdLoadFailed("adSlotId is null", ConstantPool.EroType.PARAMS_ERO)
            d("AdType:" + mAdParameter?.adType + "::AdLoadFailed:adSlotId is null:" + ConstantPool.EroType.PARAMS_ERO)
            return
        }

        super.process()
        d("AdType:" + mAdParameter?.adType + "::AdLoadStart:app_id:" + mAdParameter?.appId + "::ad_slot_id:" + mAdParameter?.adSlotId)
    }

    /**
     * 广告请求失败
     *
     * @param errorMsg  错误信息
     * @param errorCode 错误码
     */
    fun onAdError(errorMsg: String?, errorCode: Int) {
        mListener?.onAdError(errorMsg, errorCode)
    }

    /**
     * 广告请求开始回调
     */
    private fun onStart() {
        mListener?.onStart()
    }

    /**
     * 解析json
     *
     * @param json json对象
     * @return 广告集合
     */
    private fun getBeanFromJson(json: String): ArrayList<AdBean>? {
        if (TextUtils.isEmpty(json)) {
            return null
        }
        val jsonObject = JSONObject(json)
        val error_code = jsonObject.optInt("errorcode")
        if (error_code != 0) {
            onAdLoadFailed("business_error_code:$error_code", error_code)
            return null
        }

        //parserSwitch(jsonObject);
        val adms = ArrayList<AdBean>()

        val jsonArray = jsonObject.getJSONArray("seatbid")
        val bidObj = jsonArray.optJSONObject(0)
        val bidArr = bidObj.optJSONArray("bid")
        if (bidArr != null) {
            for (i in 0 until bidArr.length()) {
                val `object` = bidArr.optJSONObject(i) ?: continue
                adms.add(parserAd(`object`))
            }
        }

        return adms
    }

    /**
     * 解析广告内容
     *
     */
    private fun parserAd(jsonAd: JSONObject): AdBean {
        val ad = AdBean()

        ad.id = jsonAd.optString("id")
        ad.impid = jsonAd.optString("impid")
        ad.imp_tracks = add2List(jsonAd, "imp_tracks")
        ad.dp_tracks = add2List(jsonAd, "dp_tracks")
        ad.click_tracks = add2List(jsonAd, "click_tracks")
        ad.down_start_tracks = add2List(jsonAd, "down_start_tracks")
        ad.down_complete_tracks = add2List(jsonAd, "down_complete_tracks")
        ad.install_start_tracks = add2List(jsonAd, "install_start_tracks")
        ad.install_complete_tracks = add2List(jsonAd, "install_complete_tracks")
        ad.bundle = jsonAd.optString("bundle")

        //默认图片类广告
        val imageObject = jsonAd.optJSONObject("image")
        if (imageObject != null) {
            ad.imgurl = imageObject.optString("url")
            ad.w = imageObject.optInt("w")
            ad.h = imageObject.optInt("h")
        }
        ad.title = jsonAd.optString("title")
        ad.des = jsonAd.optString("des")
        ad.is_download = jsonAd.optBoolean("is_download")
        ad.clkurl = jsonAd.optString("file_url")
        ad.landing_url = jsonAd.optString("landing_url")
        ad.price = jsonAd.optInt("price")
        Log.d("HaloSDKDemo:", "adType:" + ad.ad_type + "::price:" + ad.price)
        ad.is_deep = jsonAd.optBoolean("is_deep")
        ad.interaction_type = 1
        if (ad.is_deep) {
            ad.dplk = jsonAd.optString("deeplink")
        }
        ad.macro = jsonAd.optInt("macro")
        val source = jsonAd.optString("source")
        ad.source = source
        if (!TextUtils.isEmpty(source) && "gdt" != source) {
            ad.showAdTitle = true
        }

        //Mix广告(图文混合)
        val adType = sp?.getInt(ConstantPool.AD_TYPE, 2)
        if (adType == ConstantPool.AdType.INSERT.type || adType == ConstantPool.AdType.INSERTFULLSCREEN.type) { //插屏图文显示
            ad.creative_type = 3
            val mix = Mix()
            ad.mix = mix
            mix.title = ad.title
            mix.sub_title = ad.des
            mix.imgurl = ad.imgurl
        }

        ad.trackerGroup = SparseArray()
        if (ad.is_download) {
            ad.interaction_type = 2
            ad.trackerGroup?.put(
                ConstantPool.TrackerType.S_DOWNLOAD,
                add2List(jsonAd, "down_start_tracks")
            ) //开始下载
            ad.trackerGroup?.put(
                ConstantPool.TrackerType.DOWNLOADED,
                add2List(jsonAd, "down_complete_tracks")
            ) //下载完
            ad.trackerGroup?.put(
                ConstantPool.TrackerType.INSTALL_START,
                add2List(jsonAd, "install_start_tracks")
            ) //安装完成
            ad.trackerGroup?.put(
                ConstantPool.TrackerType.INSTALLED,
                add2List(jsonAd, "install_complete_tracks")
            ) //安装
        }

        //video
        val videoObject = jsonAd.optJSONObject("video")
        if (videoObject != null) {
            ad.creative_type = 4
            ad.reward_video = RewardVideoAds()
            ad.reward_video?.videoUrl = videoObject.optString("url")
            ad.reward_video?.title = ad.title
            ad.reward_video?.subTitle = ad.des
            val width = videoObject.optInt("w")
            val height = videoObject.optInt("h")
            ad.reward_video?.videoWidth = width.toString()
            ad.reward_video?.videoHeight = height.toString()
            ad.reward_video?.orientation = AdBean.SCREEN_ORIENTATION_PORTRAIT
            if (width > height) { //横屏
                ad.reward_video?.orientation = AdBean.SCREEN_ORIENTATION_LANDSCAPE
            }
            var durationStr = videoObject.optInt("duration").toString()
            if (TextUtils.isEmpty(durationStr) || "0" == durationStr) {
                durationStr = getPlayTime(ad.reward_video?.videoUrl)
            }
            ad.reward_video?.videoDuration = durationStr
        }
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.DISPLAY,
            add2List(jsonAd, "imp_tracks")
        ) //曝光追踪
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.CLICK,
            add2List(jsonAd, "click_tracks")
        ) //点击追踪
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.CLOSE,
            add2List(jsonAd, "ad_close_tracks")
        ) //点击关闭追踪
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.DEEP_LINK_OPEN,
            add2List(jsonAd, "dp_tracks")
        ) //Deeplink唤起
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.MUTE_TRACKER,
            add2List(jsonAd, "video_unmute_tracks")
        ) //取消静音
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.UNMUTE_TRACKER,
            add2List(jsonAd, "video_mute_tracks")
        ) //静音.
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.VIDEO_START_TRACKER,
            add2List(jsonAd, "video_start_tracks")
        ) //播放开始.
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.VIDEO_CLICK_TRACKER,
            add2List(jsonAd, "video_start_tracks")
        ) //播放点击.
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.VIDEO_PAUSE_TRACKER,
            add2List(jsonAd, "pauseTracker")
        ) //播放暂停.
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.VIDEO_COMPLETE_TRACKER,
            add2List(jsonAd, "video_complete_tracks")
        ) //播放完成.
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.VIDEO_SKIP_TRACKER,
            add2List(jsonAd, "video_skip_tracks")
        ) //视频跳过播放.
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.VIDEO_PLAYING_1_TRACKER,
            add2List(jsonAd, "video_play_first_quartile_tracks")
        ) //视频播放1/4.
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.VIDEO_PLAYING_2_TRACKER,
            add2List(jsonAd, "video_play_midpoint_tracks")
        ) //视频播放1/2.
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.VIDEO_PLAYING_3_TRACKER,
            add2List(jsonAd, "video_play_third_quartile_tracks")
        ) //视频播放3/4.
        ad.trackerGroup?.put(
            ConstantPool.TrackerType.VIDEO_PLAYING_3_TRACKER,
            add2List(jsonAd, "video_play_third_quartile_tracks")
        ) //视频播放3/4.
        return ad
    }

    private fun getPlayTime(mUri: String?): String {
        val mmr = MediaMetadataRetriever()
        try {
            if (mUri != null) {
                mmr.setDataSource(mUri)
            }
            val durationStr =
                mmr.extractMetadata(MediaMetadataRetriever.METADATA_KEY_DURATION) //时长(毫秒)
            if (!TextUtils.isEmpty(durationStr)) {
                return (durationStr?.toInt()?.div(1000)).toString()
            }
        } catch (_: Exception) {
        } finally {
            mmr.release()
        }
        return "0"
    }


    /**
     * 解析String泛型的list
     */
    private fun add2List(jsonObject: JSONObject, arrayKey: String): Array<String?>? {
        val dataArray: Array<String?>
        try {
            val jsonArray = jsonObject.optJSONArray(arrayKey)
            if (jsonArray == null || jsonArray.length() == 0) {
                return null
            }
            val length = jsonArray.length()
            dataArray = arrayOfNulls(length)
            for (i in 0 until length) {
                dataArray[i] = jsonArray.opt(i) as String
            }
            return dataArray
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
        }
        return null
    }
    /**
     * 解析String泛型的list
     */
    //    private String[] addToList(JSONObject jsonObject, String arrayKey, String arrayKey2) {
    //
    //        String[] dataArray;
    //        try {
    //            JSONArray jsonArray = jsonObject.getJSONArray(arrayKey);
    //            JSONArray jsonArray2 = jsonObject.getJSONArray(arrayKey2);
    //            if (jsonArray2 == null) {
    //                int length = jsonArray.length();
    //                dataArray = new String[length];
    //                for (int i = 0; i < length; i++) {
    //                    dataArray[i] = (String) jsonArray.opt(i);
    //                }
    //            } else {
    //                int length = jsonArray.length();
    //                int length2 = jsonArray2.length();
    //                dataArray = new String[length + length2];
    //                for (int i = 0; i < length; i++) {
    //                    dataArray[i] = (String) jsonArray.opt(i);
    //                }
    //                for (int i = 0; i < length2; i++) {
    //                    dataArray[i + length] = (String) jsonArray2.opt(i);
    //                }
    //            }
    //            return dataArray;
    //        } catch (JSONException e) {
    //            e.printStackTrace();
    //        }
    //        return null;
    //    }
    /**
     * 解析开关配置
     *
     * @param jsonObject 服务器返回数据
     * @throws JSONException
     */
    //    private void parserSwitch(JSONObject jsonObject) throws JSONException {
    //
    //        long ex_time = jsonObject.getLong("expiration_time");
    //        if (ex_time > 0) {
    //            EXPIRATION_TIME = ex_time * 1000;
    //        }
    //        long ad_interval = jsonObject.getLong("get_ad_in_same_view_interval");
    //        if (ad_interval > 0) {
    //            ConfigPool.BANNER_REQUEST_INTERVAL = ad_interval * 1000;
    //        }
    //
    //        //		读取自更新开关
    //        SharedPreferences.Editor edit = Utils.getSp().edit();
    //        try {
    //            edit.putBoolean(DEX_UPDATE_SWITCH, jsonObject.getInt(SWICH_UPDATE) == 1);
    //        } catch (Exception e) {
    //            LogUtils.d("无自更新开关字段返回");
    //        }
    //        //		读取自更新开关
    //        //		读取下载二次确认功能开关
    //        try {
    //            edit.putBoolean(SWICH_DWLCONFIRM, jsonObject.getInt(SWICH_DWLCONFIRM) == 1);
    //        } catch (Exception e) {
    //            LogUtils.d("无下载二次确认开关字段返回");
    //        }
    //        //		读取下载二次确认功能开关
    //
    //        // 		读取Logo开关
    //        try {
    //            edit.putBoolean(LOGO_SWITCH, jsonObject.getInt(LOGO_SWITCH) == 1);
    //            long ad_float_rotation_time = jsonObject.getLong("rotation_time");
    //            if (ad_float_rotation_time > 0) {
    //                ConfigPool.FLOAT_REQUEST_INTERVAL = ad_float_rotation_time * 1000;
    //            }
    //        } catch (Exception e) {
    //            LogUtils.d("无logo开关字段返回");
    //        }
    //        //		读取Logo开关
    //
    //        edit.commit();
    //    }
}
