package com.halomobi.ssp.model.request.bean

import android.util.SparseArray
import com.halomobi.ssp.base.common.ConfigPool
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.base.utils.Utils.sp
import java.io.UnsupportedEncodingException
import java.net.URLEncoder
import java.util.UUID

class AdBean {
    var id: String? = null
    var impid: String? = null
    var imp_tracks: Array<String?> ?= null
    var dp_tracks: Array<String?>? = null
    var click_tracks: Array<String?>? = null
    var down_start_tracks: Array<String?>?=null
    var down_complete_tracks: Array<String?>?=null
    var install_start_tracks: Array<String?>?=null
    var install_complete_tracks: Array<String?>?=null
    var title: String? = null
    var des: String? = null
    var showAdTitle: Boolean = false
    var is_download: Boolean = false
    var is_deep: Boolean = false
    var landing_url: String? = null
    var macro: Int = 0
    var source: String? = null
    var price: Int = 0

    val adUUID: String = UUID.randomUUID().toString()
    var adSlotId: String? = null
    var ad_type: Int = 0
    var opted: Int = 0 //1.点击关闭按钮可以关闭  2.点击关闭按钮不可以关闭
    var w: Int = 0
    var h: Int = 0
    var show_w: Int = 0 //广告实际展示宽度
    var show_h: Int = 0 //广告实际展示高度
    var video_progress: Int = 0
    var video_progress_ms: Int = 0
    var response_time: Long = 0
    var ready_time: Long = 0
    var show_time: Long = 0
    var click_time: Long = 0

    var creative_type: Int = 2 // 0：无创意类型，1：纯文字创意，2：纯图片创意，3:图文混合创意，4:H5创意/视频类广告 5:信息流广告
    var bundle: String? = null // 下载类广告，Android应用包名，例：”com.wk.demo”
    var imgurl: String? = null
    var mix: Mix? = null
    var dplk: String? = null //deepLink链接
    var clkurl: String = ""
    var interaction_type: Int = 2 // 0：无动作，1：使用浏览器打开网页，2：下载应用。此版本只支持使用浏览器打开网页
    var trackerGroup: SparseArray<Array<String?>>? = null
    var img: ByteArray?=null
    var reward_video: RewardVideoAds? = null
    var rqto: Long = ConfigPool.AD_REQUEST_TIME_OUT //请求超时时间
    var rdto: Long =  ConfigPool.AD_RENDER_TIME_OUT //渲染超时时间
    var ctop: Long = 1 //点击监播发送时机  1:开始加载落地页面时发送；2:落地页面加载完成时发送；3:点击就发送；4:开始下载时发送。

    var down_x: Float = 0f
    var down_y: Float = 0f
    var up_x: Float = 0f
    var up_y: Float = 0f
    var isSubDisTrk: Boolean = false //为了避免重复发送监播，用来记录是否已经发送过
    var isSubmitClkTrk: Boolean = false //为了避免重复发送监播，用来记录是否已经发送过
    var isSubmitDeepLinkTrk: Boolean = false //为了避免重复发送监播，用来记录是否已经发送过


    var isShow: Boolean = false //标记是否展示了
    var isCached: Boolean = false //标记视频是否已经回调缓存成功监听.

    var is25Report: Boolean = false // 激励视频播放进度25% 是否已经上报
    var is50Report: Boolean = false // 激励视频播放进度50% 是否已经上报
    var is75Report: Boolean = false // 激励视频播放进度75% 是否已经上报

    var isTencent: Boolean = false //腾讯来源
    var tencentClickId: String = "" //下载广告点击id


    fun getTracks(type: Int): Set<String?> {
        var mUserAgent: String? = ""
        try {
            mUserAgent = URLEncoder.encode(sp?.getString(ConstantPool.UA_KEY, ""), "utf-8")
        } catch (e1: UnsupportedEncodingException) {
            e1.printStackTrace()
        }
        try {
            val tracks = trackerGroup?.get(type)
            if (tracks.isNullOrEmpty()) {
                return HashSet()
            }

            val trackSet: MutableSet<String> = HashSet()
            for (track in tracks) {
                //点击监播上报点击位置
                trackSet.add(
                    track?.replace("\${down_x}", down_x.toInt().toString() + "")
                        ?.replace("\${down_y}", down_y.toInt().toString() + "")
                        ?.replace("\${up_x}", up_x.toInt().toString() + "")
                        ?.replace("\${up_y}", up_y.toInt().toString() + "")
                        ?.replace("\${User-Agent}", mUserAgent.toString())
                        ?.replace("__DOWN_X__", down_x.toInt().toString() + "")
                        ?.replace("__DOWN_Y__", down_y.toInt().toString() + "")
                        ?.replace("__UP_X__", up_x.toInt().toString() + "")
                        ?.replace("__UP_Y__", up_y.toInt().toString() + "")
                        ?.replace("__WIDTH__", show_w.toString() + "")
                        ?.replace("__HEIGHT__", show_h.toString() + "")
                        ?.replace("__CLICK_ID__", tencentClickId)
                        ?.replace("__PROGRESS__", video_progress.toString() + "")
                        ?.replace("__PROGRESS_MS__", video_progress_ms.toString() + "")
                        ?.replace("__READY_TIME__", ready_time.toString() + "")
                        ?.replace("__SHOW_TIME__", show_time.toString() + "")
                        ?.replace("__CLICK_TIME__", click_time.toString() + "")
                        ?.replace("__RESPONSE_TIME__", response_time.toString() + "")
                        ?.replace("__TS_S__", (System.currentTimeMillis() / 100).toString())
                        ?.replace("__TS__", System.currentTimeMillis().toString())
                        ?.replace("__PRICE__", price.toString()).toString()
                )
            }
            return trackSet
        } catch (e: Exception) {
            e.printStackTrace()
            return HashSet()
        }
    }

    /**
     * 腾讯下载开始、下载完成、安装监波宏替换
     *
     * @param tracks
     * @return
     */
    private fun setTencentDownLoadTracks(tracks: Array<String>): HashSet<String> {
        val hashSet = HashSet<String>()
        for (i in tracks.indices) {
            hashSet.add(
                tracks[i].replace("__DOWN_X__", down_x.toString() + "")
                    .replace("__DOWN_Y__", down_y.toString() + "")
                    .replace("__UP_X__", up_x.toString() + "")
                    .replace("__UP_Y__", up_y.toString() + "")
                    .replace("__WIDTH__", show_w.toString() + "")
                    .replace("__HEIGHT__", show_h.toString() + "")
                    .replace("__CLICK_ID__", tencentClickId)
            )
        }
        return hashSet
    }

    //    /**
    //     * 腾讯展示广告点击落地页宏替换
    //     */
    //    public void getTencentUrl() {
    //
    //        if (!isTencent) {
    //            return;
    //        }
    //        clkurl = clkurl.replace("__DOWN_X__", down_x + "").replace("__DOWN_Y__", down_y + "")
    //                .replace("__UP_X__", up_x + "").replace("__UP_Y__", up_y + "").replace("__WIDTH__", show_w + "").replace("__HEIGHT__", show_h + "");
    //    }
    val tencentDownloadUrl: Unit
        /**
         * 设置腾讯下载链接宏替换
         */
        get() {
            clkurl = clkurl.replace("__DOWN_X__", down_x.toString() + "")
                .replace("__DOWN_Y__", down_y.toString() + "")
                .replace("__UP_X__", up_x.toString() + "").replace("__UP_Y__", up_y.toString() + "")
                .replace("__WIDTH__", show_w.toString() + "")
                .replace("__HEIGHT__", show_h.toString() + "")
        }


    fun setAdType(ad_type: Int) {
        this.ad_type = ad_type
        setRqto(0) //根据ad_type设置默认广告请求超时时间
        setRdto(0) //根据ad_type设置默认广告渲染超时时间
    }

    fun setInteractionType(interaction_type: Int) {
        this.interaction_type = interaction_type
        setCtop(0) //根据相应类型不同设置默认点击监播发送时机
    }

     fun setRqto(value: Int) {
        rqto = if (value == 0) {
            when (ad_type) {
                1, 2, 3, 4, 5, 6 -> ConfigPool.AD_REQUEST_TIME_OUT
                else -> ConfigPool.AD_REQUEST_TIME_OUT
            }
        } else {
            value * 1000L
        }
    }

    fun setRdto(value: Int) {
        rdto = if (value == 0) {
            when (ad_type) {
                1, 2, 3, 4 -> ConfigPool.AD_RENDER_TIME_OUT
                else -> ConfigPool.AD_RENDER_TIME_OUT
            }
        } else {
            value * 1000L
        }
    }

    fun setCtop(value: Int) {
        if (value == 0) {
            when (interaction_type) {
                0, 1 -> ctop = 2
                2 -> ctop = 4
                3 -> ctop = 3
                else -> {}
            }
        } else {
            ctop = value.toLong()
        }
    }

    //    public static class Image {
    //        public String url;
    //    }
    //
    //    public static class Logo {
    //        public String url;
    //    }
    //
    //    public static class Native {
    //        public int type;
    //        public String title;
    //        public String sub_title;
    //        public String[] imgurl;
    //        public String source;
    //    }
    class Mix {
        var title: String? = null
        var sub_title: String? = null
        var imgurl: String? = null
        var acimgurl: String? = null
        var bg_color: String? = null // 背景色
        var text_color: String? = null // 文字颜色
        var icon: ByteArray?=null
        var acicon: ByteArray?=null
    }

    class RewardVideoAds {
        var videoUrl: String? = null
        var videoWidth: String? = null
        var videoHeight: String? = null
        var orientation: String? = null
        var videoDuration: String? = null
        var type: String? = null
        var prefetch: Boolean = false

        //1.12.0 新增
        var endCardType: String? = null
        var endCardHtml: String? = null
        var endCardUrl: String? = null
        var title: String? = null
        var subTitle: String? = null
        var icon: String? = null
    }
    //    public static class RewardVideoPlayTrackers {
    //        public String muteTracker;
    //        public String unmuteTracker;
    //        public String cacheSuccTracker;
    //        public String playErrorTracker;
    //        public String pauseTracker;
    //        public String closeTracker;
    //        public List<RewardVideoPlayCentageTracke> playCentageTrackers;
    //    }
    //    public static class RewardVideoPlayCentageTracke {
    //        public List<String> urls;
    //        public double checkpoint;
    //
    //    }


    companion object {
        const val CRE_TYPE_IMAGE: Int = 2
        const val CRE_TYPE_MIX: Int = 3
        const val CRE_TYPE_H5: Int = 4
        const val CRE_TYPE_VIDEO: Int = 4
        const val SCREEN_ORIENTATION_PORTRAIT: String = "1"
        const val SCREEN_ORIENTATION_LANDSCAPE: String = "2"
    }
}
