package com.halomobi.ssp.model.request.bean

import android.text.TextUtils
import com.halomobi.ssp.base.common.ConfigPool
import com.halomobi.ssp.base.common.ConfigPool.appPackageName
import com.halomobi.ssp.base.common.ConfigPool.appVersion
import com.halomobi.ssp.base.common.ConfigPool.deviceModel
import com.halomobi.ssp.base.common.ConfigPool.deviceType
import com.halomobi.ssp.base.common.ConfigPool.getAppName
import com.halomobi.ssp.base.common.ConfigPool.oSVersion
import com.halomobi.ssp.base.common.ConfigPool.uUID
import com.halomobi.ssp.base.common.ConfigPool.vendor
import com.halomobi.ssp.base.common.ConstantPool
import com.halomobi.ssp.base.utils.KeyUtil.encryptCurrentAES
import com.halomobi.ssp.base.utils.NetworkUtils.networkType
import com.halomobi.ssp.base.utils.SystemUtils
import com.halomobi.ssp.base.utils.SystemUtils.appStoreVersion
import com.halomobi.ssp.base.utils.SystemUtils.boot
import com.halomobi.ssp.base.utils.SystemUtils.formatCurrentData
import com.halomobi.ssp.base.utils.SystemUtils.getInfoByJson
import com.halomobi.ssp.base.utils.SystemUtils.operatorsName
import com.halomobi.ssp.base.utils.SystemUtils.update
import com.halomobi.ssp.base.utils.Utils.getScreenSize
import com.halomobi.ssp.base.utils.Utils.sp
import com.halomobi.ssp.model.request.config.SDKInfo
import org.json.JSONException
import org.json.JSONStringer
import kotlin.math.max

class AdRequest private constructor(p: AdParameter?) {
    /**
     * 请求ID
     */
    // private String request_id;
    /**
     * API版本
     */
    private val api_version = SDKInfo.API_VERSION

    /**
     * 应用信息
     */
    private val app =
        App(p?.appId, p?.pk)

    /**
     * 设备信息
     */
    private val device: HjDevice

    /**
     * 广告位信息
     */
    private val adSlot: AdSlot


    init {
        device = HjDevice()
        adSlot = AdSlot(p?.adSlotId, p?.adType?.type)
    }

    private class App(app_id: String?, pk: String?) {
        var app_id: String? = null
        val app_version: String?
        val app_name: String?
        var package_name: String? = null

        init {
            if (TextUtils.isEmpty(app_id)) {
                this.app_id = ConfigPool.mAppId
            } else {
                this.app_id = app_id
            }
            package_name = if (TextUtils.isEmpty(pk)) {
                appPackageName
            } else {
                pk
            }
            app_name = getAppName(package_name)
            app_version = appVersion
        }
    }

    class HjDevice {
        val userAgent: String? = sp?.getString(ConstantPool.UA_KEY, "")
        val imei: String = getInfoByJson("did")
        val oaId: String = getInfoByJson("oaId")
        val androidID: String = getInfoByJson("dpid")
        val mac: String = getInfoByJson("mac")
        val carrier: Int = operatorsName
        val systemUpdate: String = update
        val systemBoot: String = boot
        val latitude: Double = SystemUtils.latitude
        val longitude: Double = SystemUtils.longitude
        val storeVersion: String = appStoreVersion
        val make: String? = vendor
        val model: String? = deviceModel
        val os_type: Int = 2
        val os_version: String? = oSVersion
        val w: Int
        val h: Int
        val connect_type: Int = max(0.0, networkType.toDouble()).toInt()
        val device_type: Int = deviceType

        init {
            val size = getScreenSize(false)
            w = size[0]
            h = size[1]
        }
    }

    private class AdSlot(val slot_id: String?, val slot_type: Int?) {
        val app_id: String = ConfigPool.mAppId
    }

    companion object {
        @JvmStatic
        fun getParams(parameter: AdParameter?): ByteArray? {
            val jParam = AdRequest(parameter)
            return serializeJson(jParam)
        }

        private fun serializeJson(parameter: AdRequest): ByteArray? {
            val js = JSONStringer()
            try {
                js.`object`().key("id").value(uUID)
                val adSlot = parameter.adSlot
                js.key("imp")
                js.array().`object`().key("id").value(adSlot.app_id).key("slot_id").value(adSlot.slot_id).key("slot_type").value(adSlot.slot_type?.toLong()).endObject().endArray()
                val app = parameter.app
                js.key("app")
                js.`object`().key("id").value(app.app_id).key("name").value(app.app_name).key("ver").value(app.app_version).key("bundle").value(app.package_name).endObject()
                val device = parameter.device
                js.key("device")
                js.`object`().key("ua").value(device.userAgent).key("oaid").value(device.oaId).key("did").value(device.imei).key("dpid").value(device.androidID).key("mac").value(device.mac).key("carrier").value(device.carrier.toLong()).key("make").value(device.make).key("model").value(device.model).key("os").value(device.os_type.toLong()).key("osv").value(device.os_version).key("w").value(device.w.toLong()).key("h").value(device.h.toLong()).key("boot_mark").value(device.systemBoot).key("update_mark").value(device.systemUpdate).key("lat").value(device.latitude).key("lon").value(device.longitude).key("store_ver").value(device.storeVersion).key("connection_type").value(device.connect_type.toLong()).key("device_type").value(device.device_type.toLong()).endObject()
                js.key("support_dp").value(true).endObject()
            } catch (e: JSONException) {
                e.printStackTrace()
            }
            try {
                return encryptCurrentAES(formatCurrentData(), js.toString())
            } catch (e: Exception) {
                e.printStackTrace()
            }
            return null
        }
    }
}
